'####################################################################################################################
'*******************ArcGIS_SLD_Converter*****************************************************************************
'*******************Class: Analize_ArcMap_Symbols********************************************************************
'*******************AUTHOR: Albrecht Weiser, University of applied Sciences in Mainz, Germany 2005*******************
'The Application was part of my Diploma thesis:**********************************************************************
'"Transforming map-properties of maps in esri-data to an OGC-conformous SLD-document, for publishing the ArcGIS-map * 
'with an OGC- conformous map-server"*********************************************************************************
'ABSTRACT:
'The program so called "ArcGIS-map to SLD Converter" analyses an
'ArcMap-Project with respect to its symbolisation and assembles an SLD
'for the OGC-Web Map Service (WMS) from the gathered data. The program
'is started parallel to a running ArcMap 9.X-session. Subsequently the
'application deposits an SLD-file which complies the symbolisation of
'the available ArcMap-project. With the SLD a WMS-project may be
'classified and styled according to the preceding ArcMap-project. The
'application is written in VisualBasic.NET and uses the .NET 1.1
'Framework (plus XML files for configuration). For more informtion
'refer to: 
'http://arcmap2sld.geoinform.fh-mainz.de/ArcMap2SLDConverter_Eng.htm.
'LICENSE:
'This program is free software under the license of the GNU Lesser General Public License (LGPL) As published by the Free Software Foundation. 
'With the use and further development of this code you accept the terms of LGPL. For questions of the License refer to:
'http://www.gnu.org/licenses/lgpl.html
'DISCLAIMER:
'THE USE OF THE SOFTWARE ArcGIS-map to SLD Converter HAPPENS AT OWN RISK.
'I CANNOT ISSUE A GUARANTEE FOR ANY DISADVANTAGES (INCLUDING LOSS OF DATA; ETC.) THAT
'MAY ARISE FROM USING THIS SOFTWARE.
'DESCRIPTION:
'Analyze_ArcMap_Symbols is the central analyzing class of the application. It analyzes the ArcMap styling attributes
'and stores the data in structs. Each renderer has its own struct. Each symbol has its own struct. The renderer itself 
'stores the symbol structs. The class is triggered by the form-class "Motherform",button "Button1" pushes the analysis 
'by the event "Button1_Click". The analysis starts at the "Analyze_ArcMap_Symbols.CentralProcessingFunc()". When ended
'the analysis, the class  calls the class "Output_SLD" and passes the structs with the data to the class.
'CHANGES:
'01.02.2006: Bugfix: in function StoreLineFill()
'04.02.2006: Language-customizing english/german
'04.02.2006: Tooltips
'22.01.2007: Bugfix: in StorLineFill.ICartographicLineSymbol
'26.01.2007: The converter now supports Group Layers (They cannot be displayed because there's still no such structure 
'            in SLD, but the FeatureLayers inside the Group Layers are beeing stored. The the analysis of function 
'            AnalyseLayerSymbology was outsourced to the new function SpreadLayerStructure. There a recursive call rules
'            the Layer order. The routine AddOneToLayerNumber counts the Layernumbers up.
'27.03.2007: - Extract transparency based on the color.
'            - Extract dashed line settings for cartographic lines and simple lines.
'            - Simplified color handling by always having no color (empty string)
'              if the color is transparent or style indicates color is not used.
'            - Fixed several bugs where wrong object was used causing a crash, in methods
'              StoreLineFill, StoreDotDensityFill, StorePictureFill, StoreGradientFill, StoreBarChart, StorePieChart, StoreStackedChart
'28.03.2007: Added TextSymbolizer support for simple annotation with a single feature as label.
'25.04.2007: Allow conversion of only layers that are visible, ignore those not visible.
'            Using option from motherform menu to decide whether to convert all
'            layers or just selected layers.
'12.09.2007: Fix to handle cartographic lines with empty Template pattern.
'09.06.2008: Add support for grouped values in UniqueValueRenderer.
'10.09.2008: Some small code improvements
'####################################################################################################################

Imports ESRI.ArcGIS.Framework
Imports ESRI.ArcGIS.Framework.AppROTClass
Imports System.Runtime.InteropServices
Imports ESRI.ArcGIS.SystemUI
Imports ESRI.ArcGIS.ArcMapUI
Imports ESRI.ArcGIS.Carto
Imports ESRI.ArcGIS.Carto.FeatureLayerClass
Imports ESRI.ArcGIS.Display
Imports stdole
Imports ESRI.ArcGIS.Geodatabase
Imports ESRI.ArcGIS.esriSystem
Imports System.IO


Public Class Analize_ArcMap_Symbols

    '##################################################################################################
    '#################################### DEKLARATIONEN ###############################################
    '##################################################################################################

#Region "Membervariablen"
    Private m_ObjDoc As IMxDocument                 'Das augenblicklich aktive Dokument
    Private m_ObjApp As IApplication                'Die Schnittstelle zur ArcMap-Instanz
    Private m_ObjAppROT As IAppROT                  'Die Schnittstelle zum Erlangen der laufenden ArcMap-Instanz
    Private m_ObjObjectCreator As IObjectFactory    'Die Schnittstelle, um ArcMap-Objekte generieren zu knnen!!!
    Private m_ObjMap As IMap                        'Die Schnittstelle zum aktuellen Kartenobjekt
    Private frmMotherform As Motherform             'Die Instanz der aufrufenden Mutterklasse
    Friend m_StrProject As StructProject            'Die Instanz der Datenstruktur, die alle Layer enthlt
    Private m_al1, m_al2, m_al3 As ArrayList        'jede Arraylist enthlt die normalisierten Werte aus einer Tabellenspalte
    Private m_alClassifiedFields As ArrayList       'enthlt jeweils wiederum arraylist(s) von den normalisierten Werten aus den Feldern, nach denen klassifiziert wurde ( m_al1, m_al2, [m_al3])
    Private m_cFilename As String
#End Region

    '##################################################################################################
    '######################################## ENUMERATIONEN ###########################################
    '##################################################################################################

#Region "Enums"

    'Die FeatureClass wird hier festgelegt; (ob Punkt-, Linien-, oder Polygonfeature)
    Friend Enum FeatureClass
        PointFeature = 0
        LineFeature = 1
        PolygonFeature = 2
    End Enum

    Friend Enum MarkerStructs
        StructSimpleMarkerSymbol
        StructCharacterMarkerSymbol
        StructPictureMarkerSymbol
        StructArrowMarkerSymbol
        StructMultilayerMarkerSymbol
    End Enum

    Friend Enum LineStructs
        StructSimpleLineSymbol = 0
        StructMarkerLineSymbol = 1
        StructHashLineSymbol = 2
        StructPictureLineSymbol = 3
        StructMultilayerLineSymbol = 4
        StructCartographicLineSymbol = 5
    End Enum


#End Region

    '##################################################################################################
    '#################################### DATENSTRUKTUREN #############################################
    '##################################################################################################

#Region "Datenstrukturen"

    '************************************ Datenstruktur auf Projektebene *********************************

    'Die Datenstruktur fr die Layer (alle Layer in dem Projekt stecken hier drin
    Friend Structure StructProject
        Public LayerList As ArrayList   'Hier stecken alle Layer als StructLayer-Sammlung drin
        Public LayerCount As Integer    'Anzahl der Layer
    End Structure

    '************************************ Datenstrukturen auf Layerebene *********************************

    'Die Datenstruktur fr den UniqueValueRenderer
    Friend Structure StructUniqueValueRenderer
        Public FeatureCls As FeatureClass   'Ob Punkt-, Linien-, oder Polygonfeature
        Public LayerName As String  'Der Layername (ist nicht der Name, nach dem klassifiziert wird)
        Public DatasetName As String    'Der Datasetname (Der Name, nach dem klassifiziert wird)
        Public ValueCount As Integer   'Die Anzahl der Wertefelder bzw. Symbole des Layers basierend auf der aktuellen Klassifizierung
        Public SymbolList As ArrayList  'Die Sammlung der einzelnen Attributausprgungen (Symbole) als Struct*Symbol...
        Public FieldCount As Integer    'Die Anzahl der Tabellenfelder, nach denen klassifiziert wird (0: kein Feld; 1 Feld; 2 Felder; 3 Felder max.)
        Public FieldNames As ArrayList    'Die Tabellenfelder, nach denen Klassifiziert wird (max. 3 Felder) als Strings
        Public StylePath As String      'Der Pfad zur Stildefinition oder Stildatei (entspr. Layer Propertys->Symbology->Categorys->Match to Symbol in a Style in ArcMap!)
        Public Annotation As StructAnnotation 'Annotation label based on feature
    End Structure


    'Die Datenstruktur fr den ClassBreaksRenderer
    Friend Structure StructClassBreaksRenderer
        Public FeatureCls As FeatureClass   'Ob Punkt-, Linien-, oder Polygonfeature
        Public LayerName As String  'Der Layername (ist nicht der Name, nach dem klassifiziert wird)
        Public DatasetName As String    'Der Datasetname (Der Name, nach dem klassifiziert wird)
        Public BreakCount As Integer   'Die Anzahl der Wertefelder bzw. Symbole des Layers basierend auf der aktuellen Klassifizierung
        Public FieldName As String  'Das Tabellenfeld, nach dem Klassifiziert wird
        Public NormFieldName As String  'Das Tabellenfeld, nach dem normalisiert wird
        Public SymbolList As ArrayList  'Die Sammlung der einzelnen Attributausprgungen (Symbole) als Struct*Symbol...
        Public Annotation As StructAnnotation 'Annotation label based on feature
    End Structure


    'Die Datenstruktur fr den SimpleRenderer
    Friend Structure StructSimpleRenderer
        Public FeatureCls As FeatureClass   'Ob Punkt-, Linien-, oder Polygonfeature
        Public LayerName As String  'Der Layername (ist nicht der Name, nach dem klassifiziert wird)
        Public DatasetName As String    'Der Datasetname (Der Name, nach dem klassifiziert wird)
        Public SymbolList As ArrayList  'Die Sammlung der einzelnen Attributausprgungen (Symbole) als Struct*Symbol... In diesem Fall jew nur 1 Symbol
        Public Annotation As StructAnnotation 'Annotation label based on feature
    End Structure


    '********************************* Datenstrukturen auf Symbolebene **********************************

    'Die Datenstruktur fr SimpleMarkerSymbol
    Friend Structure StructSimpleMarkerSymbol
        Public Angle As Double  'Der Winkel des Symbols
        Public Filled As Boolean 'Whether fill color or not
        Public Color As String  'Die Farbe des Symbols in der Webschreibweise als #ByteByteByte
        Public Outline As Boolean   'Outline des Symbols
        Public OutlineColor As String   'Die Farbe der Outline in der Webschreibweise als #ByteByteByte
        Public OutlineSize As Double    'die Dicke der Outline
        Public Size As Double   'Die Symbolgre
        Public Style As String  'der esriSimpleMarkerStyle
        Public XOffset As Double    'der Offset zur Koordinate des Sy<mbols
        Public YOffset As Double     'der Offset zur Koordinate des Sy<mbols
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Daten ermittelt auf Rendererlevel:
        Public Label As String  'Die Feldwertbezeichnung (Label)    [alle Renderer]
        Public Fieldvalues As ArrayList 'Der Feldwert(e), nach dem Klassifiziert und dieses Symbol zugewiesen wird [Alle Renderer]. Die Reihenfolge der Werte entspricht auch der Reihenfolge der Felder (Spalten), wie sie im Renderereobjekt unter 'FieldNames as ArrayList' zu finden ist
        Public UpperLimit As Double    'Obergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
        Public LowerLimit As Double     'Untergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
    End Structure

    'Die Datenstruktur fr CharacterMarkerSymbol
    Friend Structure StructCharacterMarkerSymbol
        Public Angle As Double  'Winkel des "Buchstabens"
        Public CharacterIndex As Integer    'Stelle des Buchstabens in der ASCII (ANSII)-Tabelle des zugehrigen Fonts
        Public Color As String  'Die Farbe
        Public Font As String   'Der Font des Zeichens (Buchstabens)
        Public Size As Double   'Die Schriftgre
        Public XOffset As Double    'der Offset zur Koordinate des Sy<mbols
        Public YOffset As Double     'der Offset zur Koordinate des Sy<mbols
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Daten ermittelt auf Rendererlevel:
        Public Label As String  'Die Feldwertbezeichnung (Label)    [alle Renderer]
        Public Fieldvalues As ArrayList 'Der Feldwert(e), nach dem Klassifiziert und dieses Symbol zugewiesen wird [Alle Renderer]. Die Reihenfolge der Werte entspricht auch der Reihenfolge der Felder (Spalten), wie sie im Renderereobjekt unter 'FieldNames as ArrayList' zu finden ist
        Public UpperLimit As Double    'Obergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
        Public LowerLimit As Double     'Untergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
    End Structure

    'Die Datenstruktur fr PictureMarkerSymbol
    Friend Structure StructPictureMarkerSymbol
        Public Angle As Double  'Winkel
        Public BackgroundColor As String  'Die Farbe des BGColor in der Webschreibweise als #ByteByteByte
        Public Color As String  'Keine Ahnung welche Farbe das ist
        Public Picture As IPicture    'Das Bild
        Public Size As Double   'Die Bildgre
        Public XOffset As Double    'der Offset zur Koordinate des Sy<mbols
        Public YOffset As Double     'der Offset zur Koordinate des Sy<mbols
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Daten ermittelt auf Rendererlevel:
        Public Label As String  'Die Feldwertbezeichnung (Label)    [alle Renderer]
        Public Fieldvalues As ArrayList 'Der Feldwert(e), nach dem Klassifiziert und dieses Symbol zugewiesen wird [Alle Renderer]. Die Reihenfolge der Werte entspricht auch der Reihenfolge der Felder (Spalten), wie sie im Renderereobjekt unter 'FieldNames as ArrayList' zu finden ist
        Public UpperLimit As Double    'Obergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
        Public LowerLimit As Double     'Untergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
    End Structure

    'Die Datenstruktur fr PictureMarkerSymbol
    Friend Structure StructArrowMarkerSymbol
        Public Angle As Double  'Der Winkel
        Public Color As String  'Die Farbe
        Public Length As Double 'Die Pfeillnge
        Public Size As Double   'Die Die Pfeilgre
        Public Style As String  'Der Esrieigene Pfeilstyle
        Public Width As Double  'Die Pfeilbreite
        Public XOffset As Double    'der Offset zur Koordinate des Sy<mbols
        Public YOffset As Double     'der Offset zur Koordinate des Sy<mbols
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Daten ermittelt auf Rendererlevel:
        Public Label As String  'Die Feldwertbezeichnung (Label)    [alle Renderer]
        Public Fieldvalues As ArrayList 'Der Feldwert(e), nach dem Klassifiziert und dieses Symbol zugewiesen wird [Alle Renderer]. Die Reihenfolge der Werte entspricht auch der Reihenfolge der Felder (Spalten), wie sie im Renderereobjekt unter 'FieldNames as ArrayList' zu finden ist
        Public UpperLimit As Double    'Obergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
        Public LowerLimit As Double     'Untergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
    End Structure

    '____________________________________________________________________________________________________

    'Die Datenstruktur fr SimpleLineSymbol
    Friend Structure StructSimpleLineSymbol
        Public Color As String  'Die Farbe des Symbols in der Webschreibweise als #ByteByteByte
        Public Transparency As Byte 'Transparancy of the color.
        Public publicStyle As String   'der esriSimpleLineStyle
        Public Width As Double  'die Linienbreite
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Daten ermittelt auf Rendererlevel:
        Public Label As String  'Die Feldwertbezeichnung (Label)    [alle Renderer]
        Public Fieldvalues As ArrayList 'Der Feldwert(e), nach dem Klassifiziert und dieses Symbol zugewiesen wird [Alle Renderer]. Die Reihenfolge der Werte entspricht auch der Reihenfolge der Felder (Spalten), wie sie im Renderereobjekt unter 'FieldNames as ArrayList' zu finden ist
        Public UpperLimit As Double    'Obergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
        Public LowerLimit As Double     'Untergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
    End Structure

    'Die Datenstruktur fr CartographicLineSymbol
    Friend Structure StructCartographicLineSymbol
        Public Color As String  'Die Farbe des Symbols in der Webschreibweise als #ByteByteByte
        Public Transparency As Byte 'Transparancy of the color.
        Public Width As Double  'Die Strichbreite
        Public Join As String   'sagt aus, wie die Linien verbunden sind
        Public MiterLimit As Double     'Schwellenwert, ab welchem Abstand Zwickel angezeigt werden
        Public Cap As String    'Die Form des Linienendes
        Public DashArray As ArrayList 'The dasharray ("Template" in terms of ESRI) for dashed lines
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Daten ermittelt auf Rendererlevel:
        Public Label As String  'Die Feldwertbezeichnung (Label)    [alle Renderer]
        Public Fieldvalues As ArrayList 'Der Feldwert(e), nach dem Klassifiziert und dieses Symbol zugewiesen wird [Alle Renderer]. Die Reihenfolge der Werte entspricht auch der Reihenfolge der Felder (Spalten), wie sie im Renderereobjekt unter 'FieldNames as ArrayList' zu finden ist
        Public UpperLimit As Double    'Obergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
        Public LowerLimit As Double     'Untergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
    End Structure

    ' Die Datenstruktur fr HashLineSymbol
    Friend Structure StructHashLineSymbol
        Public Angle As Double  'Der Winkel
        Public Color As String  'Die Farbe der Linie
        Public Transparency As Byte 'Transparancy of the color.
        Public Width As Double  'der Abstand der einzelnen Striche
        '____________________________________________________
        Public kindOfLineStruct As LineStructs  'steht drin, welcher struct im jeweiligen Fall benutzt wurde
        Public HashSymbol_SimpleLine As StructSimpleLineSymbol
        Public HashSymbol_CartographicLine As StructCartographicLineSymbol
        Public HashSymbol_MarkerLine As StructMarkerLineSymbol
        'Public HashSymbol_HashLine as StructHashLineSymbol        geht nicht,da eine Struktur sich nicht selbst enthalten kann (s. StructHashLineSymbol)
        Public HashSymbol_PictureLine As StructPictureLineSymbol
        Public HashSymbol_MultiLayerLines As StructMultilayerLineSymbol
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Daten ermittelt auf Rendererlevel:
        Public Label As String  'Die Feldwertbezeichnung (Label)    [alle Renderer]
        Public Fieldvalues As ArrayList 'Der Feldwert(e), nach dem Klassifiziert und dieses Symbol zugewiesen wird [Alle Renderer]. Die Reihenfolge der Werte entspricht auch der Reihenfolge der Felder (Spalten), wie sie im Renderereobjekt unter 'FieldNames as ArrayList' zu finden ist
        Public UpperLimit As Double    'Obergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
        Public LowerLimit As Double     'Untergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
    End Structure

    'Die Datenstruktur fr MarkerLineSymbol
    Friend Structure StructMarkerLineSymbol
        Public Color As String  'Die Farbe
        Public Transparency As Byte 'Transparancy of the color.
        Public Width As Double  'Der Abstand der einzelnen Marker
        '____________________________________________________
        Public kindOfMarkerStruct As MarkerStructs  'steht drin, welcher struct im jeweiligen Fall benutzt wurde
        Public MarkerSymbol_SimpleMarker As StructSimpleMarkerSymbol
        Public MarkerSymbol_CharacterMarker As StructCharacterMarkerSymbol
        Public MarkerSymbol_PictureMarker As StructPictureMarkerSymbol
        Public MarkerSymbol_ArrowMarker As StructArrowMarkerSymbol
        Public MarkerSymbol_MultilayerMarker As StructMultilayerMarkerSymbol
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Daten ermittelt auf Rendererlevel:
        Public Label As String  'Die Feldwertbezeichnung (Label)    [alle Renderer]
        Public Fieldvalues As ArrayList 'Der Feldwert(e), nach dem Klassifiziert und dieses Symbol zugewiesen wird [Alle Renderer]. Die Reihenfolge der Werte entspricht auch der Reihenfolge der Felder (Spalten), wie sie im Renderereobjekt unter 'FieldNames as ArrayList' zu finden ist
        Public UpperLimit As Double    'Obergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
        Public LowerLimit As Double     'Untergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
    End Structure

    'Die Datenstruktur fr PictureLineSymbol
    Friend Structure StructPictureLineSymbol
        Public BackgroundColor As String    'Hintergrundfarbe
        Public BackgroundTransparency As Byte 'Transparancy of the color.
        Public Color As String  '??
        Public Transparency As Byte 'Transparancy of the color.
        Public Offset As Double '??
        Public Picture As IPicture    'Das Bild
        Public Rotate As Boolean    'Bild rotieren?
        Public Width As Double  'Gre
        Public XScale As Double     'Seitenverhltnis X
        Public YScale As Double 'Seitenverhltnis Y
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Daten ermittelt auf Rendererlevel:
        Public Label As String  'Die Feldwertbezeichnung (Label)    [alle Renderer]
        Public Fieldvalues As ArrayList 'Der Feldwert(e), nach dem Klassifiziert und dieses Symbol zugewiesen wird [Alle Renderer]. Die Reihenfolge der Werte entspricht auch der Reihenfolge der Felder (Spalten), wie sie im Renderereobjekt unter 'FieldNames as ArrayList' zu finden ist
        Public UpperLimit As Double    'Obergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
        Public LowerLimit As Double     'Untergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
    End Structure

    '____________________________________________________________________________________________________

    'Die Datenstruktur fr SimpleFillSymbol
    Friend Structure StructSimpleFillSymbol
        Public Color As String  'Die Farbe des Fillsymbols
        Public Style As String  'esriSimpleFillStyle
        Public Transparency As Byte
        '____________________________________________________
        Public kindOfOutlineStruct As LineStructs  'steht drin, welcher struct im jeweiligen Fall benutzt wurde
        Public Outline_SimpleLine As StructSimpleLineSymbol
        Public Outline_CartographicLine As StructCartographicLineSymbol
        Public Outline_MarkerLine As StructMarkerLineSymbol
        Public Outline_HashLine As StructHashLineSymbol
        Public Outline_PictureLine As StructPictureLineSymbol
        Public Outline_MultiLayerLines As StructMultilayerLineSymbol
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Daten ermittelt auf Rendererlevel:
        Public Label As String  'Die Feldwertbezeichnung (Label)    [alle Renderer]
        Public Fieldvalues As ArrayList 'Der Feldwert(e), nach dem Klassifiziert und dieses Symbol zugewiesen wird [Alle Renderer]. Die Reihenfolge der Werte entspricht auch der Reihenfolge der Felder (Spalten), wie sie im Renderereobjekt unter 'FieldNames as ArrayList' zu finden ist
        Public UpperLimit As Double    'Obergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
        Public LowerLimit As Double     'Untergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
    End Structure

    'Die Datenstruktur fr MarkerFillSymbol
    Friend Structure StructMarkerFillSymbol
        Public Color As String  'Die Fllfarbe
        Public Transparency As Byte
        Public GridAngle As Double  'der Winkel des Grids
        '____________________________________________________
        Public kindOfMarkerStruct As MarkerStructs  'steht drin, welcher struct im jeweiligen Fall benutzt wurde
        Public MarkerSymbol_SimpleMarker As StructSimpleMarkerSymbol
        Public MarkerSymbol_CharacterMarker As StructCharacterMarkerSymbol
        Public MarkerSymbol_PictureMarker As StructPictureMarkerSymbol
        Public MarkerSymbol_ArrowMarker As StructArrowMarkerSymbol
        Public MarkerSymbol_MultilayerMarker As StructMultilayerMarkerSymbol
        '____________________________________________________
        Public kindOfOutlineStruct As LineStructs  'steht drin, welcher struct im jeweiligen Fall benutzt wurde
        Public Outline_SimpleLine As StructSimpleLineSymbol
        Public Outline_CartographicLine As StructCartographicLineSymbol
        Public Outline_MarkerLine As StructMarkerLineSymbol
        Public Outline_HashLine As StructHashLineSymbol
        Public Outline_PictureLine As StructPictureLineSymbol
        Public Outline_MultiLayerLines As StructMultilayerLineSymbol
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Daten ermittelt auf Rendererlevel:
        Public Label As String  'Die Feldwertbezeichnung (Label)    [alle Renderer]
        Public Fieldvalues As ArrayList 'Der Feldwert(e), nach dem Klassifiziert und dieses Symbol zugewiesen wird [Alle Renderer]. Die Reihenfolge der Werte entspricht auch der Reihenfolge der Felder (Spalten), wie sie im Renderereobjekt unter 'FieldNames as ArrayList' zu finden ist
        Public UpperLimit As Double    'Obergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
        Public LowerLimit As Double     'Untergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
    End Structure

    'Die Datenstruktur fr LineFillSymbol
    Friend Structure StructLineFillSymbol
        Public Angle As Double  'Der Linienwinkel
        Public Color As String  'Die Hintergrundfarbe
        Public Transparency As Byte
        Public Offset As Double     'der Linienversatz bei z.B. versch.-farbigen gegeneinander verschobenen Linien
        Public Separation As Double 'Der Linienabstand
        '____________________________________________________
        Public kindOfLineStruct As LineStructs       'steht drin, welcher struct im jeweiligen Fall benutzt wurde
        Public LineSymbol_SimpleLine As StructSimpleLineSymbol
        Public LineSymbol_CartographicLine As StructCartographicLineSymbol
        Public LineSymbol_MarkerLine As StructMarkerLineSymbol
        Public LineSymbol_HashLine As StructHashLineSymbol
        Public LineSymbol_PictureLine As StructPictureLineSymbol
        Public LineSymbol_MultiLayerLines As StructMultilayerLineSymbol
        '____________________________________________________
        Public kindOfOutlineStruct As LineStructs        'steht drin, welcher struct im jeweiligen Fall benutzt wurde
        Public Outline_SimpleLine As StructSimpleLineSymbol
        Public Outline_CartographicLine As StructCartographicLineSymbol
        Public Outline_MarkerLine As StructMarkerLineSymbol
        Public Outline_HashLine As StructHashLineSymbol
        Public Outline_PictureLine As StructPictureLineSymbol
        Public Outline_MultiLayerLines As StructMultilayerLineSymbol
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Daten ermittelt auf Rendererlevel:
        Public Label As String  'Die Feldwertbezeichnung (Label)    [alle Renderer]
        Public Fieldvalues As ArrayList 'Der Feldwert(e), nach dem Klassifiziert und dieses Symbol zugewiesen wird [Alle Renderer]. Die Reihenfolge der Werte entspricht auch der Reihenfolge der Felder (Spalten), wie sie im Renderereobjekt unter 'FieldNames as ArrayList' zu finden ist
        Public UpperLimit As Double    'Obergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
        Public LowerLimit As Double     'Untergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
    End Structure

    'Die Datenstruktur fr DotDensityFillSymbol
    Friend Structure StructDotDensityFillSymbol
        Public BackgroundColor As String    'Die Hintergrundfarbe
        Public BackgroundTransparency As Byte 'Transparancy of the color.
        Public Color As String  '?
        Public Transparency As Byte
        Public DotCount As Integer  'Anzahl der Punkte
        Public DotSize As Double    'Symbolgre
        Public DotSpacing As Double 'Distanz zwischen den Mittelpunkten der Symbole
        Public FixedPlacement As Boolean    'Bei true, werdfen die Symbole immer an der gleichen Stelle plaziert, andernfalls random verteilt
        Public SymbolList As ArrayList  'Das Array mit den einzelnen Marker-Symbolen des DotDensityFillSymbol 
        Public SymbolCount As Integer   'Die Anzahl der unterschiedlichen Symbole
        '____________________________________________________
        Public kindOfOutlineStruct As LineStructs       'steht drin, welcher struct im jeweiligen Fall benutzt wurde
        Public Outline_SimpleLine As StructSimpleLineSymbol
        Public Outline_CartographicLine As StructCartographicLineSymbol
        Public Outline_MarkerLine As StructMarkerLineSymbol
        Public Outline_HashLine As StructHashLineSymbol
        Public Outline_PictureLine As StructPictureLineSymbol
        Public Outline_MultiLayerLines As StructMultilayerLineSymbol
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Daten ermittelt auf Rendererlevel:
        Public Label As String  'Die Feldwertbezeichnung (Label)    [alle Renderer]
        Public Fieldvalues As ArrayList 'Der Feldwert(e), nach dem Klassifiziert und dieses Symbol zugewiesen wird [Alle Renderer]. Die Reihenfolge der Werte entspricht auch der Reihenfolge der Felder (Spalten), wie sie im Renderereobjekt unter 'FieldNames as ArrayList' zu finden ist
        Public UpperLimit As Double    'Obergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
        Public LowerLimit As Double     'Untergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
    End Structure

    'Die Datenstruktur fr PictureFillSymbol
    Friend Structure StructPictureFillSymbol
        Public Angle As Double  'Der Winkel der Bilddrehung
        Public BackgroundColor As String    'die Hintergrundfarbe
        Public BackgroundTransparency As Byte 'Transparancy of the color.
        Public Color As String  '?
        Public Transparency As Byte
        Public Picture As IPictureDisp    'Das Bild
        Public XScale As Double     'Seitenverhltnis X
        Public YScale As Double 'Seitenverhltnis Y
        '____________________________________________________
        Public kindOfOutlineStruct As LineStructs       'steht drin, welcher struct im jeweiligen Fall benutzt wurde
        Public Outline_SimpleLine As StructSimpleLineSymbol
        Public Outline_CartographicLine As StructCartographicLineSymbol
        Public Outline_MarkerLine As StructMarkerLineSymbol
        Public Outline_HashLine As StructHashLineSymbol
        Public Outline_PictureLine As StructPictureLineSymbol
        Public Outline_MultiLayerLines As StructMultilayerLineSymbol
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Daten ermittelt auf Rendererlevel:
        Public Label As String  'Die Feldwertbezeichnung (Label)    [alle Renderer]
        Public Fieldvalues As ArrayList 'Der Feldwert(e), nach dem Klassifiziert und dieses Symbol zugewiesen wird [Alle Renderer]. Die Reihenfolge der Werte entspricht auch der Reihenfolge der Felder (Spalten), wie sie im Renderereobjekt unter 'FieldNames as ArrayList' zu finden ist
        Public UpperLimit As Double    'Obergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
        Public LowerLimit As Double     'Untergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
    End Structure

    'Die Datenstruktur fr GradientFillSymbol
    Friend Structure StructGradientFillSymbol
        Public Color As String  '?
        Public Transparency As Byte
        Public Colors As ArrayList  'die einzelnen Farben der ColorRamp as string
        Public GradientAngle As Double  'Neigungswinkel fr das Fllmuster
        Public GradientPercentage As Double 'Zahl zw. 0 und 1. 1=Farbverlauf ber die gesamte Flche. 0.5=Hlfte der Flche wird mit Farbverlauf dargestellt
        Public IntervallCount As Integer    'Anzahl der Farben im Farbverlauf
        Public Style As String  'der esriGradientFillStyle
        '____________________________________________________
        Public kindOfOutlineStruct As LineStructs       'steht drin, welcher struct im jeweiligen Fall benutzt wurde
        Public Outline_SimpleLine As StructSimpleLineSymbol
        Public Outline_CartographicLine As StructCartographicLineSymbol
        Public Outline_MarkerLine As StructMarkerLineSymbol
        Public Outline_HashLine As StructHashLineSymbol
        Public Outline_PictureLine As StructPictureLineSymbol
        Public Outline_MultiLayerLines As StructMultilayerLineSymbol
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Daten ermittelt auf Rendererlevel:
        Public Label As String  'Die Feldwertbezeichnung (Label)    [alle Renderer]
        Public Fieldvalues As ArrayList 'Der Feldwert(e), nach dem Klassifiziert und dieses Symbol zugewiesen wird [Alle Renderer]. Die Reihenfolge der Werte entspricht auch der Reihenfolge der Felder (Spalten), wie sie im Renderereobjekt unter 'FieldNames as ArrayList' zu finden ist
        Public UpperLimit As Double    'Obergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
        Public LowerLimit As Double     'Untergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
    End Structure

    '____________________________________________________________________________________________________


    'Die Datenstruktur fr BarChartSymbol
    Friend Structure StructBarChartSymbol
        Public ShowAxes As Boolean  '...
        Public Spacing As Double    'Der Platz zwischen den Balken
        Public VerticalBars As Boolean  '...
        Public Width As Double  '...
        '____________________________________________________
        Public kindOfAxeslineStruct As LineStructs       'steht drin, welcher struct im jeweiligen Fall benutzt wurde
        Public Axes_SimpleLine As StructSimpleLineSymbol
        Public Axes_CartographicLine As StructCartographicLineSymbol
        Public Axes_MarkerLine As StructMarkerLineSymbol
        Public Axes_HashLine As StructHashLineSymbol
        Public Axes_PictureLine As StructPictureLineSymbol
        Public Axes_MultiLayerLines As StructMultilayerLineSymbol
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Daten ermittelt auf Rendererlevel:
        Public Label As String  'Die Feldwertbezeichnung (Label)    [alle Renderer]
        Public Fieldvalues As ArrayList 'Der Feldwert(e), nach dem Klassifiziert und dieses Symbol zugewiesen wird [Alle Renderer]. Die Reihenfolge der Werte entspricht auch der Reihenfolge der Felder (Spalten), wie sie im Renderereobjekt unter 'FieldNames as ArrayList' zu finden ist
        Public UpperLimit As Double    'Obergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
        Public LowerLimit As Double     'Untergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
    End Structure

    'Die Datenstruktur fr PieChartSymbol
    Friend Structure StructPieChartSymbol
        Public Clockwise As Boolean 'Uhrzeigersinn oder nicht
        Public UseOutline As Boolean '...
        '____________________________________________________
        Public kindOfOutlineStruct As LineStructs       'steht drin, welcher struct im jeweiligen Fall benutzt wurde
        Public Outline_SimpleLine As StructSimpleLineSymbol
        Public Outline_CartographicLine As StructCartographicLineSymbol
        Public Outline_MarkerLine As StructMarkerLineSymbol
        Public Outline_HashLine As StructHashLineSymbol
        Public Outline_PictureLine As StructPictureLineSymbol
        Public Outline_MultiLayerLines As StructMultilayerLineSymbol
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Daten ermittelt auf Rendererlevel:
        Public Label As String  'Die Feldwertbezeichnung (Label)    [alle Renderer]
        Public Fieldvalues As ArrayList 'Der Feldwert(e), nach dem Klassifiziert und dieses Symbol zugewiesen wird [Alle Renderer]. Die Reihenfolge der Werte entspricht auch der Reihenfolge der Felder (Spalten), wie sie im Renderereobjekt unter 'FieldNames as ArrayList' zu finden ist
        Public UpperLimit As Double    'Obergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
        Public LowerLimit As Double     'Untergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
    End Structure

    'Die Datenstruktur fr StackedChartSymbol
    Friend Structure StructStackedChartSymbol
        Public Fixed As Boolean '...
        Public UseOutline As Boolean '...
        Public VerticalBar As Boolean
        Public Width As Double  '...
        '____________________________________________________
        Public kindOfOutlineStruct As LineStructs       'steht drin, welcher struct im jeweiligen Fall benutzt wurde
        Public Outline_SimpleLine As StructSimpleLineSymbol
        Public Outline_CartographicLine As StructCartographicLineSymbol
        Public Outline_MarkerLine As StructMarkerLineSymbol
        Public Outline_HashLine As StructHashLineSymbol
        Public Outline_PictureLine As StructPictureLineSymbol
        Public Outline_MultiLayerLines As StructMultilayerLineSymbol
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Daten ermittelt auf Rendererlevel:
        Public Label As String  'Die Feldwertbezeichnung (Label)    [alle Renderer]
        Public Fieldvalues As ArrayList 'Der Feldwert(e), nach dem Klassifiziert und dieses Symbol zugewiesen wird [Alle Renderer]. Die Reihenfolge der Werte entspricht auch der Reihenfolge der Felder (Spalten), wie sie im Renderereobjekt unter 'FieldNames as ArrayList' zu finden ist
        Public UpperLimit As Double    'Obergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
        Public LowerLimit As Double     'Untergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
    End Structure

    '____________________________________________________________________________________________________

    'Die Datenstruktur fr TextSymbol
    Friend Structure StructTextSymbol
        Public Angle As Double  'Der Textwinkel
        Public Color As String  '...
        Public Font As String   '...
        Public Style As String  'Font style in CSS notation, e.g. italic or normal
        Public Weight As String 'Font weight in CSS notation, e.g. bold or normal
        Public HorizontalAlignment As String    ' die Horizontale Textausrichtung als esriTextHorizontalAlignment
        Public RightToLeft As Boolean   '?
        Public Size As Double   '...
        Public Text As String  '...
        Public VerticalAlignment As String  'die Verticale Textausrichtung als esriTextVerticalAlignment
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Daten ermittelt auf Rendererlevel:
        Public Label As String  'Die Feldwertbezeichnung (Label)    [alle Renderer]
        Public Fieldvalues As ArrayList 'Der Feldwert(e), nach dem Klassifiziert und dieses Symbol zugewiesen wird [Alle Renderer]. Die Reihenfolge der Werte entspricht auch der Reihenfolge der Felder (Spalten), wie sie im Renderereobjekt unter 'FieldNames as ArrayList' zu finden ist
        Public UpperLimit As Double    'Obergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
        Public LowerLimit As Double     'Untergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
    End Structure



    '*************************** Die Strukturen fr dieMultilayer-Symbole ******************************

    'Das Marker-Multilayersymbol
    Friend Structure StructMultilayerMarkerSymbol
        Public MultiMarkerLayers As ArrayList    'Das Symbol kann aus mehreren Markersymbollayern zusammengestzt sein
        Public LayerCount As Integer    'Anzahl der Layer
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Daten ermittelt auf Rendererlevel:
        Public Label As String  'Die Feldwertbezeichnung (Label)    [alle Renderer]
        Public Fieldvalues As ArrayList 'Der Feldwert(e), nach dem Klassifiziert und dieses Symbol zugewiesen wird [Alle Renderer]. Die Reihenfolge der Werte entspricht auch der Reihenfolge der Felder (Spalten), wie sie im Renderereobjekt unter 'FieldNames as ArrayList' zu finden ist
        Public UpperLimit As Double    'Obergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
        Public LowerLimit As Double     'Untergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
    End Structure

    'Das Line-Multilayersymbol
    Friend Structure StructMultilayerLineSymbol
        Public MultiLineLayers As ArrayList  'Das Symbol kann aus mehreren Linesymbollayern zusammengestzt sein
        Public LayerCount As Integer    'Anzahl der Layer
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Daten ermittelt auf Rendererlevel:
        Public Label As String  'Die Feldwertbezeichnung (Label)    [alle Renderer]
        Public Fieldvalues As ArrayList 'Der Feldwert(e), nach dem Klassifiziert und dieses Symbol zugewiesen wird [Alle Renderer]. Die Reihenfolge der Werte entspricht auch der Reihenfolge der Felder (Spalten), wie sie im Renderereobjekt unter 'FieldNames as ArrayList' zu finden ist
        Public UpperLimit As Double    'Obergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
        Public LowerLimit As Double     'Untergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
    End Structure

    'Das Fill-Multilayersymbol
    Friend Structure StructMultilayerFillSymbol
        Public MultiFillLayers As ArrayList  'Das Symbol kann aus mehreren Fillsymbollayern zusammengestzt sein
        Public LayerCount As Integer    'Anzahl der Layer
        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Daten ermittelt auf Rendererlevel:
        Public Label As String  'Die Feldwertbezeichnung (Label)    [alle Renderer]
        Public Fieldvalues As ArrayList 'Der Feldwert(e), nach dem Klassifiziert und dieses Symbol zugewiesen wird [Alle Renderer]. Die Reihenfolge der Werte entspricht auch der Reihenfolge der Felder (Spalten), wie sie im Renderereobjekt unter 'FieldNames as ArrayList' zu finden ist
        Public UpperLimit As Double    'Obergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
        Public LowerLimit As Double     'Untergrenze der das aktuelle Symbol betr. Range beim ClassBreaksRenderer
    End Structure

    Friend Structure StructAnnotation
        Public IsSingleProperty As Boolean
        Public PropertyName As String
        Public TextSymbol As StructTextSymbol
    End Structure
#End Region  'Die Deklaration der Datenstrukturen, in denen alle analysierten Daten erfasst werden

    '##################################################################################################
    '########################################### ROUTINEN #############################################
    '##################################################################################################

#Region "Routinen" 'u.a. die public Sub New()

    Public Sub New(ByVal value As Motherform, ByVal Filename As String)
        m_cFilename = Filename  '1.)
        frmMotherform = value   '2.)

        m_ObjApp = Nothing  'Um sicherzustellen, da kein Unsinn initialisiert ist
        CentralProcessingFunc()
    End Sub

    '************************************************************************************************
    'Every time when a layer is added to the List the function adds one to the layer number
    '************************************************************************************************
    Private Sub AddOneToLayerNumber()
        m_StrProject.LayerCount = m_StrProject.LayerCount + 1
    End Sub

#End Region

    '##################################################################################################
    '######################################### PROPERTIES #############################################
    '##################################################################################################

#Region "Properties"

    'Gibt die Datenstruktur mit den gespeicherten Projektdaten weiter
    Public ReadOnly Property GetProjectData()
        Get
            Return m_StrProject
        End Get
    End Property

#End Region

    '##################################################################################################
    '######################################### FUNKTIONEN #############################################
    '##################################################################################################

#Region "Speicherfuntionen der Datenstrukturen"

    '************************************************************************************************
    'Speichert die Daten in einer SimpleRenderer-Datenstruktur
    '************************************************************************************************
    Private Function StoreStructSimpleRenderer(ByVal Renderer As ISimpleRenderer, ByVal Layer As IFeatureLayer) As StructSimpleRenderer
        Dim strRenderer As New StructSimpleRenderer
        Dim objFstOrderSymbol As ISymbol
        Dim objDataset As IDataset
        objDataset = Layer.FeatureClass
        strRenderer.SymbolList = New ArrayList

        Try

            strRenderer.LayerName = Layer.Name
            strRenderer.DatasetName = objDataset.Name
            strRenderer.Annotation = GetAnnotation(Layer)
            'AB HIER BEGINNEN DIE FALLUNTERSCHEIDUNGEN DER SYMBOLE
            objFstOrderSymbol = Renderer.Symbol  'Die Zuweisung der jeweiligen einzelnen Symbole
            'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
            If TypeOf objFstOrderSymbol Is ITextSymbol Then
                Dim strTS As StructTextSymbol
                Dim objSymbol As ITextSymbol
                objSymbol = objFstOrderSymbol
                strTS = StoreText(objSymbol)
                '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                strTS.Label = Renderer.Label
                strRenderer.SymbolList.Add(strTS)

            End If
            'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx()
            If TypeOf objFstOrderSymbol Is IMarkerSymbol Then
                strRenderer.FeatureCls = FeatureClass.PointFeature
                Dim objSymbol As IMarkerSymbol
                objSymbol = objFstOrderSymbol
                Select Case MarkerSymbolScan(objSymbol)
                    Case "ISimpleMarkerSymbol"
                        Dim SMS As ISimpleMarkerSymbol
                        SMS = objSymbol
                        Dim strSMS As StructSimpleMarkerSymbol
                        strSMS = StoreSimpleMarker(SMS)
                        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                        strSMS.Label = Renderer.Label
                        strRenderer.SymbolList.Add(strSMS)

                    Case "ICharacterMarkerSymbol"
                        Dim CMS As ICharacterMarkerSymbol
                        CMS = objSymbol
                        Dim strCMS As StructCharacterMarkerSymbol
                        strCMS = StoreCharacterMarker(CMS)
                        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                        strCMS.Label = Renderer.Label
                        strRenderer.SymbolList.Add(strCMS)

                    Case "IPictureMarkerSymbol"
                        Dim PMS As IPictureMarkerSymbol
                        PMS = objSymbol
                        Dim strPMS As StructPictureMarkerSymbol
                        strPMS = StorePictureMarker(PMS)
                        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                        strPMS.Label = Renderer.Label
                        strRenderer.SymbolList.Add(strPMS)

                    Case "IArrowMarkerSymbol"
                        Dim AMS As IArrowMarkerSymbol
                        AMS = objSymbol
                        Dim strAMS As StructArrowMarkerSymbol
                        strAMS = StoreArrowMarker(AMS)
                        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                        strAMS.Label = Renderer.Label
                        strRenderer.SymbolList.Add(strAMS)

                    Case "IMultiLayerMarkerSymbol"
                        Dim MLMS As IMultiLayerMarkerSymbol
                        MLMS = objSymbol
                        Dim strMLMS As StructMultilayerMarkerSymbol
                        strMLMS = StoreMultiLayerMarker(MLMS)
                        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                        strMLMS.Label = Renderer.Label
                        strRenderer.SymbolList.Add(strMLMS)

                    Case "false"
                        InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreStructLayer")
                End Select
            End If
            'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
            If TypeOf objFstOrderSymbol Is ILineSymbol Then
                strRenderer.FeatureCls = FeatureClass.LineFeature
                Dim objSymbol As ILineSymbol
                objSymbol = objFstOrderSymbol
                Select Case LineSymbolScan(objSymbol)
                    Case "ICartographicLineSymbol"
                        Dim CLS As ICartographicLineSymbol
                        CLS = objSymbol
                        Dim strCLS As StructCartographicLineSymbol
                        strCLS = StoreCartographicLine(CLS)
                        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                        strCLS.Label = Renderer.Label
                        strRenderer.SymbolList.Add(strCLS)

                    Case "IHashLineSymbol"
                        Dim HLS As IHashLineSymbol
                        HLS = objSymbol
                        Dim strHLS As StructHashLineSymbol
                        strHLS = StoreHashLine(HLS)
                        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                        strHLS.Label = Renderer.Label
                        strRenderer.SymbolList.Add(strHLS)

                    Case "IMarkerLineSymbol"
                        Dim MLS As IMarkerLineSymbol
                        MLS = objSymbol
                        Dim strMLS As StructMarkerLineSymbol
                        strMLS = StoreMarkerLine(MLS)
                        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                        strMLS.Label = Renderer.Label
                        strRenderer.SymbolList.Add(strMLS)

                    Case "ISimpleLineSymbol"
                        Dim SLS As ISimpleLineSymbol
                        SLS = objSymbol
                        Dim strSLS As StructSimpleLineSymbol
                        strSLS = StoreSimpleLine(SLS)
                        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                        strSLS.Label = Renderer.Label
                        strRenderer.SymbolList.Add(strSLS)

                    Case "IPictureLineSymbol"
                        Dim PLS As IPictureLineSymbol
                        PLS = objSymbol
                        Dim strPLS As StructPictureLineSymbol
                        strPLS = StorePictureLine(PLS)
                        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                        strPLS.Label = Renderer.Label
                        strRenderer.SymbolList.Add(strPLS)

                    Case "IMultiLayerLineSymbol"
                        Dim MLLS As IMultiLayerLineSymbol
                        MLLS = objSymbol
                        Dim strMLLS As StructMultilayerLineSymbol
                        strMLLS = StoreMultilayerLines(MLLS)
                        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                        strMLLS.Label = Renderer.Label
                        strRenderer.SymbolList.Add(strMLLS)

                    Case "false"
                        InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreStructLayer")
                End Select

            End If
            'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
            If TypeOf objFstOrderSymbol Is IFillSymbol Then
                strRenderer.FeatureCls = FeatureClass.PolygonFeature
                Dim objSymbol As IFillSymbol
                objSymbol = objFstOrderSymbol
                Select Case FillSymbolScan(objSymbol)
                    Case "ISimpleFillSymbol"
                        Dim SFS As ISimpleFillSymbol
                        SFS = objSymbol
                        Dim strSFS As StructSimpleFillSymbol
                        strSFS = StoreSimpleFill(SFS)
                        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                        strSFS.Label = Renderer.Label
                        strRenderer.SymbolList.Add(strSFS)

                    Case "IMarkerFillSymbol"
                        Dim MFS As IMarkerFillSymbol
                        MFS = objSymbol
                        Dim strMFS As StructMarkerFillSymbol
                        strMFS = StoreMarkerFill(MFS)
                        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                        strMFS.Label = Renderer.Label
                        strRenderer.SymbolList.Add(strMFS)

                    Case "ILineFillSymbol"
                        Dim LFS As ILineFillSymbol
                        LFS = objSymbol
                        Dim strLFS As StructLineFillSymbol
                        strLFS = StoreLineFill(LFS)
                        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                        strLFS.Label = Renderer.Label
                        strRenderer.SymbolList.Add(strLFS)

                    Case "IDotDensityFillSymbol"
                        Dim DFS As IDotDensityFillSymbol
                        DFS = objSymbol
                        Dim strDFS As StructDotDensityFillSymbol
                        strDFS = StoreDotDensityFill(DFS)
                        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                        strDFS.Label = Renderer.Label
                        strRenderer.SymbolList.Add(strDFS)

                    Case "IPictureFillSymbol"
                        Dim PFS As IPictureFillSymbol
                        PFS = objSymbol
                        Dim strPFS As StructPictureFillSymbol
                        strPFS = StorePictureFill(PFS)
                        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                        strPFS.Label = Renderer.Label
                        strRenderer.SymbolList.Add(strPFS)

                    Case "IGradientFillSymbol"
                        Dim GFS As IGradientFillSymbol
                        GFS = objSymbol
                        Dim strGFS As StructGradientFillSymbol
                        strGFS = StoreGradientFill(GFS)
                        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                        strGFS.Label = Renderer.Label
                        strRenderer.SymbolList.Add(strGFS)

                    Case "IMultiLayerFillSymbol"
                        Dim MLFS As IMultiLayerFillSymbol
                        MLFS = objSymbol
                        Dim strMLFS As StructMultilayerFillSymbol
                        strMLFS = StoreMultiLayerFill(MLFS)
                        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                        strMLFS.Label = Renderer.Label
                        strRenderer.SymbolList.Add(strMLFS)

                    Case "false"
                        InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreStructLayer")
                End Select
            End If
            'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
            If TypeOf objFstOrderSymbol Is I3DChartSymbol Then
                Dim objSymbol As I3DChartSymbol
                objSymbol = objFstOrderSymbol
                Select Case IIIDChartSymbolScan(objSymbol)
                    Case "IBarChartSymbol"
                        Dim BCS As IBarChartSymbol
                        BCS = objSymbol
                        Dim strBCS As StructBarChartSymbol
                        strBCS = StoreBarChart(BCS)
                        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                        strBCS.Label = Renderer.Label
                        strRenderer.SymbolList.Add(strBCS)

                    Case "IPieChartSymbol"
                        Dim PCS As IPieChartSymbol
                        PCS = objSymbol
                        Dim strPCS As StructPieChartSymbol
                        strPCS = StorePieChart(PCS)
                        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                        strPCS.Label = Renderer.Label
                        strRenderer.SymbolList.Add(strPCS)

                    Case "IStackedChartSymbol"
                        Dim SCS As IStackedChartSymbol
                        SCS = objSymbol
                        Dim strSCS As StructStackedChartSymbol
                        strSCS = StoreStackedChart(SCS)
                        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                        strSCS.Label = Renderer.Label
                        strRenderer.SymbolList.Add(strSCS)

                    Case "false"
                        InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreStructLayer")
                End Select
            End If
            'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
            Return strRenderer
        Catch ex As Exception

            ErrorMsg("Fehler beim Speichern der Symbole in den Layerstrukturen", ex.Message, ex.StackTrace, "StoreStructSimpleRenderer")
        End Try
    End Function


    '************************************************************************************************
    'Speichert die Daten in einer ClassBreaksRenderer-Datenstruktur. Gespeichert wird auf 2 Ebenen:
    '1. Daten auf Symbolebene: Symbole werden abgefragt und in Fallunterscheidung auf Untersymbole verteilt
    'dort werden die symboleigenen Daten in den Symboldatenstrukturen gespeichert
    '2. Daten auf Rendererebene: Daten, die nur auf Rendererebene verfgbar sind, werden hier ebenfalls auf 
    'Symbolebene gespeichert, damit sie direkt im Zusammenhang mit dem Symbol verfgbar sind.
    'Parameter: Renderer:   das Rendererobjekt des aktuellen Layers
    '           Layer:      das Layerobjekt des aktuellen Layers                       
    '************************************************************************************************
    Private Function StoreStructCBRenderer(ByVal Renderer As IClassBreaksRenderer, ByVal Layer As IFeatureLayer) As StructClassBreaksRenderer
        Dim strRenderer As New StructClassBreaksRenderer    'Hierin wird das eine Rendererobjekt gespeichert plus zustzliche Layerinformationen
        strRenderer.SymbolList = New ArrayList
        Dim iNumberOfSymbols As Integer     'Anzahl aller Symbole des Rendererobjekts
        iNumberOfSymbols = Renderer.BreakCount 'Anzahl der Symbole
        strRenderer.BreakCount = Renderer.BreakCount
        Dim objFstOrderSymbol As ISymbol    'Das gerade aktuelle Symbol des durchlaufenen Rendererobjekts
        Dim objDataset As IDataset
        objDataset = Layer.FeatureClass
        Dim bIsJoined As Boolean
        bIsJoined = False

        Try
            strRenderer.LayerName = Layer.Name
            strRenderer.DatasetName = objDataset.Name
            strRenderer.Annotation = GetAnnotation(Layer)

            'diese Objekte dienen lediglich der berprfung, ob eine andere Tabelle an die Featuretable gejoint ist
            '++++++++++++++++++++++++++++++++++++++++++
            Dim pTable As ITable
            Dim pDisplayTable As IDisplayTable
            pDisplayTable = Layer
            pTable = pDisplayTable.DisplayTable
            If TypeOf pTable Is IRelQueryTable Then 'Dann ist eine Tabelle drangejoint
                bIsJoined = True
            End If
            '++++++++++++++++++++++++++++++++++++++++++

            If bIsJoined = False Then   'Wenn eine Tabelle drangejoint wurde, ist sowieso schon der Datasetname mit dabei
                'Je nachdem, welcher Zielkartendienst gewhlt wurde, muss Fieldname ein anderes Format besitzen
                If frmMotherform.chkArcIMS.Checked = True Then
                    strRenderer.FieldName = objDataset.Name & "." & Renderer.Field
                    strRenderer.NormFieldName = objDataset.Name & "." & Renderer.NormField
                Else
                    strRenderer.FieldName = Renderer.Field
                    strRenderer.NormFieldName = Renderer.NormField
                End If
            End If



            'AB HIER BEGINNEN DIE FALLUNTERSCHEIDUNGEN DER SYMBOLE
            Dim j As Integer
            For j = 0 To iNumberOfSymbols - 1
                frmMotherform.CHLabelSmall("Symbol " & (j + 1).ToString & " von " & iNumberOfSymbols.ToString) 'Anzeige auf dem Label
                objFstOrderSymbol = Renderer.Symbol(j)  'Die Zuweisung der jeweiligen einzelnen Symbole
                Dim objClassBreaksProp As IClassBreaksUIProperties
                objClassBreaksProp = Renderer
                Dim cLowerLimit, cUpperLimit As String
                cLowerLimit = CStr(objClassBreaksProp.LowBreak(j))    'Die Untergrenze der aktuellen Klasse
                cUpperLimit = CStr(Renderer.Break(j))

                'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
                If TypeOf objFstOrderSymbol Is ITextSymbol Then
                    Dim strTS As StructTextSymbol
                    Dim objSymbol As ITextSymbol
                    objSymbol = objFstOrderSymbol
                    strTS = StoreText(objSymbol)
                    '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                    strTS.Label = Renderer.Label(j)
                    strTS.LowerLimit = cLowerLimit
                    strTS.UpperLimit = cUpperLimit
                    strRenderer.SymbolList.Add(strTS)

                End If
                'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx()
                If TypeOf objFstOrderSymbol Is IMarkerSymbol Then
                    strRenderer.FeatureCls = FeatureClass.PointFeature
                    Dim objSymbol As IMarkerSymbol
                    objSymbol = objFstOrderSymbol
                    Select Case MarkerSymbolScan(objSymbol)
                        Case "ISimpleMarkerSymbol"
                            Dim SMS As ISimpleMarkerSymbol
                            SMS = objSymbol
                            Dim strSMS As StructSimpleMarkerSymbol
                            strSMS = StoreSimpleMarker(SMS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strSMS.Label = Renderer.Label(j)
                            strSMS.LowerLimit = cLowerLimit
                            strSMS.UpperLimit = cUpperLimit
                            strRenderer.SymbolList.Add(strSMS)

                        Case "ICharacterMarkerSymbol"
                            Dim CMS As ICharacterMarkerSymbol
                            CMS = objSymbol
                            Dim strCMS As StructCharacterMarkerSymbol
                            strCMS = StoreCharacterMarker(CMS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strCMS.Label = Renderer.Label(j)
                            strCMS.LowerLimit = cLowerLimit
                            strCMS.UpperLimit = cUpperLimit
                            strRenderer.SymbolList.Add(strCMS)

                        Case "IPictureMarkerSymbol"
                            Dim PMS As IPictureMarkerSymbol
                            PMS = objSymbol
                            Dim strPMS As StructPictureMarkerSymbol
                            strPMS = StorePictureMarker(PMS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strPMS.Label = Renderer.Label(j)
                            strPMS.LowerLimit = cLowerLimit
                            strPMS.UpperLimit = cUpperLimit
                            strRenderer.SymbolList.Add(strPMS)

                        Case "IArrowMarkerSymbol"
                            Dim AMS As IArrowMarkerSymbol
                            AMS = objSymbol
                            Dim strAMS As StructArrowMarkerSymbol
                            strAMS = StoreArrowMarker(AMS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strAMS.Label = Renderer.Label(j)
                            strAMS.LowerLimit = cLowerLimit
                            strAMS.UpperLimit = cUpperLimit
                            strRenderer.SymbolList.Add(strAMS)

                        Case "IMultiLayerMarkerSymbol"
                            Dim MLMS As IMultiLayerMarkerSymbol
                            MLMS = objSymbol
                            Dim strMLMS As StructMultilayerMarkerSymbol
                            strMLMS = StoreMultiLayerMarker(MLMS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strMLMS.Label = Renderer.Label(j)
                            strMLMS.LowerLimit = cLowerLimit
                            strMLMS.UpperLimit = cUpperLimit
                            strRenderer.SymbolList.Add(strMLMS)

                        Case "false"
                            InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreStructLayer")
                    End Select
                End If
                'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
                If TypeOf objFstOrderSymbol Is ILineSymbol Then
                    strRenderer.FeatureCls = FeatureClass.LineFeature
                    Dim objSymbol As ILineSymbol
                    objSymbol = objFstOrderSymbol
                    Select Case LineSymbolScan(objSymbol)
                        Case "ICartographicLineSymbol"
                            Dim CLS As ICartographicLineSymbol
                            CLS = objSymbol
                            Dim strCLS As StructCartographicLineSymbol
                            strCLS = StoreCartographicLine(CLS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strCLS.Label = Renderer.Label(j)
                            strCLS.LowerLimit = cLowerLimit
                            strCLS.UpperLimit = cUpperLimit
                            strRenderer.SymbolList.Add(strCLS)

                        Case "IHashLineSymbol"
                            Dim HLS As IHashLineSymbol
                            HLS = objSymbol
                            Dim strHLS As StructHashLineSymbol
                            strHLS = StoreHashLine(HLS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strHLS.Label = Renderer.Label(j)
                            strHLS.LowerLimit = cLowerLimit
                            strHLS.UpperLimit = cUpperLimit
                            strRenderer.SymbolList.Add(strHLS)

                        Case "IMarkerLineSymbol"
                            Dim MLS As IMarkerLineSymbol
                            MLS = objSymbol
                            Dim strMLS As StructMarkerLineSymbol
                            strMLS = StoreMarkerLine(MLS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strMLS.Label = Renderer.Label(j)
                            strMLS.LowerLimit = cLowerLimit
                            strMLS.UpperLimit = cUpperLimit
                            strRenderer.SymbolList.Add(strMLS)

                        Case "ISimpleLineSymbol"
                            Dim SLS As ISimpleLineSymbol
                            SLS = objSymbol
                            Dim strSLS As StructSimpleLineSymbol
                            strSLS = StoreSimpleLine(SLS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strSLS.Label = Renderer.Label(j)
                            strSLS.LowerLimit = cLowerLimit
                            strSLS.UpperLimit = cUpperLimit
                            strRenderer.SymbolList.Add(strSLS)

                        Case "IPictureLineSymbol"
                            Dim PLS As IPictureLineSymbol
                            PLS = objSymbol
                            Dim strPLS As StructPictureLineSymbol
                            strPLS = StorePictureLine(PLS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strPLS.Label = Renderer.Label(j)
                            strPLS.LowerLimit = cLowerLimit
                            strPLS.UpperLimit = cUpperLimit
                            strRenderer.SymbolList.Add(strPLS)

                        Case "IMultiLayerLineSymbol"
                            Dim MLLS As IMultiLayerLineSymbol
                            MLLS = objSymbol
                            Dim strMLLS As StructMultilayerLineSymbol
                            strMLLS = StoreMultilayerLines(MLLS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strMLLS.Label = Renderer.Label(j)
                            strMLLS.LowerLimit = cLowerLimit
                            strMLLS.UpperLimit = cUpperLimit
                            strRenderer.SymbolList.Add(strMLLS)

                        Case "false"
                            InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreStructLayer")
                    End Select

                End If
                'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
                If TypeOf objFstOrderSymbol Is IFillSymbol Then
                    strRenderer.FeatureCls = FeatureClass.PolygonFeature
                    Dim objSymbol As IFillSymbol
                    objSymbol = objFstOrderSymbol
                    Select Case FillSymbolScan(objSymbol)
                        Case "ISimpleFillSymbol"
                            Dim SFS As ISimpleFillSymbol
                            SFS = objSymbol
                            Dim strSFS As StructSimpleFillSymbol
                            strSFS = StoreSimpleFill(SFS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strSFS.Label = Renderer.Label(j)
                            strSFS.LowerLimit = cLowerLimit
                            strSFS.UpperLimit = cUpperLimit
                            strRenderer.SymbolList.Add(strSFS)

                        Case "IMarkerFillSymbol"
                            Dim MFS As IMarkerFillSymbol
                            MFS = objSymbol
                            Dim strMFS As StructMarkerFillSymbol
                            strMFS = StoreMarkerFill(MFS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strMFS.Label = Renderer.Label(j)
                            strMFS.LowerLimit = cLowerLimit
                            strMFS.UpperLimit = cUpperLimit
                            strRenderer.SymbolList.Add(strMFS)

                        Case "ILineFillSymbol"
                            Dim LFS As ILineFillSymbol
                            LFS = objSymbol
                            Dim strLFS As StructLineFillSymbol
                            strLFS = StoreLineFill(LFS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strLFS.Label = Renderer.Label(j)
                            strLFS.LowerLimit = cLowerLimit
                            strLFS.UpperLimit = cUpperLimit
                            strRenderer.SymbolList.Add(strLFS)

                        Case "IDotDensityFillSymbol"
                            Dim DFS As IDotDensityFillSymbol
                            DFS = objSymbol
                            Dim strDFS As StructDotDensityFillSymbol
                            strDFS = StoreDotDensityFill(DFS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strDFS.Label = Renderer.Label(j)
                            strDFS.LowerLimit = cLowerLimit
                            strDFS.UpperLimit = cUpperLimit
                            strRenderer.SymbolList.Add(strDFS)

                        Case "IPictureFillSymbol"
                            Dim PFS As IPictureFillSymbol
                            PFS = objSymbol
                            Dim strPFS As StructPictureFillSymbol
                            strPFS = StorePictureFill(PFS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strPFS.Label = Renderer.Label(j)
                            strPFS.LowerLimit = cLowerLimit
                            strPFS.UpperLimit = cUpperLimit
                            strRenderer.SymbolList.Add(strPFS)

                        Case "IGradientFillSymbol"
                            Dim GFS As IGradientFillSymbol
                            GFS = objSymbol
                            Dim strGFS As StructGradientFillSymbol
                            strGFS = StoreGradientFill(GFS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strGFS.Label = Renderer.Label(j)
                            strGFS.LowerLimit = cLowerLimit
                            strGFS.UpperLimit = cUpperLimit
                            strRenderer.SymbolList.Add(strGFS)

                        Case "IMultiLayerFillSymbol"
                            Dim MLFS As IMultiLayerFillSymbol
                            MLFS = objSymbol
                            Dim strMLFS As StructMultilayerFillSymbol
                            strMLFS = StoreMultiLayerFill(MLFS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strMLFS.Label = Renderer.Label(j)
                            strMLFS.LowerLimit = cLowerLimit
                            strMLFS.UpperLimit = cUpperLimit
                            strRenderer.SymbolList.Add(strMLFS)

                        Case "false"
                            InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreStructLayer")
                    End Select
                End If
                'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
                If TypeOf objFstOrderSymbol Is I3DChartSymbol Then
                    Dim objSymbol As I3DChartSymbol
                    objSymbol = objFstOrderSymbol
                    Select Case IIIDChartSymbolScan(objSymbol)
                        Case "IBarChartSymbol"
                            Dim BCS As IBarChartSymbol
                            BCS = objSymbol
                            Dim strBCS As StructBarChartSymbol
                            strBCS = StoreBarChart(BCS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strBCS.Label = Renderer.Label(j)
                            strBCS.LowerLimit = cLowerLimit
                            strBCS.UpperLimit = cUpperLimit
                            strRenderer.SymbolList.Add(strBCS)

                        Case "IPieChartSymbol"
                            Dim PCS As IPieChartSymbol
                            PCS = objSymbol
                            Dim strPCS As StructPieChartSymbol
                            strPCS = StorePieChart(PCS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strPCS.Label = Renderer.Label(j)
                            strPCS.LowerLimit = cLowerLimit
                            strPCS.UpperLimit = cUpperLimit
                            strRenderer.SymbolList.Add(strPCS)

                        Case "IStackedChartSymbol"
                            Dim SCS As IStackedChartSymbol
                            SCS = objSymbol
                            Dim strSCS As StructStackedChartSymbol
                            strSCS = StoreStackedChart(SCS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strSCS.Label = Renderer.Label(j)
                            strSCS.LowerLimit = cLowerLimit
                            strSCS.UpperLimit = cUpperLimit
                            strRenderer.SymbolList.Add(strSCS)

                        Case "false"
                            InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreStructLayer")
                    End Select
                End If
                'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
            Next
            Return strRenderer
        Catch ex As Exception

            ErrorMsg("Fehler beim Speichern der Symbole in den Layerstrukturen", ex.Message, ex.StackTrace, "StoreStructCBRenderer")
        End Try
    End Function

    '************************************************************************************************
    'Die Funktion durchluft die die diversen Symbolarten, verteilt sie auf die jeweiligen Unter-
    'symbolarten, und speichert diese dann in der Datenstruktur StructUniqueValueRenderer ab.
    'In der Funktion werden Daten auf 2 Ebenen gespeichert: Auf Rendererebene (die ersten Speicherungen);
    'und auf Symbolebene (die Fallunterscheidungen der einzelnen Symbolarten)
    'Parameter: Renderer: das aktuelle Rendererobjekt. Pro Layer eines
    '           Layer: das aktuelle Layerobjekt
    '************************************************************************************************
    Private Function StoreStructUVRenderer(ByVal Renderer As IUniqueValueRenderer, ByVal Layer As IFeatureLayer) As StructUniqueValueRenderer
        Dim strRenderer As New StructUniqueValueRenderer     'Hierin wird das eine Rendererobjekt gespeichert plus zustzliche Layerinformationen
        Dim iNumberOfSymbols As Integer     'Anzahl aller Symbole des Rendererobjekts
        iNumberOfSymbols = Renderer.ValueCount  'Anzahl der Symbole
        Dim objFstOrderSymbol As ISymbol    'Das gerade aktuelle Symbol des durchlaufenen Rendererobjekts
        Dim alFieldNames As New ArrayList 'Die ArrayLisit mit den Feldnamen
        Dim bNoSepFieldVal As Boolean    'Wenn nur nach einem Feld klassifiziert wurde muss der Flag auf true gesetzt werden
        Dim objTable As ITable  'Der FeatureTable, der an den Layer gebunden ist
        Dim objDataset As IDataset  'Der aktuelle Dataset
        objTable = Layer.FeatureClass
        objDataset = objTable
        strRenderer.SymbolList = New ArrayList
        bNoSepFieldVal = False  '!!!! Damit nicht in die aufwendige Funktion GimmeSeperateFieldValues(..) gegangen werden mu, wenn nicht ntig
        m_alClassifiedFields = New ArrayList 'enthlt jeweils wiederum arraylist(s) von den normalisierten Werten aus den Feldern, nach denen klassifiziert wurde
        Dim bIsJoined As Boolean
        bIsJoined = False


        Try
            strRenderer.ValueCount = iNumberOfSymbols
            strRenderer.LayerName = Layer.Name
            strRenderer.DatasetName = objDataset.Name
            strRenderer.Annotation = GetAnnotation(Layer)
            '++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            'Hier werden die Informationen ber Felder nach denen klassifiziert wurde gesammelt
            Dim iFieldCount As Integer  'Anzahl der Spalten, nach denen klassifiziert wurde
            iFieldCount = Renderer.FieldCount
            strRenderer.FieldCount = iFieldCount
            If iFieldCount > 1 Then   '!!!! Damit nicht in die aufwendige Funktion GimmeSeperateFieldValues(..) gegangen werden mu, wenn nicht ntig
                bNoSepFieldVal = True   'und damit nicht die aufwndigen UniqueValue-Listen erzeugt werden mssen, wenn nicht notwendig
            End If

            'diese Objekte dienen lediglich der berprfung, ob eine andere Tabelle an die Featuretable gejoint ist
            '++++++++++++++++++++++++++++++++++++++++++
            Dim pTable As ITable
            Dim pDisplayTable As IDisplayTable
            pDisplayTable = Layer
            pTable = pDisplayTable.DisplayTable
            If TypeOf pTable Is IRelQueryTable Then 'Dann ist eine Tabelle drangejoint
                bIsJoined = True
            End If
            '++++++++++++++++++++++++++++++++++++++++++

            '++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
            'Das ganze nachfolgende Konstrukt dient nur dem Zweck, Unique-Value-Listen aus den Datenbanktabellen zu erstellen,
            'um spter mit der Funktion GimmeSeperateFieldValues die separaten Spaltenwerte, die zu einem Symbol gehren, zu erhalten,
            'wenn (und nur wenn) nach mehr als 1 Wert klassifiziert wurde (da esri in diesem Fall nur einen aus allen Feldwerten 
            'zusammengesetzten String liefert)
            Dim i As Integer
            If bNoSepFieldVal = True Then     'Feldnamen (Spaltennamen) werden nur dann gespeichert, wenn nach mehr als 1 Feld klassifiziert wurde. (da sehr aufwndige Sache!)

                'Wenn der aktuelle Layer aus einem Shapefile stammt, ist das weitere Vorgehen 
                'unterschiedlich zu dem Fall, wenn er aus DB's stammt:
                If objDataset.Workspace.Type = esriWorkspaceType.esriFileSystemWorkspace Then
                    For i = 1 To iFieldCount
                        alFieldNames.Add(Renderer.Field(i - 1)) 'Die Spaltennamen werden alle abgespeichert
                    Next
                    GimmeUniqueValuesFromShape(objTable, alFieldNames)
                    'GimmeUniqueValuesFromShape(Layer, alFieldNames)
                    strRenderer.FieldNames = alFieldNames
                Else
                    For i = 1 To iFieldCount
                        alFieldNames.Add(Renderer.Field(i - 1)) 'Die Spaltennamen werden alle abgespeichert
                        'wenn eine andere Tabelle an die Featuretable gejoint ist
                        If TypeOf pTable Is IRelQueryTable Then
                            ' ++ Get the list of joined tables
                            Dim pRelQueryTable As IRelQueryTable
                            Dim pDestTable As ITable
                            Dim pDataSet As IDataset
                            'Dim cTable As String
                            Dim alJoinedTableNames As New ArrayList
                            Do While TypeOf pTable Is IRelQueryTable
                                pRelQueryTable = pTable
                                pDestTable = pRelQueryTable.DestinationTable
                                pDataSet = pDestTable
                                'cTable = cTable & pDataSet.Name
                                pTable = pRelQueryTable.SourceTable
                                alJoinedTableNames.Add(pDataSet.Name)
                            Loop
                            GimmeUniqeValuesForFieldname(objTable, Renderer.Field(i - 1), alJoinedTableNames) 'Hier wird die Funktion aufgerufen, um die Unique Values des jew. Feldes abzuspeichern 
                            pTable = pDisplayTable.DisplayTable 'Zurcksetzen des pTable, damit beim nchsten Durchlauf wieder abgefragt werden kann
                        Else 'Bei nichtgejointer Tabelle  natrlich Aufruf der Funktion ohne die gejointen Tabellennamen
                            GimmeUniqeValuesForFieldname(objTable, Renderer.Field(i - 1)) 'Hier wird die Funktion aufgerufen, um die Unique Values des jew. Feldes abzuspeichern 
                        End If
                    Next
                    strRenderer.FieldNames = alFieldNames
                End If
            Else
                alFieldNames.Add(Renderer.Field(iFieldCount - 1))
                strRenderer.FieldNames = alFieldNames
            End If

            'Je nachdem, welcher Zielkartendienst gewhlt wurde, muss Fieldname ein anderes Format besitzen
            If bIsJoined = False Then      'Wenn eine Tabelle drangejoint wurde, ist sowieso schon der Datasetname mit dabei
                Dim idummy As Integer
                If frmMotherform.chkArcIMS.Checked = True Then
                    For i = 0 To strRenderer.FieldNames.Count - 1
                        strRenderer.FieldNames(i) = objDataset.Name & "." & strRenderer.FieldNames(i)
                    Next
                End If
            End If


            '++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

            'AB HIER BEGINNEN DIE FALLUNTERSCHEIDUNGEN DER SYMBOLE
            Dim j As Integer
            For j = 0 To iNumberOfSymbols - 1
                frmMotherform.CHLabelSmall("Symbol " & (j + 1).ToString & " von " & iNumberOfSymbols.ToString) 'Anzeige auf dem Label
                objFstOrderSymbol = Renderer.Symbol(Renderer.Value(j))  'Die Zuweisung der jeweiligen einzelnen Symbole
                'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
                If TypeOf objFstOrderSymbol Is ITextSymbol Then
                    Dim strTS As StructTextSymbol
                    Dim objSymbol As ITextSymbol
                    objSymbol = objFstOrderSymbol
                    strTS = StoreText(objSymbol)
                    '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                    strTS.Label = Renderer.Label(Renderer.Value(j))
                    strTS.Fieldvalues = getUVFieldValues(Renderer, j)
                    strRenderer.SymbolList.Add(strTS)
                End If
                'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx()
                If TypeOf objFstOrderSymbol Is IMarkerSymbol Then
                    strRenderer.FeatureCls = FeatureClass.PointFeature
                    Dim objSymbol As IMarkerSymbol
                    objSymbol = objFstOrderSymbol
                    Select Case MarkerSymbolScan(objSymbol)
                        Case "ISimpleMarkerSymbol"
                            Dim SMS As ISimpleMarkerSymbol
                            SMS = objSymbol
                            Dim strSMS As StructSimpleMarkerSymbol
                            strSMS = StoreSimpleMarker(SMS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strSMS.Label = Renderer.Label(Renderer.Value(j))
                            strSMS.Fieldvalues = getUVFieldValues(Renderer, j)
                            strRenderer.SymbolList.Add(strSMS)

                        Case "ICharacterMarkerSymbol"
                            Dim CMS As ICharacterMarkerSymbol
                            CMS = objSymbol
                            Dim strCMS As StructCharacterMarkerSymbol
                            strCMS = StoreCharacterMarker(CMS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strCMS.Label = Renderer.Label(Renderer.Value(j))
                            strCMS.Fieldvalues = getUVFieldValues(Renderer, j)
                            strRenderer.SymbolList.Add(strCMS)

                        Case "IPictureMarkerSymbol"
                            Dim PMS As IPictureMarkerSymbol
                            PMS = objSymbol
                            Dim strPMS As StructPictureMarkerSymbol
                            strPMS = StorePictureMarker(PMS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strPMS.Label = Renderer.Label(Renderer.Value(j))
                            strPMS.Fieldvalues = getUVFieldValues(Renderer, j)
                            strRenderer.SymbolList.Add(strPMS)

                        Case "IArrowMarkerSymbol"
                            Dim AMS As IArrowMarkerSymbol
                            AMS = objSymbol
                            Dim strAMS As StructArrowMarkerSymbol
                            strAMS = StoreArrowMarker(AMS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strAMS.Label = Renderer.Label(Renderer.Value(j))
                            strAMS.Fieldvalues = getUVFieldValues(Renderer, j)
                            strRenderer.SymbolList.Add(strAMS)

                        Case "IMultiLayerMarkerSymbol"
                            Dim MLMS As IMultiLayerMarkerSymbol
                            MLMS = objSymbol
                            Dim strMLMS As StructMultilayerMarkerSymbol
                            strMLMS = StoreMultiLayerMarker(MLMS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strMLMS.Label = Renderer.Label(Renderer.Value(j))
                            strMLMS.Fieldvalues = getUVFieldValues(Renderer, j)
                            strRenderer.SymbolList.Add(strMLMS)

                        Case "false"
                            InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreStructLayer")
                    End Select
                End If
                'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
                If TypeOf objFstOrderSymbol Is ILineSymbol Then
                    strRenderer.FeatureCls = FeatureClass.LineFeature
                    Dim objSymbol As ILineSymbol
                    objSymbol = objFstOrderSymbol
                    Select Case LineSymbolScan(objSymbol)
                        Case "ICartographicLineSymbol"
                            Dim CLS As ICartographicLineSymbol
                            CLS = objSymbol
                            Dim strCLS As StructCartographicLineSymbol
                            strCLS = StoreCartographicLine(CLS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strCLS.Label = Renderer.Label(Renderer.Value(j))
                            strCLS.Fieldvalues = getUVFieldValues(Renderer, j)
                            strRenderer.SymbolList.Add(strCLS)

                        Case "IHashLineSymbol"
                            Dim HLS As IHashLineSymbol
                            HLS = objSymbol
                            Dim strHLS As StructHashLineSymbol
                            strHLS = StoreHashLine(HLS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strHLS.Label = Renderer.Label(Renderer.Value(j))
                            strHLS.Fieldvalues = getUVFieldValues(Renderer, j)
                            strRenderer.SymbolList.Add(strHLS)

                        Case "IMarkerLineSymbol"
                            Dim MLS As IMarkerLineSymbol
                            MLS = objSymbol
                            Dim strMLS As StructMarkerLineSymbol
                            strMLS = StoreMarkerLine(MLS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strMLS.Label = Renderer.Label(Renderer.Value(j))
                            strMLS.Fieldvalues = getUVFieldValues(Renderer, j)
                            strRenderer.SymbolList.Add(strMLS)

                        Case "ISimpleLineSymbol"
                            Dim SLS As ISimpleLineSymbol
                            SLS = objSymbol
                            Dim strSLS As StructSimpleLineSymbol
                            strSLS = StoreSimpleLine(SLS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strSLS.Label = Renderer.Label(Renderer.Value(j))
                            strSLS.Fieldvalues = getUVFieldValues(Renderer, j)
                            strRenderer.SymbolList.Add(strSLS)

                        Case "IPictureLineSymbol"
                            Dim PLS As IPictureLineSymbol
                            PLS = objSymbol
                            Dim strPLS As StructPictureLineSymbol
                            strPLS = StorePictureLine(PLS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strPLS.Label = Renderer.Label(Renderer.Value(j))
                            strPLS.Fieldvalues = getUVFieldValues(Renderer, j)
                            strRenderer.SymbolList.Add(strPLS)

                        Case "IMultiLayerLineSymbol"
                            Dim MLLS As IMultiLayerLineSymbol
                            MLLS = objSymbol
                            Dim strMLLS As StructMultilayerLineSymbol
                            strMLLS = StoreMultilayerLines(MLLS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strMLLS.Label = Renderer.Label(Renderer.Value(j))
                            strMLLS.Fieldvalues = getUVFieldValues(Renderer, j)
                            strRenderer.SymbolList.Add(strMLLS)

                        Case "false"
                            InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreStructLayer")
                    End Select

                End If
                'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
                If TypeOf objFstOrderSymbol Is IFillSymbol Then
                    strRenderer.FeatureCls = FeatureClass.PolygonFeature
                    Dim objSymbol As IFillSymbol
                    objSymbol = objFstOrderSymbol
                    Select Case FillSymbolScan(objSymbol)
                        Case "ISimpleFillSymbol"
                            Dim SFS As ISimpleFillSymbol
                            SFS = objSymbol
                            Dim strSFS As StructSimpleFillSymbol
                            strSFS = StoreSimpleFill(SFS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strSFS.Label = Renderer.Label(Renderer.Value(j))
                            strSFS.Fieldvalues = getUVFieldValues(Renderer, j)
                            strRenderer.SymbolList.Add(strSFS)

                        Case "IMarkerFillSymbol"
                            Dim MFS As IMarkerFillSymbol
                            MFS = objSymbol
                            Dim strMFS As StructMarkerFillSymbol
                            strMFS = StoreMarkerFill(MFS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strMFS.Label = Renderer.Label(Renderer.Value(j))
                            strMFS.Fieldvalues = getUVFieldValues(Renderer, j)
                            strRenderer.SymbolList.Add(strMFS)

                        Case "ILineFillSymbol"
                            Dim LFS As ILineFillSymbol
                            LFS = objSymbol
                            Dim strLFS As StructLineFillSymbol
                            strLFS = StoreLineFill(LFS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strLFS.Label = Renderer.Label(Renderer.Value(j))
                            strLFS.Fieldvalues = getUVFieldValues(Renderer, j)
                            strRenderer.SymbolList.Add(strLFS)

                        Case "IDotDensityFillSymbol"
                            Dim DFS As IDotDensityFillSymbol
                            DFS = objSymbol
                            Dim strDFS As StructDotDensityFillSymbol
                            strDFS = StoreDotDensityFill(DFS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strDFS.Label = Renderer.Label(Renderer.Value(j))
                            strDFS.Fieldvalues = getUVFieldValues(Renderer, j)
                            strRenderer.SymbolList.Add(strDFS)

                        Case "IPictureFillSymbol"
                            Dim PFS As IPictureFillSymbol
                            PFS = objSymbol
                            Dim strPFS As StructPictureFillSymbol
                            strPFS = StorePictureFill(PFS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strPFS.Label = Renderer.Label(Renderer.Value(j))
                            strPFS.Fieldvalues = getUVFieldValues(Renderer, j)
                            strRenderer.SymbolList.Add(strPFS)

                        Case "IGradientFillSymbol"
                            Dim GFS As IGradientFillSymbol
                            GFS = objSymbol
                            Dim strGFS As StructGradientFillSymbol
                            strGFS = StoreGradientFill(GFS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strGFS.Label = Renderer.Label(Renderer.Value(j))
                            strGFS.Fieldvalues = getUVFieldValues(Renderer, j)
                            strRenderer.SymbolList.Add(strGFS)

                        Case "IMultiLayerFillSymbol"
                            Dim MLFS As IMultiLayerFillSymbol
                            MLFS = objSymbol
                            Dim strMLFS As StructMultilayerFillSymbol
                            strMLFS = StoreMultiLayerFill(MLFS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strMLFS.Label = Renderer.Label(Renderer.Value(j))
                            strMLFS.Fieldvalues = getUVFieldValues(Renderer, j)
                            strRenderer.SymbolList.Add(strMLFS)

                        Case "false"
                            InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreStructLayer")
                    End Select
                End If
                'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
                If TypeOf objFstOrderSymbol Is I3DChartSymbol Then
                    Dim objSymbol As I3DChartSymbol
                    objSymbol = objFstOrderSymbol
                    Select Case IIIDChartSymbolScan(objSymbol)
                        Case "IBarChartSymbol"
                            Dim BCS As IBarChartSymbol
                            BCS = objSymbol
                            Dim strBCS As StructBarChartSymbol
                            strBCS = StoreBarChart(BCS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strBCS.Label = Renderer.Label(Renderer.Value(j))
                            strBCS.Fieldvalues = getUVFieldValues(Renderer, j)
                            strRenderer.SymbolList.Add(strBCS)

                        Case "IPieChartSymbol"
                            Dim PCS As IPieChartSymbol
                            PCS = objSymbol
                            Dim strPCS As StructPieChartSymbol
                            strPCS = StorePieChart(PCS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strPCS.Label = Renderer.Label(Renderer.Value(j))
                            strPCS.Fieldvalues = getUVFieldValues(Renderer, j)
                            strRenderer.SymbolList.Add(strPCS)

                        Case "IStackedChartSymbol"
                            Dim SCS As IStackedChartSymbol
                            SCS = objSymbol
                            Dim strSCS As StructStackedChartSymbol
                            strSCS = StoreStackedChart(SCS)
                            '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                            strSCS.Label = Renderer.Label(Renderer.Value(j))
                            strSCS.Fieldvalues = getUVFieldValues(Renderer, j)
                            strRenderer.SymbolList.Add(strSCS)

                        Case "false"
                            InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreStructLayer")
                    End Select
                End If
                'xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
            Next
            Return strRenderer
        Catch ex As Exception
            ErrorMsg("Fehler beim Speichern der Symbole in den Layerstrukturen", ex.Message, ex.StackTrace, "StoreStructUVRenderer")
        End Try
    End Function

    '************************************************************************************************
    'For Unique Value renderers get the field value or values for the given value index.
    'With multiple fields, the result is a list of the values for each of the fields.
    'With a single field, the result is a list with either the single value, or a
    'list with multiple values if several of the next value indices belong to the
    'same group.
    '************************************************************************************************
    Private Function getUVFieldValues(ByVal Renderer As IUniqueValueRenderer, ByVal Index As Integer) As ArrayList
        'Es macht nur Sinn hier Fieldvalues einzufgen, wenn berhaupt nach einem Feld klassifiziert wurde
        Dim Fieldvalues As ArrayList
        Dim iFieldCount As Integer  'Anzahl der Spalten, nach denen klassifiziert wurde
        Dim Index2 As Integer

        iFieldCount = Renderer.FieldCount
        Fieldvalues = Nothing
        If iFieldCount > 0 Then
            Dim bNoSepFieldVal As Boolean    'Wenn nur nach einem Feld klassifiziert wurde muss der Flag auf true gesetzt werden
            Dim Label As String
            Dim Label2 As String
            Dim objSymbol As ISymbol
            Dim iNumberOfSymbols As Integer     'Anzahl aller Symbole des Rendererobjekts
            iNumberOfSymbols = Renderer.ValueCount  'Anzahl der Symbole
            bNoSepFieldVal = False
            If iFieldCount > 1 Then   '!!!! Damit nicht in die aufwendige Funktion GimmeSeperateFieldValues(..) gegangen werden mu, wenn nicht ntig
                bNoSepFieldVal = True   'und damit nicht die aufwndigen UniqueValue-Listen erzeugt werden mssen, wenn nicht notwendig
            End If

            Label = Renderer.Label(Renderer.Value(Index))
            If bNoSepFieldVal = False Then
                Fieldvalues = New ArrayList
                Fieldvalues.Add(Renderer.Value(Index))    'Wenn nur nach 1 Feld klassifiziert wurde, muss nicht in GimmeSeperateFieldValues gegangen werden (Zeit!!!)
                'Find grouped values with the same label (values where no symbol defined).
                Index2 = Index + 1
                While Index2 < iNumberOfSymbols
                    objSymbol = Renderer.Symbol(Renderer.Value(Index2))
                    Label2 = Renderer.Label(Renderer.Value(Index2))
                    If objSymbol Is Nothing And Label = Label2 Then
                        Fieldvalues.Add(Renderer.Value(Index2))
                    Else
                        Exit While
                    End If
                    Index2 = Index2 + 1
                End While
            Else
                Fieldvalues = GimmeSeperateFieldValues(Renderer.Value(Index), Renderer.FieldDelimiter)
            End If
        End If
        Return Fieldvalues
    End Function

    '************************************************************************************************
    'Die Funktion speichert die SimpleMarker in seiner Datenstruktur
    '************************************************************************************************
    Private Function StoreSimpleMarker(ByVal symbol As ISimpleMarkerSymbol) As StructSimpleMarkerSymbol
        Dim StructStorage As New StructSimpleMarkerSymbol
        StructStorage.Angle = symbol.Angle
        StructStorage.Filled = symbol.Color.Transparency <> 0
        StructStorage.Color = GimmeStringForColor(symbol.Color)
        StructStorage.Outline = symbol.Outline
        StructStorage.OutlineColor = GimmeStringForColor(symbol.OutlineColor)
        StructStorage.OutlineSize = symbol.OutlineSize
        StructStorage.Size = symbol.Size
        StructStorage.Style = symbol.Style.ToString
        StructStorage.XOffset = symbol.XOffset
        StructStorage.YOffset = symbol.YOffset
        Return StructStorage
    End Function


    '************************************************************************************************
    'Die Funktion speichert die CharacterMarker in seiner Datenstruktur
    '************************************************************************************************
    Private Function StoreCharacterMarker(ByVal symbol As ICharacterMarkerSymbol) As StructCharacterMarkerSymbol
        Dim StructStorage As New StructCharacterMarkerSymbol
        StructStorage.Angle = symbol.Angle
        StructStorage.CharacterIndex = symbol.CharacterIndex
        StructStorage.Color = GimmeStringForColor(symbol.Color)
        StructStorage.Font = symbol.Font.Name
        StructStorage.Size = symbol.Size
        StructStorage.XOffset = symbol.XOffset
        StructStorage.YOffset = symbol.YOffset
        Return StructStorage
    End Function

    '************************************************************************************************
    'Die Funktion speichert die PictureMarker in seiner Datenstruktur
    '************************************************************************************************
    Private Function StorePictureMarker(ByVal symbol As IPictureMarkerSymbol) As StructPictureMarkerSymbol
        Dim StructStorage As New StructPictureMarkerSymbol
        StructStorage.Angle = symbol.Angle
        StructStorage.BackgroundColor = GimmeStringForColor(symbol.BackgroundColor)
        StructStorage.Color = GimmeStringForColor(symbol.Color)
        StructStorage.Picture = symbol.Picture
        StructStorage.Size = symbol.Size
        StructStorage.XOffset = symbol.XOffset
        StructStorage.YOffset = symbol.YOffset
        Return StructStorage
    End Function

    '************************************************************************************************
    'Die Funktion speichert die ArrowMarker in seiner Datenstruktur
    '************************************************************************************************
    Private Function StoreArrowMarker(ByVal symbol As IArrowMarkerSymbol) As StructArrowMarkerSymbol
        Dim StructStorage As New StructArrowMarkerSymbol
        StructStorage.Angle = symbol.Angle
        StructStorage.Color = GimmeStringForColor(symbol.Color)
        StructStorage.Length = symbol.Length
        StructStorage.Size = symbol.Size
        StructStorage.Style = symbol.Style
        StructStorage.Width = symbol.Width
        StructStorage.XOffset = symbol.XOffset
        StructStorage.YOffset = symbol.YOffset
        Return StructStorage
    End Function

    '************************************************************************************************
    'Die Funktion speichert die MultiLayerMarker in ArrayList. Wieviel Layer wird in LayerCount gesp.
    'Das MultilayerMarker-Symbol kann nicht selbst nocheinmal aus einem MultilayerMarker bestehen
    '************************************************************************************************
    Private Function StoreMultiLayerMarker(ByVal symbol As IMultiLayerMarkerSymbol) As StructMultilayerMarkerSymbol
        Dim StructStorage As New StructMultilayerMarkerSymbol
        StructStorage.MultiMarkerLayers = New ArrayList
        Dim i As Integer
        StructStorage.LayerCount = symbol.LayerCount
        For i = 0 To symbol.LayerCount - 1  'Damit alle Layer erfasst werden
            Select Case MarkerSymbolScan(symbol.Layer(i))   'Das Multilayersymbol besteht aus mehreren Layern, die alle durchlaufen werden mssen
                Case "ISimpleMarkerSymbol"
                    Dim SMS As ISimpleMarkerSymbol
                    SMS = symbol.Layer(i)
                    StructStorage.MultiMarkerLayers.Add(StoreSimpleMarker(SMS)) 'Der Layer wird der Arraylist von Multilayern hinzugefgt
                Case "ICharacterMarkerSymbol"
                    Dim CMS As ICharacterMarkerSymbol
                    CMS = symbol.Layer(i)
                    StructStorage.MultiMarkerLayers.Add(StoreCharacterMarker(CMS)) 'Der Layer wird der Arraylist von Multilayern hinzugefgt
                Case "IPictureMarkerSymbol"
                    Dim PMS As IPictureMarkerSymbol
                    PMS = symbol.Layer(i)
                    StructStorage.MultiMarkerLayers.Add(StorePictureMarker(PMS)) 'Der Layer wird der Arraylist von Multilayern hinzugefgt
                Case "IArrowMarkerSymbol"
                    Dim AMS As IArrowMarkerSymbol
                    AMS = symbol.Layer(i)
                    StructStorage.MultiMarkerLayers.Add(StoreArrowMarker(AMS)) 'Der Layer wird der Arraylist von Multilayern hinzugefgt
                Case "false"
                    InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreMultiLayerMarker")
            End Select
        Next
        Return StructStorage
    End Function
    '_______________________________________________________________________________________________________________________

    '************************************************************************************************
    'Die Funktion speichert die SimpleLines in seiner Datenstruktur
    '************************************************************************************************
    Private Function StoreSimpleLine(ByVal symbol As ISimpleLineSymbol) As StructSimpleLineSymbol
        Dim StructStorage As New StructSimpleLineSymbol
        If (symbol.Style = esriSimpleLineStyle.esriSLSNull) Then
            StructStorage.Color = ""
        Else
            StructStorage.Color = GimmeStringForColor(symbol.Color)
        End If
        StructStorage.Transparency = symbol.Color.Transparency
        StructStorage.publicStyle = symbol.Style
        StructStorage.Width = symbol.Width
        Return StructStorage
    End Function

    '************************************************************************************************
    'Die Funktion speichert die SimpleLines in seiner Datenstruktur
    '************************************************************************************************
    Private Function StoreCartographicLine(ByVal symbol As ICartographicLineSymbol) As StructCartographicLineSymbol
        Dim StructStorage As New StructCartographicLineSymbol
        StructStorage.Color = GimmeStringForColor(symbol.Color)
        StructStorage.Transparency = symbol.Color.Transparency
        StructStorage.Width = symbol.Width
        StructStorage.Join = symbol.Join
        StructStorage.MiterLimit = symbol.MiterLimit
        StructStorage.Cap = symbol.Cap
        StructStorage.DashArray = New ArrayList()
        If TypeOf symbol Is ILineProperties Then
            Dim lineProperties As ILineProperties
            Dim template As ITemplate
            Dim markLen As Double
            Dim gapLen As Double
            Dim interval As Double
            Dim templateIdx As Integer
            lineProperties = symbol
            If TypeOf lineProperties.Template Is ITemplate Then
                template = lineProperties.Template
                interval = template.Interval
                For templateIdx = 0 To template.PatternElementCount - 1
                    template.GetPatternElement(templateIdx, markLen, gapLen)
                    StructStorage.DashArray.Add(markLen * interval)
                    StructStorage.DashArray.Add(gapLen * interval)
                Next
            End If
        End If
        Return StructStorage
    End Function

    '************************************************************************************************
    'Die Funktion speichert die HashLines in seiner Datenstruktur
    '************************************************************************************************
    Private Function StoreHashLine(ByVal symbol As IHashLineSymbol) As StructHashLineSymbol
        Dim StructStorage As New StructHashLineSymbol
        StructStorage.Angle = symbol.Angle
        StructStorage.Color = GimmeStringForColor(symbol.Color)
        StructStorage.Transparency = symbol.Color.Transparency
        Select Case LineSymbolScan(symbol.HashSymbol)   'symbol.HashSymbol ist ein Liniensymbol. deshalb bei case... kein IHashSymbol/StructHashLineSymbol
            Case "ICartographicLineSymbol"
                Dim CLS As ICartographicLineSymbol
                CLS = symbol.HashSymbol
                StructStorage.HashSymbol_CartographicLine = StoreCartographicLine(CLS)
                StructStorage.kindOfLineStruct = LineStructs.StructCartographicLineSymbol
            Case "IMarkerLineSymbol"
                Dim MLS As IMarkerLineSymbol
                MLS = symbol.HashSymbol
                StructStorage.HashSymbol_MarkerLine = StoreMarkerLine(MLS)
                StructStorage.kindOfLineStruct = LineStructs.StructMarkerLineSymbol
            Case "ISimpleLineSymbol"
                Dim SLS As ISimpleLineSymbol
                SLS = symbol.HashSymbol
                StructStorage.HashSymbol_SimpleLine = StoreSimpleLine(SLS)
                StructStorage.kindOfLineStruct = LineStructs.StructSimpleLineSymbol
            Case "IPictureLineSymbol"
                Dim PLS As IPictureLineSymbol
                PLS = symbol.HashSymbol
                StructStorage.HashSymbol_PictureLine = StorePictureLine(PLS)
                StructStorage.kindOfLineStruct = LineStructs.StructPictureLineSymbol
            Case "IMultiLayerLineSymbol"
                Dim MLLS As IMultiLayerLineSymbol
                MLLS = symbol.HashSymbol
                StructStorage.HashSymbol_MultiLayerLines = StoreMultilayerLines(MLLS)
                StructStorage.kindOfLineStruct = LineStructs.StructMultilayerLineSymbol
            Case "false"
                InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreHashLine")
        End Select
        Return StructStorage
    End Function

    '************************************************************************************************
    'Die Funktion speichert die MarkerLines in seiner Datenstruktur
    '************************************************************************************************
    Private Function StoreMarkerLine(ByVal symbol As IMarkerLineSymbol) As StructMarkerLineSymbol
        Dim StructStorage As New StructMarkerLineSymbol
        StructStorage.Color = GimmeStringForColor(symbol.Color)
        StructStorage.Transparency = symbol.Color.Transparency
        StructStorage.Width = symbol.Width
        Select Case MarkerSymbolScan(symbol.MarkerSymbol)
            Case "ISimpleMarkerSymbol"
                Dim SMS As ISimpleMarkerSymbol
                SMS = symbol.MarkerSymbol
                StructStorage.MarkerSymbol_SimpleMarker = StoreSimpleMarker(SMS)
                StructStorage.kindOfMarkerStruct = MarkerStructs.StructSimpleMarkerSymbol
            Case "ICharacterMarkerSymbol"
                Dim CMS As ICharacterMarkerSymbol
                CMS = symbol.MarkerSymbol
                StructStorage.MarkerSymbol_CharacterMarker = StoreCharacterMarker(CMS)
                StructStorage.kindOfMarkerStruct = MarkerStructs.StructCharacterMarkerSymbol
            Case "IPictureMarkerSymbol"
                Dim PMS As IPictureMarkerSymbol
                PMS = symbol.MarkerSymbol
                StructStorage.MarkerSymbol_PictureMarker = StorePictureMarker(PMS)
                StructStorage.kindOfMarkerStruct = MarkerStructs.StructPictureMarkerSymbol
            Case "IArrowMarkerSymbol"
                Dim AMS As IArrowMarkerSymbol
                AMS = symbol.MarkerSymbol
                StructStorage.MarkerSymbol_ArrowMarker = StoreArrowMarker(AMS)
                StructStorage.kindOfMarkerStruct = MarkerStructs.StructArrowMarkerSymbol
            Case "IMultiLayerMarkerSymbol"
                Dim MLMS As IMultiLayerMarkerSymbol
                MLMS = symbol.MarkerSymbol
                StructStorage.MarkerSymbol_MultilayerMarker = StoreMultiLayerMarker(MLMS)
                StructStorage.kindOfMarkerStruct = MarkerStructs.StructMultilayerMarkerSymbol
            Case "false"
                InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreMarkerLine")
        End Select
        Return StructStorage
    End Function

    '************************************************************************************************
    'Die Funktion speichert die PictureLines in seiner Datenstruktur
    '************************************************************************************************
    Private Function StorePictureLine(ByVal symbol As IPictureLineSymbol) As StructPictureLineSymbol
        Dim StructStorage As New StructPictureLineSymbol
        StructStorage.BackgroundColor = GimmeStringForColor(symbol.BackgroundColor)
        StructStorage.BackgroundTransparency = symbol.BackgroundColor.Transparency
        StructStorage.Color = GimmeStringForColor(symbol.Color)
        StructStorage.Transparency = symbol.Color.Transparency
        StructStorage.Offset = symbol.Offset
        StructStorage.Picture = symbol.Picture
        StructStorage.Rotate = symbol.Rotate
        StructStorage.Width = symbol.Width
        StructStorage.XScale = symbol.XScale
        StructStorage.YScale = symbol.YScale
        Return StructStorage
    End Function

    '************************************************************************************************
    'Die Funktion speichert die MultilayerLines in seiner Datenstruktur
    '************************************************************************************************
    Private Function StoreMultilayerLines(ByVal symbol As IMultiLayerLineSymbol) As StructMultilayerLineSymbol
        Dim StructStorage As New StructMultilayerLineSymbol
        StructStorage.MultiLineLayers = New ArrayList
        Dim i As Integer
        StructStorage.LayerCount = symbol.LayerCount
        For i = 0 To symbol.LayerCount - 1
            Select Case LineSymbolScan(symbol.Layer(i))
                Case "ICartographicLineSymbol"
                    Dim CLS As ICartographicLineSymbol
                    CLS = symbol.Layer(i)
                    StructStorage.MultiLineLayers.Add(StoreCartographicLine(CLS))
                Case "IHashLineSymbol"
                    Dim HLS As IHashLineSymbol
                    HLS = symbol.Layer(i)
                    StructStorage.MultiLineLayers.Add(StoreHashLine(HLS))
                Case "IMarkerLineSymbol"
                    Dim MLS As IMarkerLineSymbol
                    MLS = symbol.Layer(i)
                    StructStorage.MultiLineLayers.Add(StoreMarkerLine(MLS))
                Case "ISimpleLineSymbol"
                    Dim SLS As ISimpleLineSymbol
                    SLS = symbol.Layer(i)
                    StructStorage.MultiLineLayers.Add(StoreSimpleLine(SLS))
                Case "IPictureLineSymbol"
                    Dim PLS As IPictureLineSymbol
                    PLS = symbol.Layer(i)
                    StructStorage.MultiLineLayers.Add(StorePictureLine(PLS))
                Case "IMultiLayerLineSymbol"
                    Dim MLLS As IMultiLayerLineSymbol
                    MLLS = symbol.Layer(i)
                    StructStorage.MultiLineLayers.Add(StoreMultilayerLines(MLLS))   'Hier ist ein rekursiver Aufruf
                Case "false"
                    InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreMultilayerLines")
            End Select
        Next
        Return StructStorage
    End Function

    '_______________________________________________________________________________________________________________________


    '************************************************************************************************
    'Die Funktion speichert die SimpleFills in seiner Datenstruktur
    '************************************************************************************************
    Private Function StoreSimpleFill(ByVal symbol As ISimpleFillSymbol) As StructSimpleFillSymbol
        Dim StructStorage As New StructSimpleFillSymbol
        If (symbol.Style = esriSimpleFillStyle.esriSFSHollow) Then
            StructStorage.Color = ""
        Else
            StructStorage.Color = GimmeStringForColor(symbol.Color)
        End If
        StructStorage.Style = symbol.Style
        StructStorage.Transparency = symbol.Color.Transparency
        Select Case LineSymbolScan(symbol.Outline)   'symbol.Outline ist ein Liniensymbol
            Case "ICartographicLineSymbol"
                Dim CLS As ICartographicLineSymbol
                CLS = symbol.Outline
                StructStorage.Outline_CartographicLine = StoreCartographicLine(CLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructCartographicLineSymbol
            Case "IMarkerLineSymbol"
                Dim MLS As IMarkerLineSymbol
                MLS = symbol.Outline
                StructStorage.Outline_MarkerLine = StoreMarkerLine(MLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructMarkerLineSymbol
            Case "IHashLineSymbol"
                Dim HLS As IHashLineSymbol
                HLS = symbol.Outline
                StructStorage.Outline_HashLine = StoreHashLine(HLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructHashLineSymbol
            Case "ISimpleLineSymbol"
                Dim SLS As ISimpleLineSymbol
                SLS = symbol.Outline
                StructStorage.Outline_SimpleLine = StoreSimpleLine(SLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructSimpleLineSymbol
            Case "IPictureLineSymbol"
                Dim PLS As IPictureLineSymbol
                PLS = symbol.Outline
                StructStorage.Outline_PictureLine = StorePictureLine(PLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructPictureLineSymbol
            Case "IMultiLayerLineSymbol"
                Dim MLLS As IMultiLayerLineSymbol
                MLLS = symbol.Outline
                StructStorage.Outline_MultiLayerLines = StoreMultilayerLines(MLLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructMultilayerLineSymbol
            Case "false"
                InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreSimpleFill")
        End Select
        Return StructStorage
    End Function


    '************************************************************************************************
    'Die Funktion speichert die MarkerFillSymbols in seiner Datenstruktur
    '************************************************************************************************
    Private Function StoreMarkerFill(ByVal symbol As IMarkerFillSymbol) As StructMarkerFillSymbol
        Dim StructStorage As New StructMarkerFillSymbol
        StructStorage.Color = GimmeStringForColor(symbol.Color)
        StructStorage.Transparency = symbol.Color.Transparency
        StructStorage.GridAngle = symbol.GridAngle
        Select Case MarkerSymbolScan(symbol.MarkerSymbol)
            Case "ISimpleMarkerSymbol"
                Dim SMS As ISimpleMarkerSymbol
                SMS = symbol.MarkerSymbol
                StructStorage.MarkerSymbol_SimpleMarker = StoreSimpleMarker(SMS)
                StructStorage.kindOfMarkerStruct = MarkerStructs.StructSimpleMarkerSymbol
            Case "ICharacterMarkerSymbol"
                Dim CMS As ICharacterMarkerSymbol
                CMS = symbol.MarkerSymbol
                StructStorage.MarkerSymbol_CharacterMarker = StoreCharacterMarker(CMS)
                StructStorage.kindOfMarkerStruct = MarkerStructs.StructCharacterMarkerSymbol
            Case "IPictureMarkerSymbol"
                Dim PMS As IPictureMarkerSymbol
                PMS = symbol.MarkerSymbol
                StructStorage.MarkerSymbol_PictureMarker = StorePictureMarker(PMS)
                StructStorage.kindOfMarkerStruct = MarkerStructs.StructPictureMarkerSymbol
            Case "IArrowMarkerSymbol"
                Dim AMS As IArrowMarkerSymbol
                AMS = symbol.MarkerSymbol
                StructStorage.MarkerSymbol_ArrowMarker = StoreArrowMarker(AMS)
                StructStorage.kindOfMarkerStruct = MarkerStructs.StructArrowMarkerSymbol
            Case "IMultiLayerMarkerSymbol"
                Dim MLMS As IMultiLayerMarkerSymbol
                MLMS = symbol.MarkerSymbol
                StructStorage.MarkerSymbol_MultilayerMarker = StoreMultiLayerMarker(MLMS)
                StructStorage.kindOfMarkerStruct = MarkerStructs.StructMultilayerMarkerSymbol
            Case "false"
                InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreMarkerFill")
        End Select
        Select Case LineSymbolScan(symbol.Outline)
            Case "ICartographicLineSymbol"
                Dim CLS As ICartographicLineSymbol
                CLS = symbol.Outline
                StructStorage.Outline_CartographicLine = StoreCartographicLine(CLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructCartographicLineSymbol
            Case "IMarkerLineSymbol"
                Dim MLS As IMarkerLineSymbol
                MLS = symbol.Outline
                StructStorage.Outline_MarkerLine = StoreMarkerLine(MLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructMarkerLineSymbol
            Case "IHashLineSymbol"
                Dim HLS As IHashLineSymbol
                HLS = symbol.Outline
                StructStorage.Outline_HashLine = StoreHashLine(HLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructHashLineSymbol
            Case "ISimpleLineSymbol"
                Dim SLS As ISimpleLineSymbol
                SLS = symbol.Outline
                StructStorage.Outline_SimpleLine = StoreSimpleLine(SLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructSimpleLineSymbol
            Case "IPictureLineSymbol"
                Dim PLS As IPictureLineSymbol
                PLS = symbol.Outline
                StructStorage.Outline_PictureLine = StorePictureLine(PLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructPictureLineSymbol
            Case "IMultiLayerLineSymbol"
                Dim MLLS As IMultiLayerLineSymbol
                MLLS = symbol.Outline
                StructStorage.Outline_MultiLayerLines = StoreMultilayerLines(MLLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructMultilayerLineSymbol
            Case "false"
                InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreMarkerFill")
        End Select
        Return StructStorage
    End Function

    '************************************************************************************************
    'Die Funktion speichert die LineFillSymbols in seiner Datenstruktur
    '************************************************************************************************
    Private Function StoreLineFill(ByVal symbol As ILineFillSymbol) As StructLineFillSymbol
        Dim StructStorage As New StructLineFillSymbol
        StructStorage.Angle = symbol.Angle
        StructStorage.Color = GimmeStringForColor(symbol.Color)
        StructStorage.Transparency = symbol.Color.Transparency
        StructStorage.Offset = symbol.Offset
        StructStorage.Separation = symbol.Separation
        Select Case LineSymbolScan(symbol.LineSymbol)
            Case "ICartographicLineSymbol"
                Dim CLS As ICartographicLineSymbol
                CLS = symbol.LineSymbol
                StructStorage.LineSymbol_CartographicLine = StoreCartographicLine(CLS)
                StructStorage.kindOfLineStruct = LineStructs.StructCartographicLineSymbol
            Case "IMarkerLineSymbol"
                Dim MLS As IMarkerLineSymbol
                MLS = symbol.LineSymbol
                StructStorage.LineSymbol_MarkerLine = StoreMarkerLine(MLS)
                StructStorage.kindOfLineStruct = LineStructs.StructMarkerLineSymbol
            Case "IHashLineSymbol"
                Dim HLS As IHashLineSymbol
                HLS = symbol.LineSymbol
                StructStorage.LineSymbol_HashLine = StoreHashLine(HLS)
                StructStorage.kindOfLineStruct = LineStructs.StructHashLineSymbol
            Case "ISimpleLineSymbol"
                Dim SLS As ISimpleLineSymbol
                SLS = symbol.LineSymbol
                StructStorage.LineSymbol_SimpleLine = StoreSimpleLine(SLS)
                StructStorage.kindOfLineStruct = LineStructs.StructSimpleLineSymbol
            Case "IPictureLineSymbol"
                Dim PLS As IPictureLineSymbol
                PLS = symbol.LineSymbol
                StructStorage.LineSymbol_PictureLine = StorePictureLine(PLS)
                StructStorage.kindOfLineStruct = LineStructs.StructPictureLineSymbol
            Case "IMultiLayerLineSymbol"
                Dim MLLS As IMultiLayerLineSymbol
                MLLS = symbol.LineSymbol
                StructStorage.LineSymbol_MultiLayerLines = StoreMultilayerLines(MLLS)
                StructStorage.kindOfLineStruct = LineStructs.StructMultilayerLineSymbol
            Case "false"
                InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreLineFill")
        End Select
        Select Case LineSymbolScan(symbol.Outline)
            Case "ICartographicLineSymbol"
                Dim CLS As ICartographicLineSymbol
                CLS = symbol.Outline
                StructStorage.Outline_CartographicLine = StoreCartographicLine(CLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructCartographicLineSymbol
            Case "IMarkerLineSymbol"
                Dim MLS As IMarkerLineSymbol
                MLS = symbol.Outline
                StructStorage.Outline_MarkerLine = StoreMarkerLine(MLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructMarkerLineSymbol
            Case "IHashLineSymbol"
                Dim HLS As IHashLineSymbol
                HLS = symbol.Outline
                StructStorage.Outline_HashLine = StoreHashLine(HLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructHashLineSymbol
            Case "ISimpleLineSymbol"
                Dim SLS As ISimpleLineSymbol
                SLS = symbol.Outline
                StructStorage.Outline_SimpleLine = StoreSimpleLine(SLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructSimpleLineSymbol
            Case "IPictureLineSymbol"
                Dim PLS As IPictureLineSymbol
                PLS = symbol.Outline
                StructStorage.Outline_PictureLine = StorePictureLine(PLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructPictureLineSymbol
            Case "IMultiLayerLineSymbol"
                Dim MLLS As IMultiLayerLineSymbol
                MLLS = symbol.Outline
                StructStorage.Outline_MultiLayerLines = StoreMultilayerLines(MLLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructMultilayerLineSymbol
            Case "false"
                InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreLineFill")
        End Select
        Return StructStorage
    End Function

    '************************************************************************************************
    'Die Funktion speichert die DotDensityFillSymbols in seiner Datenstruktur
    '************************************************************************************************
    Private Function StoreDotDensityFill(ByVal symbol As IDotDensityFillSymbol) As StructDotDensityFillSymbol
        Dim StructStorage As New StructDotDensityFillSymbol
        Dim objSymbolArray As ISymbolArray
        StructStorage.SymbolList = New ArrayList
        objSymbolArray = symbol
        Dim i As Integer
        StructStorage.BackgroundColor = GimmeStringForColor(symbol.BackgroundColor)
        StructStorage.BackgroundTransparency = symbol.BackgroundColor.Transparency
        StructStorage.Color = GimmeStringForColor(symbol.Color)
        StructStorage.Transparency = symbol.Color.Transparency
        StructStorage.FixedPlacement = symbol.FixedPlacement
        StructStorage.DotSpacing = symbol.DotSpacing
        StructStorage.SymbolCount = objSymbolArray.SymbolCount

        For i = 0 To objSymbolArray.SymbolCount - 1
            If TypeOf objSymbolArray.Symbol(i) Is IMarkerSymbol Then   'Nur der Marker macht hier als Symbol berhaupt Sinn
                Dim MS As IMarkerSymbol
                MS = objSymbolArray.Symbol(i)
                '!!!ACHTUNG!!! In der ArrayList wird immer abwechselnd ein Symbol und danach die zugehrige Symbolanzahl
                'abgespeichert. Das ist nicht elegant aber spter geschickter beim auslesen!
                StructStorage.SymbolList = New ArrayList
                Select Case MarkerSymbolScan(MS)
                    Case "ISimpleMarkerSymbol"
                        Dim SMS As ISimpleMarkerSymbol
                        SMS = MS
                        StructStorage.SymbolList.Add(StoreSimpleMarker(SMS))
                        StructStorage.SymbolList.Add(symbol.DotCount(i))
                    Case "ICharacterMarkerSymbol"
                        Dim CMS As ICharacterMarkerSymbol
                        CMS = MS
                        StructStorage.SymbolList.Add(StoreCharacterMarker(CMS))
                        StructStorage.SymbolList.Add(symbol.DotCount(i))
                    Case "IPictureMarkerSymbol"
                        Dim PMS As IPictureMarkerSymbol
                        PMS = MS
                        StructStorage.SymbolList.Add(StorePictureMarker(PMS))
                        StructStorage.SymbolList.Add(symbol.DotCount(i))
                    Case "IArrowMarkerSymbol"
                        Dim AMS As IArrowMarkerSymbol
                        AMS = MS
                        StructStorage.SymbolList.Add(StoreArrowMarker(AMS))
                        StructStorage.SymbolList.Add(symbol.DotCount(i))
                    Case "IMultiLayerMarkerSymbol"
                        Dim MLMS As IMultiLayerMarkerSymbol
                        MLMS = MS
                        StructStorage.SymbolList.Add(StoreMultiLayerMarker(MLMS))
                        StructStorage.SymbolList.Add(symbol.DotCount(i))
                    Case "false"
                        InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreDotDensityFill")
                End Select
            End If
        Next

        Select Case LineSymbolScan(symbol.Outline)
            Case "ICartographicLineSymbol"
                Dim CLS As ICartographicLineSymbol
                CLS = symbol.Outline
                StructStorage.Outline_CartographicLine = StoreCartographicLine(CLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructCartographicLineSymbol
            Case "IMarkerLineSymbol"
                Dim MLS As IMarkerLineSymbol
                MLS = symbol.Outline
                StructStorage.Outline_MarkerLine = StoreMarkerLine(MLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructMarkerLineSymbol
            Case "IHashLineSymbol"
                Dim HLS As IHashLineSymbol
                HLS = symbol.Outline
                StructStorage.Outline_HashLine = StoreHashLine(HLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructHashLineSymbol
            Case "ISimpleLineSymbol"
                Dim SLS As ISimpleLineSymbol
                SLS = symbol.Outline
                StructStorage.Outline_SimpleLine = StoreSimpleLine(SLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructSimpleLineSymbol
            Case "IPictureLineSymbol"
                Dim PLS As IPictureLineSymbol
                PLS = symbol.Outline
                StructStorage.Outline_PictureLine = StorePictureLine(PLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructPictureLineSymbol
            Case "IMultiLayerLineSymbol"
                Dim MLLS As IMultiLayerLineSymbol
                MLLS = symbol.Outline
                StructStorage.Outline_MultiLayerLines = StoreMultilayerLines(MLLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructMultilayerLineSymbol
            Case "false"
                InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreLineFill")
        End Select

        Return StructStorage
    End Function

    '************************************************************************************************
    'Die Funktion speichert die PictureFillSymbols in seiner Datenstruktur
    '************************************************************************************************
    Private Function StorePictureFill(ByVal symbol As IPictureFillSymbol) As StructPictureFillSymbol
        Dim StructStorage As New StructPictureFillSymbol
        StructStorage.Angle = symbol.Angle
        StructStorage.BackgroundColor = Me.GimmeStringForColor(symbol.BackgroundColor)
        StructStorage.BackgroundTransparency = symbol.BackgroundColor.Transparency
        StructStorage.Color = Me.GimmeStringForColor(symbol.Color)
        StructStorage.Transparency = symbol.Color.Transparency
        'objPicture = Microsoft.VisualBasic.Compatibility.VB6.IPictureDispToImage(symbol.Picture) 'doesn't work with esri
        'StructStorage.Picture = symbol.Picture.
        StructStorage.XScale = symbol.XScale
        StructStorage.YScale = symbol.YScale
        Select Case LineSymbolScan(symbol.Outline)
            Case "ICartographicLineSymbol"
                Dim CLS As ICartographicLineSymbol
                CLS = symbol.Outline
                StructStorage.Outline_CartographicLine = StoreCartographicLine(CLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructCartographicLineSymbol
            Case "IMarkerLineSymbol"
                Dim MLS As IMarkerLineSymbol
                MLS = symbol.Outline
                StructStorage.Outline_MarkerLine = StoreMarkerLine(MLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructMarkerLineSymbol
            Case "IHashLineSymbol"
                Dim HLS As IHashLineSymbol
                HLS = symbol.Outline
                StructStorage.Outline_HashLine = StoreHashLine(HLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructHashLineSymbol
            Case "ISimpleLineSymbol"
                Dim SLS As ISimpleLineSymbol
                SLS = symbol.Outline
                StructStorage.Outline_SimpleLine = StoreSimpleLine(SLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructSimpleLineSymbol
            Case "IPictureLineSymbol"
                Dim PLS As IPictureLineSymbol
                PLS = symbol.Outline
                StructStorage.Outline_PictureLine = StorePictureLine(PLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructPictureLineSymbol
            Case "IMultiLayerLineSymbol"
                Dim MLLS As IMultiLayerLineSymbol
                MLLS = symbol.Outline
                StructStorage.Outline_MultiLayerLines = StoreMultilayerLines(MLLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructMultilayerLineSymbol
            Case "false"
                InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StorePictureFill")
        End Select
        Return StructStorage
    End Function

    '************************************************************************************************
    'Die Funktion speichert die GradientFillSymbols in seiner Datenstruktur
    '************************************************************************************************
    Private Function StoreGradientFill(ByVal symbol As IGradientFillSymbol) As StructGradientFillSymbol
        Dim StructStorage As New StructGradientFillSymbol
        StructStorage.Color = GimmeStringForColor(symbol.Color)
        StructStorage.Transparency = symbol.Color.Transparency
        StructStorage.Colors = GimmeArrayListForColorRamp(symbol.ColorRamp)
        StructStorage.GradientAngle = symbol.GradientAngle
        StructStorage.GradientPercentage = symbol.GradientPercentage
        StructStorage.IntervallCount = symbol.IntervalCount
        Select Case LineSymbolScan(symbol.Outline)
            Case "ICartographicLineSymbol"
                Dim CLS As ICartographicLineSymbol
                CLS = symbol.Outline
                StructStorage.Outline_CartographicLine = StoreCartographicLine(CLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructCartographicLineSymbol
            Case "IMarkerLineSymbol"
                Dim MLS As IMarkerLineSymbol
                MLS = symbol.Outline
                StructStorage.Outline_MarkerLine = StoreMarkerLine(MLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructMarkerLineSymbol
            Case "IHashLineSymbol"
                Dim HLS As IHashLineSymbol
                HLS = symbol.Outline
                StructStorage.Outline_HashLine = StoreHashLine(HLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructHashLineSymbol
            Case "ISimpleLineSymbol"
                Dim SLS As ISimpleLineSymbol
                SLS = symbol.Outline
                StructStorage.Outline_SimpleLine = StoreSimpleLine(SLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructSimpleLineSymbol
            Case "IPictureLineSymbol"
                Dim PLS As IPictureLineSymbol
                PLS = symbol.Outline
                StructStorage.Outline_PictureLine = StorePictureLine(PLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructPictureLineSymbol
            Case "IMultiLayerLineSymbol"
                Dim MLLS As IMultiLayerLineSymbol
                MLLS = symbol.Outline
                StructStorage.Outline_MultiLayerLines = StoreMultilayerLines(MLLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructMultilayerLineSymbol
            Case "false"
                InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StorePictureFill")
        End Select
        Return StructStorage
    End Function

    '************************************************************************************************
    'Die Funktion speichert die MultiLayerFillSymbols in seiner Datenstruktur
    '************************************************************************************************
    Private Function StoreMultiLayerFill(ByVal symbol As IMultiLayerFillSymbol) As StructMultilayerFillSymbol
        Dim StructStorage As New StructMultilayerFillSymbol
        StructStorage.LayerCount = symbol.LayerCount
        StructStorage.MultiFillLayers = New ArrayList
        Dim i As Integer
        For i = 0 To symbol.LayerCount - 1
            Select Case FillSymbolScan(symbol.Layer(i))
                Case "ISimpleFillSymbol"
                    Dim SFS As ISimpleFillSymbol
                    SFS = symbol.Layer(i)
                    StructStorage.MultiFillLayers.Add(StoreSimpleFill(SFS))
                Case "IMarkerFillSymbol"
                    Dim MFS As IMarkerFillSymbol
                    MFS = symbol.Layer(i)
                    StructStorage.MultiFillLayers.Add(StoreMarkerFill(MFS))
                Case "ILineFillSymbol"
                    Dim LFS As ILineFillSymbol
                    LFS = symbol.Layer(i)
                    StructStorage.MultiFillLayers.Add(StoreLineFill(LFS))
                Case "IPictureFillSymbol"
                    Dim PFS As IPictureFillSymbol
                    PFS = symbol.Layer(i)
                    StructStorage.MultiFillLayers.Add(StorePictureFill(PFS))
                Case "IDotDensityFillSymbol"
                    Dim DFS As IDotDensityFillSymbol
                    DFS = symbol.Layer(i)
                    StructStorage.MultiFillLayers.Add(StoreDotDensityFill(DFS))
                Case "IGradientFillSymbol"
                    Dim GFS As IGradientFillSymbol
                    GFS = symbol.Layer(i)
                    StructStorage.MultiFillLayers.Add(StoreGradientFill(GFS))
                Case "IMultiLayerFillSymbol"
                    Dim MLFS As IMultiLayerFillSymbol
                    MLFS = symbol
                    StructStorage.MultiFillLayers.Add(StoreMultiLayerFill(MLFS)) 'Hier ist ein rekursiver Aufruf
                Case "false"
                    InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreMultilayerFill")
            End Select
        Next
        Return StructStorage
    End Function

    '_______________________________________________________________________________________________________________________

    '************************************************************************************************
    'Die Funktion speichert die BarChartSymbols in seiner Datenstruktur
    '************************************************************************************************
    Private Function StoreBarChart(ByVal symbol As IBarChartSymbol) As StructBarChartSymbol
        Dim StructStorage As New StructBarChartSymbol
        StructStorage.ShowAxes = symbol.ShowAxes
        StructStorage.Spacing = symbol.Spacing
        StructStorage.VerticalBars = symbol.VerticalBars
        StructStorage.Width = symbol.Width
        Select Case LineSymbolScan(symbol.Axes)
            Case "ICartographicLineSymbol"
                Dim CLS As ICartographicLineSymbol
                CLS = symbol.Axes
                StructStorage.Axes_CartographicLine = StoreCartographicLine(CLS)
                StructStorage.kindOfAxeslineStruct = LineStructs.StructCartographicLineSymbol
            Case "IMarkerLineSymbol"
                Dim MLS As IMarkerLineSymbol
                MLS = symbol.Axes
                StructStorage.Axes_MarkerLine = StoreMarkerLine(MLS)
                StructStorage.kindOfAxeslineStruct = LineStructs.StructMarkerLineSymbol
            Case "IHashLineSymbol"
                Dim HLS As IHashLineSymbol
                HLS = symbol.Axes
                StructStorage.Axes_HashLine = StoreHashLine(HLS)
                StructStorage.kindOfAxeslineStruct = LineStructs.StructHashLineSymbol
            Case "ISimpleLineSymbol"
                Dim SLS As ISimpleLineSymbol
                SLS = symbol.Axes
                StructStorage.Axes_SimpleLine = StoreSimpleLine(SLS)
                StructStorage.kindOfAxeslineStruct = LineStructs.StructSimpleLineSymbol
            Case "IPictureLineSymbol"
                Dim PLS As IPictureLineSymbol
                PLS = symbol.Axes
                StructStorage.Axes_PictureLine = StorePictureLine(PLS)
                StructStorage.kindOfAxeslineStruct = LineStructs.StructPictureLineSymbol
            Case "IMultiLayerLineSymbol"
                Dim MLLS As IMultiLayerLineSymbol
                MLLS = symbol.Axes
                StructStorage.Axes_MultiLayerLines = StoreMultilayerLines(MLLS)
                StructStorage.kindOfAxeslineStruct = LineStructs.StructMultilayerLineSymbol
            Case "false"
                InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StoreBarChart")
        End Select
        Return StructStorage
    End Function

    '************************************************************************************************
    'Die Funktion speichert die PieChartSymbols in seiner Datenstruktur
    '************************************************************************************************
    Private Function StorePieChart(ByVal symbol As IPieChartSymbol) As StructPieChartSymbol
        Dim StructStorage As New StructPieChartSymbol
        StructStorage.Clockwise = symbol.Clockwise
        StructStorage.UseOutline = symbol.UseOutline
        Select Case LineSymbolScan(symbol.Outline)
            Case "ICartographicLineSymbol"
                Dim CLS As ICartographicLineSymbol
                CLS = symbol.Outline
                StructStorage.Outline_CartographicLine = StoreCartographicLine(CLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructCartographicLineSymbol
            Case "IMarkerLineSymbol"
                Dim MLS As IMarkerLineSymbol
                MLS = symbol.Outline
                StructStorage.Outline_MarkerLine = StoreMarkerLine(MLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructMarkerLineSymbol
            Case "IHashLineSymbol"
                Dim HLS As IHashLineSymbol
                HLS = symbol.Outline
                StructStorage.Outline_HashLine = StoreHashLine(HLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructHashLineSymbol
            Case "ISimpleLineSymbol"
                Dim SLS As ISimpleLineSymbol
                SLS = symbol.Outline
                StructStorage.Outline_SimpleLine = StoreSimpleLine(SLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructSimpleLineSymbol
            Case "IPictureLineSymbol"
                Dim PLS As IPictureLineSymbol
                PLS = symbol.Outline
                StructStorage.Outline_PictureLine = StorePictureLine(PLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructPictureLineSymbol
            Case "IMultiLayerLineSymbol"
                Dim MLLS As IMultiLayerLineSymbol
                MLLS = symbol.Outline
                StructStorage.Outline_MultiLayerLines = StoreMultilayerLines(MLLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructMultilayerLineSymbol
            Case "false"
                InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StorePictureFill")
        End Select
        Return StructStorage
    End Function

    '************************************************************************************************
    'Die Funktion speichert die StackedChartSymbols in seiner Datenstruktur
    '************************************************************************************************
    Private Function StoreStackedChart(ByVal symbol As IStackedChartSymbol) As StructStackedChartSymbol
        Dim StructStorage As New StructStackedChartSymbol
        StructStorage.Fixed = symbol.Fixed
        StructStorage.UseOutline = symbol.UseOutline
        StructStorage.VerticalBar = symbol.VerticalBar
        StructStorage.Width = symbol.Width
        Select Case LineSymbolScan(symbol.Outline)
            Case "ICartographicLineSymbol"
                Dim CLS As ICartographicLineSymbol
                CLS = symbol.Outline
                StructStorage.Outline_CartographicLine = StoreCartographicLine(CLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructCartographicLineSymbol
            Case "IMarkerLineSymbol"
                Dim MLS As IMarkerLineSymbol
                MLS = symbol.Outline
                StructStorage.Outline_MarkerLine = StoreMarkerLine(MLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructMarkerLineSymbol
            Case "IHashLineSymbol"
                Dim HLS As IHashLineSymbol
                HLS = symbol.Outline
                StructStorage.Outline_HashLine = StoreHashLine(HLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructHashLineSymbol
            Case "ISimpleLineSymbol"
                Dim SLS As ISimpleLineSymbol
                SLS = symbol.Outline
                StructStorage.Outline_SimpleLine = StoreSimpleLine(SLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructSimpleLineSymbol
            Case "IPictureLineSymbol"
                Dim PLS As IPictureLineSymbol
                PLS = symbol.Outline
                StructStorage.Outline_PictureLine = StorePictureLine(PLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructPictureLineSymbol
            Case "IMultiLayerLineSymbol"
                Dim MLLS As IMultiLayerLineSymbol
                MLLS = symbol.Outline
                StructStorage.Outline_MultiLayerLines = StoreMultilayerLines(MLLS)
                StructStorage.kindOfOutlineStruct = LineStructs.StructMultilayerLineSymbol
            Case "false"
                InfoMsg("Seit Erstellen der Programmversion ist eine neue Symbolvariante zu den esri-Symbolen hinzugekommen", "StorePictureFill")
        End Select
        Return StructStorage
    End Function

    '_______________________________________________________________________________________________________________________

    '************************************************************************************************
    'Die Funktion speichert TextSymbol in seiner Datenstruktur
    '************************************************************************************************
    Private Function StoreText(ByVal symbol As ITextSymbol) As StructTextSymbol
        Dim StructStorage As New StructTextSymbol
        StructStorage.Angle = symbol.Angle
        StructStorage.Color = GimmeStringForColor(symbol.Color)
        StructStorage.Font = symbol.Font.Name
        StructStorage.Style = "normal"
        If symbol.Font.Italic Then
            StructStorage.Style = "italic"
        End If
        StructStorage.Weight = "normal"
        If symbol.Font.Bold Then
            StructStorage.Weight = "bold"
        End If
        StructStorage.HorizontalAlignment = symbol.HorizontalAlignment
        StructStorage.RightToLeft = symbol.RightToLeft
        StructStorage.Size = symbol.Size
        StructStorage.Text = symbol.Text
        StructStorage.VerticalAlignment = symbol.VerticalAlignment
        Return StructStorage
    End Function

#End Region   'Speicherungen auf Symbolebene und auf Rendererebene

    '************************************************************************************************
    'Die Funktion steuert die Prozesse zentral. Sie ist somit die erste Funktion in dieser Klasse
    'Wird von public Sub New() aufgerufen
    '************************************************************************************************
    Private Function CentralProcessingFunc() As Boolean
        If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
            frmMotherform.CHLabelTop("Die Analyse des ArcMap-Projekts luft")
        ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
            frmMotherform.CHLabelTop("Analysis of the ArcMap-Project is running")
        End If

        Dim oFile As File

        If GetProcesses() = False Then
            MyTermination()
            Return False
        End If
        If GetApplication() = False Then
            MyTermination()
            Return False
        End If
        If GetMap() = False Then
            MyTermination()
            Return False
        End If
        If AnalyseLayerSymbology() = False Then
            MyTermination()
            Return False
        End If
        If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
            'Sicherheitsabfrage, falls noch kein Filename fr das SLD angegeben wurde. Dann Aufruf der Ausgabeklasse
            If m_cFilename Is Nothing Or m_cFilename = "" Then
                If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
                    frmMotherform.CHLabelTop("Die Analyse des ArcMap-Projekts ist beendet")
                ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
                    frmMotherform.CHLabelTop("Analysis of the ArcMap-Project has finished")
                End If
                If MessageBox.Show("Sie haben noch keinen Dateinamen/Speicherort angegeben. Wenn Sie jetzt keinen Dateinamen angeben" & _
                ",wird die Anwendung beendet." & vbCrLf & "Wollen Sie jetzt einen Dateinamen angeben?", "ArcGIS_SLD_Converter | " & _
                "Analize_ArcMap_Symbols | CentralProcessingFunc", MessageBoxButtons.YesNo, MessageBoxIcon.Information) = DialogResult.Yes Then
                    If File.Exists(frmMotherform.GetSLDFileFromConfigXML) Then
                        frmMotherform.dlgSave.InitialDirectory = frmMotherform.GetSLDFileFromConfigXML
                    End If
                    Dim swOutput As IO.StreamWriter
                    If frmMotherform.dlgSave.ShowDialog = DialogResult.OK Then
                        With frmMotherform.dlgSave
                            .CheckFileExists = False
                            .CheckPathExists = True
                            .DefaultExt = "sld"
                            .Filter = "SLD-files (*.sld)|*.sld"
                            .AddExtension = True
                            .InitialDirectory = IO.Path.GetDirectoryName(m_cFilename)
                            .OverwritePrompt = True
                            .CreatePrompt = False
                            If .ShowDialog() = DialogResult.OK Then
                                m_cFilename = .FileName
                                frmMotherform.txtFileName.Text = m_cFilename
                            End If
                        End With
                        Dim objOutputSLD As New Output_SLD(frmMotherform, Me, m_cFilename)  'Aufruf der Ausgabeklasse
                    Else
                        MyTermination()
                    End If
                Else
                    MyTermination()
                End If
            Else
                Dim objOutputSLD As New Output_SLD(frmMotherform, Me, m_cFilename)  'Aufruf der Ausgabeklasse
            End If
        ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
            'Sicherheitsabfrage, falls noch kein Filename fr das SLD angegeben wurde. Dann Aufruf der Ausgabeklasse
            If m_cFilename Is Nothing Or m_cFilename = "" Then
                If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
                    frmMotherform.CHLabelTop("Die Analyse des ArcMap-Projekts ist beendet")
                ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
                    frmMotherform.CHLabelTop("Analysis of the ArcMap-Project has finished")
                End If
                If MessageBox.Show("You haven't specified an SLD store location until now. If you don't specify a location" & _
                ",the application will be terminated." & vbCrLf & "Do you want to specify a location now?", "ArcGIS_SLD_Converter | " & _
                "Analize_ArcMap_Symbols | CentralProcessingFunc", MessageBoxButtons.YesNo, MessageBoxIcon.Information) = DialogResult.Yes Then
                    If File.Exists(frmMotherform.GetSLDFileFromConfigXML) Then
                        frmMotherform.dlgSave.InitialDirectory = frmMotherform.GetSLDFileFromConfigXML
                    End If
                    Dim swOutput As IO.StreamWriter
                    If frmMotherform.dlgSave.ShowDialog = DialogResult.OK Then
                        With frmMotherform.dlgSave
                            .CheckFileExists = False
                            .CheckPathExists = True
                            .DefaultExt = "sld"
                            .Filter = "SLD-files (*.sld)|*.sld"
                            .AddExtension = True
                            .InitialDirectory = IO.Path.GetDirectoryName(m_cFilename)
                            .OverwritePrompt = True
                            .CreatePrompt = False
                            If .ShowDialog() = DialogResult.OK Then
                                m_cFilename = .FileName
                                frmMotherform.txtFileName.Text = m_cFilename
                            End If
                        End With
                        Dim objOutputSLD As New Output_SLD(frmMotherform, Me, m_cFilename)  'Aufruf der Ausgabeklasse
                    Else
                        MyTermination()
                    End If
                Else
                    MyTermination()
                End If
            Else
                Dim objOutputSLD As New Output_SLD(frmMotherform, Me, m_cFilename)  'Aufruf der Ausgabeklasse
            End If
        End If
        frmMotherform.CHLabelBottom("")
        frmMotherform.CHLabelSmall("")
        frmMotherform.ReadBackValues() 'Liest die Benutzerdefinierten Einstellungen in die XML-Datei zurck
    End Function


#Region "Zentrale Verwaltungsfunktionen"
    '************************************************************************************************
    'Hier werden alle laufenden Prozesse auf dem System durchgesehen, um zu sehen, ob ArcMap
    'gestartet ist.
    '************************************************************************************************
    Private Function GetProcesses() As Boolean
        Dim objArcGISProcess As New Process
        Dim objProcess As New Process
        Dim bSwitch As Boolean = False 'der Schalter ist notwendig, weil eine ganze Anzahl Prozesse durchlaufen wird

        If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
            frmMotherform.CHLabelBottom("Suche ArcMap-Prozess")
        ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
            frmMotherform.CHLabelTop("Searching an ArcMap process")
        End If
        Try
            For Each objProcess In Process.GetProcesses()
                If objProcess.ProcessName = "ArcMap" Then
                    bSwitch = True
                End If
            Next

            If bSwitch = True Then
                Return True
            Else
                If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
                    MessageBox.Show("Sie mssen erst ArcMap ffnen!")
                ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
                    frmMotherform.CHLabelTop("You must open ArcMap first")
                End If
                Return False
                Exit Function
            End If
        Catch ex As Exception
            ErrorMsg("Fehler beim Durchsehen der laufenden Prozesse auf dem System", ex.Message, ex.StackTrace, "GetProcesses")
            Return False
        End Try
    End Function


    '************************************************************************************************
    'Der erste zentrale Punkt in der Anwendung: hier wird die Referenz auf die laufende ArcMap-Instanz
    'geholt (fr exe-Anwendung sehr wichtig!!!)
    '************************************************************************************************
    Private Function GetApplication() As Boolean
        Dim Zahl As Long
        m_ObjApp = Nothing

        If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
            frmMotherform.CHLabelBottom("Hole Verweis auf laufende ArcMap-Sitzung")
        ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
            frmMotherform.CHLabelTop("get reference on running ArcMap-session")
        End If

        Try
            m_ObjAppROT = New AppROT
            Zahl = m_ObjAppROT.Count

            If Zahl > 1 Then
                If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
                    MessageBox.Show("Sie haben mehrere ArcMap Sessions gleichzeitig geffnet." & _
                "Bitte schlieen Sie alle ArcMap-Anwendungen bis auf jene, von der Sie das" & _
                " SLD-Dokument generieren mchten und starten Sie die Anwendung erneut!", "Bitte Beachten!", MessageBoxButtons.OK, MessageBoxIcon.Information)
                    Return False
                    Exit Function
                ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
                    MessageBox.Show("You started several ArcMap-sessions at one time." & _
                "Please close all sessions except of that, you want to analyse and" & _
                " start the application again!", "Please notice!", MessageBoxButtons.OK, MessageBoxIcon.Information)
                    Return False
                    Exit Function
                End If

            Else
                If TypeOf m_ObjAppROT.Item(0) Is IMxApplication Then 'berprfung, ob das richtige Objekt erhalten wurde
                    m_ObjApp = m_ObjAppROT.Item(0)
                    m_ObjDoc = m_ObjApp.Document()
                    m_ObjObjectCreator = m_ObjApp
                    Return True
                End If
            End If

        Catch ex As Exception
            ErrorMsg("Fehler beim Referenzieren auf die ArcMap-Instanz ", ex.Message, ex.StackTrace, "GetApplication")
            Return False
        End Try
    End Function


    '************************************************************************************************
    'Das Kartenobjekt: Hier wird auf das Kartenobjekt referenziert. Wenn es mehrere Karten gibt, erfolg
    'eine Abfrage, ob die aktive Karte die Karte ist, die umgewandelt werden soll
    '************************************************************************************************
    Private Function GetMap() As Boolean

        If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
            frmMotherform.CHLabelBottom("Verweis auf das aktuelle Kartenfenster")
        ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
            frmMotherform.CHLabelTop("Reference on the current Session")
        End If

        Try
            If m_ObjDoc.Maps.Count() > 1 Then 'Wenn mehr es mehr als eine Karte in dem Dokument gibt
                If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
                    If MessageBox.Show("Ist die von Ihnen zum Umwandeln in SLD gewnschte Karte gerade die aktive Karte? " & _
                            "Wenn ja, drcken Sie 'Ja' wenn nicht, drcken Sie 'Nein', whlen in ArcMap die richtige Karte aus, " & _
                            "und bettigen den Befehl noch einmal", "Auswahl der gewnschten Karte", MessageBoxButtons.YesNo, MessageBoxIcon.Information) = DialogResult.Yes Then
                        m_ObjMap = m_ObjDoc.FocusMap
                        Return True
                    Else
                        Return False
                        Exit Function
                    End If
                ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
                    If MessageBox.Show("Is that current map this one you want to turn into SLD? " & _
                            "if yes push 'yes' if no push 'no' and choose the right map in ArcMap " & _
                            "and use that procedure again", "Choice of the right map", MessageBoxButtons.YesNo, MessageBoxIcon.Information) = DialogResult.Yes Then
                        m_ObjMap = m_ObjDoc.FocusMap
                        Return True
                    Else
                        Return False
                        Exit Function
                    End If
                End If
            Else
                m_ObjMap = m_ObjDoc.FocusMap
                Return True
            End If
        Catch ex As Exception
            ErrorMsg("Fehler beim Erhalten des aktuellen Kartenobjekts ", ex.Message, ex.StackTrace, "GetMap")
            Return False
        End Try
    End Function

    '************************************************************************************************
    'Hier ist der zweite zentrale Punkt der Anwendung zu finden:
    'Die Layer der aktiven Karte werden durchlaufen und alle Layerobjekte in der Collection zusammen-
    ' gefasst. Auerdem: Collection von jedem Rendererobjekt jedes FeatureLayers. 
    'Hauptschlich: die Symbolwerte der einzelnen Layer werden durchlaufen und die Einzelsymbole in 
    'einer Datenstruktur gespeichert
    '************************************************************************************************
    Private Function AnalyseLayerSymbology() As Boolean
        Dim objLayer As ILayer  'Die Schnittstelle zum aktuellen Layer
        Dim iNumberLayers As Integer    'Die Anzahl aller Layer
        Dim cLayerName As String     'Der Name des aktuellen Layers
        Dim objFstOrderSymbol As ISymbol    'Das ISymbol des entsprechenden Renderers
        m_StrProject = New StructProject    'Projektstruct wird hier initialisiert
        iNumberLayers = m_ObjMap.LayerCount()
        m_StrProject.LayerList = New ArrayList

        Try
            Dim i As Integer
            'Steps through all layers of the first level
            For i = 0 To iNumberLayers - 1
                objLayer = m_ObjMap.Layer(i)
                cLayerName = objLayer.Name
                If frmMotherform.m_bAllLayers = False And objLayer.Visible = False Then
                    If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
                        frmMotherform.CHLabelBottom("Layer " & cLayerName & " wird wird bersprungen, weil sie nicht sichtbar ist.")
                    ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
                        frmMotherform.CHLabelBottom("Layer " & cLayerName & " is skipped, because it is not visible")
                    End If
                Else
                    If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
                        frmMotherform.CHLabelBottom("Layer " & cLayerName & " wird gerade analysiert")
                    ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
                        frmMotherform.CHLabelBottom("Layer " & cLayerName & " is beeing analysed")
                    End If
                    SpreadLayerStructure(objLayer)
                End If
                frmMotherform.CHLabelSmall("")
            Next
            Return True
        Catch ex As Exception
            ErrorMsg("Fehler bei der Analyse des esri-Projekts", ex.Message, ex.StackTrace, "AnalyseLayerSymbology")
            If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
                ErrorMsg("Fehler bei der Analyse des esri-Projekts", ex.Message, ex.StackTrace, "AnalyseLayerSymbology")
            ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
                ErrorMsg("Exception in ArcMap project analysis", ex.Message, ex.StackTrace, "AnalyseLayerSymbology")
            End If
            Return False
        End Try
    End Function

    '************************************************************************************************
    'Analizes the Layer structure of the passed first Level-Layer. If the passed Layer is a GeoFeature
    'Layer (or an object beyond that) it will be analized and stored in the Layerstructs and the project
    'struct. If the passed Layer is a group layer, it will be ignored
    '************************************************************************************************
    Private Function SpreadLayerStructure(ByVal objLayer As ILayer) As Boolean

        Try
            'recursive call if the current layer is a group layer. Solution is that Group Layers will be ignored
            If TypeOf objLayer Is IGroupLayer Then
                Dim j As Integer
                Dim objGRL As IGroupLayer
                Dim objCompLayer As ICompositeLayer
                objGRL = objLayer
                objCompLayer = objGRL
                For j = 0 To objCompLayer.Count() - 1
                    ''Because of this if clause group Layer will be ignored
                    'If TypeOf objCompLayer.Layer(j) Is IFeatureLayer Then

                    'End If
                    SpreadLayerStructure(objCompLayer.Layer(j)) 'recursive call

                Next
            ElseIf TypeOf objLayer Is IFeatureLayer Then
                If TypeOf objLayer Is IGeoFeatureLayer Then
                    Dim objGFL As IGeoFeatureLayer
                    objGFL = objLayer
                    'Hier die Unterscheidung der Renderertypen
                    If TypeOf objGFL.Renderer Is IUniqueValueRenderer Then
                        Dim objRenderer As IUniqueValueRenderer
                        objRenderer = objGFL.Renderer
                        m_StrProject.LayerList.Add(StoreStructUVRenderer(objRenderer, objLayer))
                        AddOneToLayerNumber()
                    End If
                    If TypeOf objGFL.Renderer Is ISimpleRenderer Then
                        Dim objRenderer As ISimpleRenderer
                        objRenderer = objGFL.Renderer
                        m_StrProject.LayerList.Add(StoreStructSimpleRenderer(objRenderer, objLayer))
                        AddOneToLayerNumber()
                    End If
                    If TypeOf objGFL.Renderer Is IClassBreaksRenderer Then
                        Dim objRenderer As IClassBreaksRenderer
                        objRenderer = objGFL.Renderer
                        m_StrProject.LayerList.Add(StoreStructCBRenderer(objRenderer, objLayer))
                        AddOneToLayerNumber()
                    End If
                    'andere Renderer werden evtl. spter abgedeckt
                    'If TypeOf objGFL.Renderer Is IChartRenderer Then
                    '    Dim objRenderer As IChartRenderer
                    '    objRenderer = objGFL.Renderer
                    '    objFstOrderSymbol = objRenderer.Symbol
                    '    'TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO
                    'End If
                    'If TypeOf objGFL.Renderer Is IDotDensityRenderer Then
                    '    Dim objRenderer As IDotDensityRenderer
                    '    objRenderer = objGFL.Renderer
                    '    'TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO
                    'End If
                    'If TypeOf objGFL.Renderer Is IProportionalSymbolRenderer Then
                    '    Dim objRenderer As IProportionalSymbolRenderer
                    '    objRenderer = objGFL.Renderer
                    '    'TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO
                    'End If
                    'If TypeOf objGFL.Renderer Is IScaleDependentRenderer Then
                    '    Dim objRenderer As IScaleDependentRenderer
                    '    objRenderer = objGFL.Renderer
                    '    'TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO
                    'End If
                End If

            Else
                If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
                    InfoMsg("Die Layerart ihres ArcMap-Projekts wird derzeit noch nicht untersttzt.", "SpreadLayerStructure")
                ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
                    InfoMsg("The kind of Layer you use in your ArcMap project is currently not beeing supported.", "SpreadLayerStructure")
                End If
                MyTermination()
            End If
        Catch ex As Exception
            If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
                InfoMsg("Unerwarteter Fehler beim Speichern in den Layerstrukturen", "SpreadLayerStructure")
            ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
                InfoMsg("Unexpected Error during storing in layerstructs", "SpreadLayerStructure")
            End If
        End Try



    End Function



    ''************************************************************************************************
    ''Hier ist der zweite zentrale Punkt der Anwendung zu finden:
    ''Die Layer der aktiven Karte werden durchlaufen und alle Layerobjekte in der Collection zusammen-
    '' gefasst. Auerdem: Collection von jedem Rendererobjekt jedes FeatureLayers. 
    ''Hauptschlich: die Symbolwerte der einzelnen Layer werden durchlaufen und die Einzelsymbole in 
    ''einer Datenstruktur gespeichert
    ''************************************************************************************************
    'Private Function AnalyseLayerSymbology() As Boolean
    '    Dim objLayer As ILayer  'Die Schnittstelle zum aktuellen Layer
    '    Dim iNumberLayers As Integer    'Die Anzahl aller Layer
    '    Dim cLayerName As String     'Der Name des aktuellen Layers
    '    Dim objFstOrderSymbol As ISymbol    'Das ISymbol des entsprechenden Renderers
    '    m_StrProject = New StructProject    'Projektstruct wird hier initialisiert
    '    iNumberLayers = m_ObjMap.LayerCount()
    '    m_StrProject.LayerCount = iNumberLayers 'Anzahl der Layer in Projektstruct gespeichert
    '    m_StrProject.LayerList = New ArrayList

    '    Try
    '        Dim i As Integer
    '        For i = 0 To iNumberLayers - 1
    '            objLayer = m_ObjMap.Layer(i)
    '            cLayerName = objLayer.Name
    '            If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
    '                frmMotherform.CHLabelBottom("Layer " & cLayerName & " wird gerade analysiert")
    '            ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
    '                frmMotherform.CHLabelBottom("Layer " & cLayerName & " is beeing analysed")
    '            End If

    '            'Hier die erste Unterscheidung der Layer: ob IFeatureLayer (sind wohl die meisten Layer)
    '            'evtl testen, ob es noch weitere relevante bergeordnete Layertypen gibt!!
    '            If TypeOf objLayer Is IFeatureLayer Then
    '                'Hier die zweite Unterscheidung der Layer: ob IGeoFeatureLayer (hier kann es andere
    '                'Mglichkeiten geben - z.B. Layer mit Text...) untergeordnete Layertypen



    '                If TypeOf objLayer Is IGeoFeatureLayer Then
    '                    Dim objGFL As IGeoFeatureLayer
    '                    objGFL = objLayer
    '                    'Hier die Unterscheidung der Renderertypen
    '                    If TypeOf objGFL.Renderer Is IUniqueValueRenderer Then
    '                        Dim objRenderer As IUniqueValueRenderer
    '                        objRenderer = objGFL.Renderer
    '                        m_StrProject.LayerList.Add(StoreStructUVRenderer(objRenderer, objLayer))
    '                    End If
    '                    If TypeOf objGFL.Renderer Is ISimpleRenderer Then
    '                        Dim objRenderer As ISimpleRenderer
    '                        objRenderer = objGFL.Renderer
    '                        m_StrProject.LayerList.Add(StoreStructSimpleRenderer(objRenderer, objLayer))
    '                    End If
    '                    If TypeOf objGFL.Renderer Is IClassBreaksRenderer Then
    '                        Dim objRenderer As IClassBreaksRenderer
    '                        objRenderer = objGFL.Renderer
    '                        m_StrProject.LayerList.Add(StoreStructCBRenderer(objRenderer, objLayer))
    '                    End If
    '                    'andere Renderer werden evtl. spter abgedeckt
    '                    'If TypeOf objGFL.Renderer Is IChartRenderer Then
    '                    '    Dim objRenderer As IChartRenderer
    '                    '    objRenderer = objGFL.Renderer
    '                    '    objFstOrderSymbol = objRenderer.Symbol
    '                    '    'TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO
    '                    'End If
    '                    'If TypeOf objGFL.Renderer Is IDotDensityRenderer Then
    '                    '    Dim objRenderer As IDotDensityRenderer
    '                    '    objRenderer = objGFL.Renderer
    '                    '    'TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO
    '                    'End If
    '                    'If TypeOf objGFL.Renderer Is IProportionalSymbolRenderer Then
    '                    '    Dim objRenderer As IProportionalSymbolRenderer
    '                    '    objRenderer = objGFL.Renderer
    '                    '    'TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO
    '                    'End If
    '                    'If TypeOf objGFL.Renderer Is IScaleDependentRenderer Then
    '                    '    Dim objRenderer As IScaleDependentRenderer
    '                    '    objRenderer = objGFL.Renderer
    '                    '    'TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO
    '                    'End If
    '                End If
    '            End If
    '            frmMotherform.CHLabelSmall("")
    '        Next
    '        Return True
    '    Catch ex As Exception
    '        MessageBox.Show(ex.StackTrace & " " & ex.Source)
    '        ErrorMsg("Fehler bei der Analyse des esri-Projekts", ex.Message, ex.StackTrace, "AnalyseLayerSymbology")
    '        Return False
    '    End Try
    'End Function
#End Region  'Verweis auf die ntigen ArcMap-Instanzen; 


#Region "Hilfsfunktionen"
    '************************************************************************************************
    'Wenn das aktuelle Symbol ein MarkerSymbol ist, wird das MarkerSymbol, und alle darunter liegenden
    'Objekte (Symbolobjekte) durchgesucht
    '************************************************************************************************
    Private Function MarkerSymbolScan(ByVal Symbol As IMarkerSymbol) As String
        Dim cValue As String
        If TypeOf Symbol Is ISimpleMarkerSymbol Then
            cValue = "ISimpleMarkerSymbol"
            Return cValue
        ElseIf TypeOf Symbol Is ICartographicMarkerSymbol Then
            Dim ICMS As ICartographicMarkerSymbol
            ICMS = Symbol
            If TypeOf ICMS Is ICharacterMarkerSymbol Then
                cValue = "ICharacterMarkerSymbol"
                Return cValue
            ElseIf TypeOf ICMS Is IPictureMarkerSymbol Then
                cValue = "IPictureMarkerSymbol"
                Return cValue
            End If
        ElseIf TypeOf Symbol Is IArrowMarkerSymbol Then
            cValue = "IArrowMarkerSymbol"
            Return cValue
        ElseIf TypeOf Symbol Is IMultiLayerMarkerSymbol Then
            cValue = "IMultiLayerMarkerSymbol"
            Return cValue
        Else
            cValue = "false"
            Return cValue
        End If
    End Function


    '************************************************************************************************
    'Wenn das aktuelle Symbol ein LineSymbol ist, wird das LineSymbol, und alle darunter liegenden
    'Objekte (Symbolobjekte) durchgesucht
    '************************************************************************************************
    Private Function LineSymbolScan(ByVal Symbol As ILineSymbol) As String
        Dim cValue As String
        Dim bSwitch As Boolean
        bSwitch = False
        If TypeOf Symbol Is ICartographicLineSymbol Then
            Dim ICLS As ICartographicLineSymbol
            ICLS = Symbol
            If TypeOf ICLS Is IHashLineSymbol Then
                cValue = "IHashLineSymbol"
                bSwitch = True
                Return cValue
            ElseIf TypeOf ICLS Is IMarkerLineSymbol Then
                cValue = "IMarkerLineSymbol"
                bSwitch = True
                Return cValue
            End If
            If bSwitch = False Then
                cValue = "ICartographicLineSymbol"
                Return cValue
            End If
        ElseIf TypeOf Symbol Is ISimpleLineSymbol Then
            cValue = "ISimpleLineSymbol"
            Return cValue
        ElseIf TypeOf Symbol Is IPictureLineSymbol Then
            cValue = "IPictureLineSymbol"
            Return cValue
        ElseIf TypeOf Symbol Is IMultiLayerLineSymbol Then
            cValue = "IMultiLayerLineSymbol"
            Return cValue
        Else
            cValue = "false"
            Return cValue
        End If
    End Function


    '************************************************************************************************
    'Wenn das aktuelle Symbol ein FillSymbol ist, wird das FillSymbol, und alle darunter liegenden
    'Objekte (Symbolobjekte) durchgesucht
    '************************************************************************************************
    Private Function FillSymbolScan(ByVal Symbol As IFillSymbol) As String
        Dim cValue As String
        If TypeOf Symbol Is ISimpleFillSymbol Then
            cValue = "ISimpleFillSymbol"
            Return cValue
        ElseIf TypeOf Symbol Is IMarkerFillSymbol Then
            cValue = "IMarkerFillSymbol"
            Return cValue
        ElseIf TypeOf Symbol Is ILineFillSymbol Then
            cValue = "ILineFillSymbol"
            Return cValue
        ElseIf TypeOf Symbol Is IDotDensityFillSymbol Then
            cValue = "IDotDensityFillSymbol"
            Return cValue
        ElseIf TypeOf Symbol Is IPictureFillSymbol Then
            cValue = "IPictureFillSymbol"
            Return cValue
        ElseIf TypeOf Symbol Is IGradientFillSymbol Then
            cValue = "IGradientFillSymbol"
            Return cValue
        ElseIf TypeOf Symbol Is IMultiLayerFillSymbol Then
            cValue = "IMultiLayerFillSymbol"
            Return cValue
        Else
            cValue = "false"
            Return cValue
        End If
    End Function


    '************************************************************************************************
    'Wenn das aktuelle Symbol ein 3DChartSymbol ist, wird das 3DChartSymbol, und alle darunter liegenden
    'Objekte (Symbolobjekte) durchgesucht
    '************************************************************************************************
    Private Function IIIDChartSymbolScan(ByVal Symbol As I3DChartSymbol) As String
        Dim cValue As String
        If TypeOf Symbol Is IBarChartSymbol Then
            cValue = "IBarChartSymbol"
            Return cValue
        ElseIf TypeOf Symbol Is IPieChartSymbol Then
            cValue = "IPieChartSymbol"
            Return cValue
        ElseIf TypeOf Symbol Is IStackedChartSymbol Then
            cValue = "IStackedChartSymbol"
            Return cValue
        Else
            cValue = "false"
            Return cValue
        End If
    End Function


    '************************************************************************************************
    'Die Funktion gibt eine Liste mit UniqeValues zu dem angegebenen Feld aus
    'Parameter:
    '       Table: Das FeatureTable-Objekt
    '       FieldName: Der Spaltenname der betroffenen Spalte, nach der klassifiziert wurde
    '************************************************************************************************
    Private Function GimmeUniqeValuesForFieldname(ByVal Table As ITable, ByVal FieldName As String) As Boolean
        Dim pQueryDef As IQueryDef
        Dim pRow As IRow
        Dim pCursor As ICursor
        Dim pFeatureCursor As IFeatureCursor
        Dim pFeatureWorkspace As IFeatureWorkspace
        Dim pDataset As IDataset
        Dim alUniqueVal As New ArrayList


        Try
            pDataset = Table
            pFeatureWorkspace = pDataset.Workspace
            pQueryDef = pFeatureWorkspace.CreateQueryDef
            With pQueryDef
                .Tables = pDataset.Name
                .SubFields = "DISTINCT(" & FieldName & ")"
                pCursor = .Evaluate
            End With
            'Hier wird die erhaltene Spalte durchlaufen und in der Arraylist abgespeichert
            pRow = pCursor.NextRow
            Do Until pRow Is Nothing
                alUniqueVal.Add(pRow.Value(0))
                pRow = pCursor.NextRow
            Loop


            m_alClassifiedFields.Add(alUniqueVal)
            Return True
        Catch ex As Exception
            ErrorMsg("Fehler beim Generieren der UniqueValues", ex.Message, ex.StackTrace, "GimmeUniqeValuesForFieldname")
        End Try
    End Function


    '************************************************************************************************
    'Die Funktion gibt eine Liste mit UniqeValues zu dem angegebenen Feld aus
    'Parameter:
    '           Table: Das FeatureTable-Objekt
    '           FieldName: Der Spaltenname der betroffenen Spalte, nach der klassifiziert wurde
    '           JoinedTables: Eine Arraylist mit den Namen aller an die Haupttabelle drangejointen Tabellen
    '************************************************************************************************
    Private Function GimmeUniqeValuesForFieldname(ByVal Table As ITable, ByVal FieldName As String, ByVal JoinedTables As ArrayList) As Boolean
        Dim pQueryDef As IQueryDef
        Dim pRow As IRow
        Dim pCursor As ICursor
        Dim pFeatureWorkspace As IFeatureWorkspace
        Dim pDataset As IDataset
        Dim alUniqueVal As New ArrayList


        Try
            Dim cMember As String
            For Each cMember In JoinedTables
                cMember = "," & cMember
            Next

            pDataset = Table
            pFeatureWorkspace = pDataset.Workspace
            pQueryDef = pFeatureWorkspace.CreateQueryDef
            With pQueryDef
                .Tables = pDataset.Name & cMember
                .SubFields = "DISTINCT(" & FieldName & ")"
                pCursor = .Evaluate
            End With
            'Hier wird die erhaltene Spalte durchlaufen und in der Arraylist abgespeichert
            pRow = pCursor.NextRow
            Do Until pRow Is Nothing
                alUniqueVal.Add(pRow.Value(0))
                pRow = pCursor.NextRow
            Loop
            m_alClassifiedFields.Add(alUniqueVal)
            Return True
        Catch ex As Exception
            MessageBox.Show(ex.Message & " " & ex.Source & " " & ex.StackTrace)
        End Try

    End Function

    '************************************************************************************************
    'Die Funktion gibt eine Liste mit UniqeValues zu dem angegebenen Feld aus fr Shape-based Projekte
    'Parameter:
    '           Table: Das FeatureTable-Objekt
    '           FieldName: Der Spaltenname der betroffenen Spalte, nach der klassifiziert wurde
    '************************************************************************************************
    Private Function GimmeUniqueValuesFromShape(ByVal Table As ITable, ByVal FieldNames As ArrayList) As Boolean
        Dim pQueryFilter As IQueryFilter
        pQueryFilter = New QueryFilter
        Dim pCursor As ICursor
        Dim pData As IDataStatistics
        pData = New DataStatistics
        Dim i, bla As Int16
        Dim objEnum As IEnumerator
        Dim al As ArrayList

        Try
            If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
                If MessageBox.Show("ACHTUNG: Sie haben Layer aus einem Shapefile mit mehr als 1 klassifizierenden Feld. " & _
                            "Wenn Sie diese Art von Layer analysieren wollen, kann das sehr lange dauern (etliche Minuten bis im 2-stelligen" & _
                            "Bereich, je nach Prozessorleistung und Datenmenge TIP: Speichern Sie die Inhalte in " & _
                            "einer Personal GDB oder SDE und starten Sie die Analyse erneut-dies dauert je nach Datenmenge nur ein paar Sekunden). " & _
                            "Wollen Sie trotzdem mit der Analyse fortfahren?", "ACHTUNG-RECHENINTENSIVE FUNKTION", MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk) = DialogResult.Yes Then
                    For i = 0 To FieldNames.Count - 1
                        Me.frmMotherform.CHLabelSmall("Bitte warten - Klassifizierungsfeld " & i + 1 & " von " & FieldNames.Count)
                        pData.Field = FieldNames(i)
                        With pQueryFilter
                            .SubFields = FieldNames(i)
                        End With
                        pCursor = Table.Search(pQueryFilter, False)
                        pData.Cursor = pCursor
                        frmMotherform.DoEvents()
                        objEnum = pData.UniqueValues()
                        al = New ArrayList
                        objEnum.MoveNext()
                        Do Until objEnum.Current Is Nothing
                            al.Add(objEnum.Current)
                            objEnum.MoveNext()
                        Loop
                        al.Sort()
                        m_alClassifiedFields.Add(al)
                    Next
                Else
                    Me.MyTermination()
                End If
            ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
                If MessageBox.Show("ATTENTION: You have a layer from a shape-file with more than one classifying field. " & _
                            "If you want to analyse that kind of layer, it can take A LOT OF TIME (several Minutes until hours " & _
                            "depending on your processor and the dimension of your data TIP: Store your data in " & _
                            "a personal GDB or an ArcSDE-DB and start the analyse again-that takes only a few seconds). " & _
                            "do you anyhow want to continue?", "ATTENTION!-CALCULATION INTENSIVE FUNCTION", MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk) = DialogResult.Yes Then
                    For i = 0 To FieldNames.Count - 1
                        Me.frmMotherform.CHLabelSmall("Please wait - classified Field Nr. " & i + 1 & " of " & FieldNames.Count)
                        pData.Field = FieldNames(i)
                        With pQueryFilter
                            .SubFields = FieldNames(i)
                        End With
                        pCursor = Table.Search(pQueryFilter, False)
                        pData.Cursor = pCursor
                        frmMotherform.DoEvents()
                        objEnum = pData.UniqueValues()
                        al = New ArrayList
                        objEnum.MoveNext()
                        Do Until objEnum.Current Is Nothing
                            al.Add(objEnum.Current)
                            objEnum.MoveNext()
                        Loop
                        al.Sort()
                        m_alClassifiedFields.Add(al)
                    Next
                Else
                    Me.MyTermination()
                End If
            End If

        Catch ex As Exception
            ErrorMsg("Fehler beim Erstellen der UniqueValues", ex.Message, ex.StackTrace, "GimmeUniqueValuesFromShape")
            MyTermination()
        End Try
    End Function

    'berladen:
    'Private Function GimmeUniqueValuesFromShape(ByVal FLayer As IFeatureLayer, ByVal alFields As ArrayList) As Boolean
    '    Dim al As ArrayList
    '    Dim cFieldName As String
    '    Dim pFCur As IFeatureCursor
    '    Dim pFeat As IFeature
    '    Dim pFSel As IFeatureSelection
    '    'Dim i, j As Integer
    '    pFSel = FLayer
    '    al = New ArrayList

    '    Select Case alFields.Count
    '        Case 2
    '            m_al1 = New ArrayList
    '            m_al2 = New ArrayList
    '        Case 3
    '            m_al1 = New ArrayList
    '            m_al2 = New ArrayList
    '            m_al3 = New ArrayList
    '    End Select
    '    Try
    '        For Each cFieldName In alFields
    '            If FLayer.FeatureClass.FindField(cFieldName) = -1 Then
    '                ErrorMsg("Fehler beim Erstellen der UniqueValues", "ein bergebenes Feld wurde nicht in dem Layer gefunden", "GimmeUniqueValuesFromShape")
    '                MyTermination()
    '            End If
    '        Next
    '        If pFSel.SelectionSet.Count = 0 Then
    '            pFCur = FLayer.FeatureClass.Search(Nothing, False)
    '        Else
    '            pFSel.SelectionSet.Search(Nothing, False, pFCur)
    '        End If

    '        frmMotherform.ShowWorld()
    '        pFeat = pFCur.NextFeature
    '        Do Until pFeat Is Nothing
    '            al.Clear()
    '            For Each cFieldName In alFields
    '                al.Add(pFeat.Value(pFCur.FindField(cFieldName)))
    '            Next
    '            SortCursorRowValues(al)
    '            pFeat = pFCur.NextFeature
    '        Loop

    '        Select Case alFields.Count
    '            Case 2
    '                m_alClassifiedFields.Add(m_al1)
    '                m_alClassifiedFields.Add(m_al2)
    '            Case 3
    '                m_alClassifiedFields.Add(m_al1)
    '                m_alClassifiedFields.Add(m_al2)
    '                m_al3 = New ArrayList
    '        End Select
    '        frmMotherform.HideWorld()
    '        Return True
    '    Catch ex As Exception
    '        ErrorMsg("Fehler beim Erstellen der UniqueValues", ex.Message, "GimmeUniqueValuesFromShape")
    '        MessageBox.Show(ex.StackTrace)
    '        MyTermination()
    '    End Try
    'End Function

    '************************************************************************************************
    'kleine Hilfsfunktion von GimmeUniqueValuesFromShape, die Werte aus einer Cursorzeile auf 
    'Arraylists verteilt
    '************************************************************************************************
    Private Function SortCursorRowValues(ByVal al As ArrayList) As Boolean
        Select Case al.Count
            Case 2
                m_al1.Add(al(0))
                m_al2.Add(al(1))
            Case 3
                m_al1.Add(al(0))
                m_al2.Add(al(1))
                m_al3.Add(al(2))
        End Select
    End Function

    '************************************************************************************************
    'Die Funktion gibt die separaten Inhalte der einzelnen Tabellenfelder in einer Arraylist zurck
    'dazu benutzt sie den Value aus UniqueValues, many Fields zum Vergleich mit den Ergebnissen, die
    'von der Query zurckgegeben wird
    'Parameter:     value (der zusammengesetzte Value einer Klassifikation, der beim Klassensymbol steht)
    '               FieldNames (Die Feldnamen [Spaltennamen] der Tabelle)
    '               FieldDelimiter (das Trennungszeichen i.a.R. ein komma)
    '               Layer (Der aktuelle, gerade durchlaufene Layer)
    '************************************************************************************************
    Private Function GimmeSeperateFieldValues(ByVal value As String, ByVal FieldDelimiter As String) As ArrayList
        Dim alSepValues As New ArrayList
        Dim objFeatCurs As IFeatureCursor
        Dim objQueryFilter As IQueryFilter
        objQueryFilter = New QueryFilter
        Dim objFeature As IFeature
        Dim cCompare As String
        Dim zahl As Integer

        Try
            Select Case m_alClassifiedFields.Count
                Case 2
                    Dim al1, al2 As ArrayList
                    al1 = m_alClassifiedFields(0)
                    al2 = m_alClassifiedFields(1)
                    Dim i, j As Integer
                    For i = 0 To al1.Count - 1
                        For j = 0 To al2.Count - 1
                            If (CStr(al1(i)) & FieldDelimiter & CStr(al2(j))) = value Then
                                alSepValues.Add(al1(i))
                                alSepValues.Add(al2(j))
                                Exit Select
                            End If
                        Next
                    Next
                Case 3
                    Dim al1, al2, al3 As ArrayList
                    al1 = m_alClassifiedFields(0)
                    al2 = m_alClassifiedFields(1)
                    al3 = m_alClassifiedFields(2)
                    Dim i, j, k As Integer
                    For i = 0 To m_alClassifiedFields(0).Count - 1
                        For j = 0 To m_alClassifiedFields(1).Count - 1
                            For k = 0 To m_alClassifiedFields(2).Count - 1
                                If (CStr(al1(i)) & FieldDelimiter & CStr(al2(j)) & FieldDelimiter & CStr(al3(k))) = value Then
                                    alSepValues.Add(al1(i))
                                    alSepValues.Add(al2(j))
                                    alSepValues.Add(al3(k))
                                    Exit Select
                                End If
                            Next
                        Next
                    Next
            End Select
            Return alSepValues
        Catch ex As Exception
            MessageBox.Show(ex.Message)
        End Try
    End Function


    '************************************************************************************************
    'Die Funktion gibt die Colors einer ColorRamp als Arraylist zurck
    '************************************************************************************************
    Private Function GimmeArrayListForColorRamp(ByVal ColorRamp As IColorRamp) As ArrayList
        Dim EColors As IEnumColors
        Dim AL As ArrayList
        Dim i As Integer
        EColors = ColorRamp.Colors
        For i = 0 To ColorRamp.Size - 1
            AL.Add(GimmeStringForColor(EColors.Next()))
        Next
        Return AL
    End Function


    '************************************************************************************************
    'Die Funktion nimmt ein esri-IColor-Objekt entgegen, wandelt die Farbe in Web-Schreibweise um, und
    'gibt sie als string zurck
    'If color is fully transparent, an empty string is returned.
    '************************************************************************************************
    Private Function GimmeStringForColor(ByVal color As IColor) As String
        Dim cCol As String
        Dim cRed, cGreen, cBlue As String
        Dim objRGB As IRgbColor

        If (color.Transparency = 0) Then
            cCol = ""
        Else
            objRGB = New RgbColor

            objRGB.RGB = color.RGB
            cRed = CheckDigits(Hex(objRGB.Red()).ToString)
            cGreen = CheckDigits(Hex(objRGB.Green()).ToString)
            cBlue = CheckDigits(Hex(objRGB.Blue()).ToString)
            cCol = "#" & cRed & cGreen & cBlue
        End If

        Return cCol
    End Function

    '************************************************************************************************
    'Die Funktion kontrolliert, ob der den Hexadezimalwert reprsentierende String alle 2 Stellen hat
    '(wenn der hexadez nur einstellig ist , wird auch nur eine Stelle zurckgegeben. Ich brauche auch
    ' die vorangestellte Null!
    '************************************************************************************************
    Private Function CheckDigits(ByVal value As String) As String
        Dim cReturn As String
        cReturn = value
        If cReturn.Length = 1 Then
            cReturn = cReturn.Insert(0, "0")
        End If
        Return cReturn
    End Function

    '************************************************************************************************
    'Extract annotation style and property name from a layer.
    'This initial implementation only handles the most basic annotation: simple expression of a
    'single property, applicable to all features/symbols.
    'Return a StructAnnotation. The PropertyName is an empty string if there is no annotation.
    '************************************************************************************************
    Private Function GetAnnotation(ByVal objLayer As IFeatureLayer) As StructAnnotation
        Dim annotation As New StructAnnotation

        annotation.PropertyName = ""
        If TypeOf objLayer Is IGeoFeatureLayer Then
            Dim objGFL As IGeoFeatureLayer
            objGFL = objLayer

            Dim annoPropsColl As IAnnotateLayerPropertiesCollection
            annoPropsColl = objGFL.AnnotationProperties
            If objGFL.DisplayAnnotation And annoPropsColl.Count > 0 Then
                Dim annoLayerProps As IAnnotateLayerProperties
                annoPropsColl.QueryItem(0, annoLayerProps)
                If TypeOf annoLayerProps Is ILabelEngineLayerProperties And annoLayerProps.DisplayAnnotation Then
                    Dim labelProps As ILabelEngineLayerProperties
                    labelProps = annoLayerProps
                    ' For the moment only implement the simplest case
                    If annoLayerProps.WhereClause = "" And labelProps.IsExpressionSimple Then
                        annotation.IsSingleProperty = True
                        annotation.PropertyName = labelProps.Expression.Replace("[", "").Replace("]", "")
                        annotation.TextSymbol = StoreText(labelProps.Symbol)
                    End If
                End If
            End If
        End If
        Return annotation
    End Function

    '************************************************************************************************
    'Die zentrale Fehlermeldung
    '************************************************************************************************
    Private Function ErrorMsg(ByVal Message As String, ByVal ExMessage As String, ByVal Stack As String, ByVal FunctionName As String)
        MessageBox.Show(Message & vbCrLf & ExMessage & vbCrLf & Stack, "ArcGIS_SLD_Converter | Analize_ArcMap_Symbols | " & FunctionName, MessageBoxButtons.OK, MessageBoxIcon.Error)
        MyTermination()
    End Function

    '************************************************************************************************
    'Die zentrale Infomeldung
    '************************************************************************************************
    Private Function InfoMsg(ByVal Message As String, ByVal FunctionName As String)
        MessageBox.Show(Message, "ArcGIS_SLD_Converter | Analize_ArcMap_Symbols | " & FunctionName, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
    End Function

    '************************************************************************************************
    'Beim (vorzeitigen) Beenden werden hier alle Objekte zurckgesetzt
    '
    '************************************************************************************************
    Public Function MyTermination() As Boolean
        End
        'oder: application.exit
    End Function
#End Region

End Class
