'####################################################################################################################
'*******************ArcGIS_SLD_Converter*****************************************************************************
'*******************Class: Output_SLD********************************************************************************
'*******************AUTHOR: Albrecht Weiser, University of applied Sciences in Mainz, Germany 2005*******************
'The Application was part of my Diploma thesis:**********************************************************************
'"Transforming map-properties of maps in esri-data to an OGC-conformous SLD-document, for publishing the ArcGIS-map * 
'with an OGC- conformous map-server"*********************************************************************************
'ABSTRACT:
'The program so called "ArcGIS-map to SLD Converter" analyses an
'ArcMap-Project with respect to its symbolisation and assembles an SLD
'for the OGC-Web Map Service (WMS) from the gathered data. The program
'is started parallel to a running ArcMap 9.X-session. Subsequently the
'application deposits an SLD-file which complies the symbolisation of
'the available ArcMap-project. With the SLD a WMS-project may be
'classified and styled according to the preceding ArcMap-project. The
'application is written in VisualBasic.NET and uses the .NET 2.0
'Framework (plus XML files for configuration). For more informtion
'refer to: 
'http://arcmap2sld.geoinform.fh-mainz.de/ArcMap2SLDConverter_Eng.htm.
'LICENSE:
'This program is free software under the license of the GNU Lesser General Public License (LGPL) As published by the Free Software Foundation. 
'With the use and further development of this code you accept the terms of LGPL. For questions of the License refer to:
'http://www.gnu.org/licenses/lgpl.html
'DISCLAIMER:
'THE USE OF THE SOFTWARE ArcGIS-map to SLD Converter HAPPENS AT OWN RISK.
'I CANNOT ISSUE A GUARANTEE FOR ANY DISADVANTAGES (INCLUDING LOSS OF DATA; ETC.) THAT
'MAY ARISE FROM USING THIS SOFTWARE.
'DESCRIPTION:
'The class Output_SLD is called by the class Analize_ArcMap_Symbols. It receives the Object of Analize_ArcMap_Symbols.
'So it possesses the structs with all stored data. The Output_SLD creates the final SLD. Main function is "CentralProcessingFunc"
'it rules all the other movements. The SLD-Skeleton is built by "WriteToSLD". The mapping from the ArcMap-symbols to the
'SLD symbols is managed by the function "GetValueFromSymbolstruct()". The XML-file of the SLD is built by the class
'"XMLHandle" The function "WriteToSLD" passes over each package of SLD-XML-code to the class "XMLHandle"
'CHANGES:
'04.02.2006: Language-customizing english/german
'11.01.2007: Changing the floating point values from commaseparated to point separated (which is OGC conformous)
'28.02.2007: - Added changes for Ionic RSW WMS: Rule.Name and FeatureTypeName
'            - Improved code readability and consistency by moving common code
'              blocks for point and line symbolizers to separate functions.
'              Same for marker code to GetMarkerValue function.
'            - Improved point symbol conversion to well known marker types,
'              including simple, arrow symbols. For some character marker
'              symbol fonts, consisting mainly of circles and rectangles,
'              it tries to map the characters to well-known markers.
'            - Improved transparent color handling: polygons with outline but no fill, and point symbols.
'27.03.2007: - For simpleRenderer added Rule Name and Title elements, needed for Ionic.
'            - For simple polygon features, do not default to solid fill, but look at polygon features.
'            - For multi-layer symbols create a symbolizer for each layer.
'            - For line features, added stroke-dasharray CSS based on the dash-type (simple lines)
'              or the template (cartographic lines) defined for a symbol.
'              Also implements HashLineSymbols using this.
'            - Improved detection of no outline by checking for outline size of 0
'            - For polygon features, don't include a fill CSS if the feature has no, or a transparent, fill.
'            - For polygon features, don't include a stroke CSS if there is no border.
'            - Determine opacity based on the transparency set in the symbol color.
'            - Simplified color handling by always having no color if the color is transparent or unused.
'25.04.2007: - Added TextSymbolizer support for simple annotation with a single feature as label.
'12.09.2007: - Changed the logical Operator "AND" to the OGC-compliant "And". Thus changed the "LUT_SLD_mapping_file.xml"
'            - Changed the format of the stroke-dasharray to the OGC-compliant format without the suffixes "px"
'30.11.2007: - LowerBoundary and UpperBoundary gives now also OGC compliant Values with decimal point instaed comma
'09.06.2008: - Add support for grouped values in UniqueValueRenderer, generating an OGC "Or" for the group of values.
'            - Bugfix for OGC And (in LUT_SLD_mapping_file.xml the OGC "And" tag was spelled as "AND").
'10.09.2008: - Added support for separation of  layers in multiple files
'####################################################################################################################

Imports System
Imports System.Xml
Imports stdole
Imports ArcGIS_SLD_Converter.Analize_ArcMap_Symbols

Public Class Output_SLD


    '##################################################################################################
    '######################################## DEKLARATIONEN ###########################################
    '##################################################################################################

#Region "Membervariablen"
    Private frmMotherForm As Motherform
    Private m_objData As Analize_ArcMap_Symbols
    Private m_strDataSavings As Analize_ArcMap_Symbols.StructProject
    Private m_objXMLHandle As XMLHandle
    Private m_cFilename As String       'The whole Path and Filename of the SLD
    Private m_cFile As String   'The filename of the SLD
    Private m_cPath As String   'Der Pfad zur SLD-Datei
    Private m_bSepFiles As Boolean  'Layers in separate files?
#End Region

    '##################################################################################################
    '######################################## ENUMERATIONEN ###########################################
    '##################################################################################################

#Region "Enums"


#End Region

    '##################################################################################################
    '########################################### ROUTINEN #############################################
    '##################################################################################################

#Region "Routinen"
    'This one is used if all Layers are written in one File. Then Filename contains both: path and Filename of SLD to create
    Public Sub New(ByVal Mother As Motherform, ByVal Analize As Analize_ArcMap_Symbols, ByVal Filename As String)
        frmMotherForm = Mother
        m_cFilename = Filename
        m_bSepFiles = frmMotherForm.GetInfoSeparateLayers
        m_cFile = frmMotherForm.GetSLDFile
        m_cPath = frmMotherForm.GetSLDPath
        m_objData = Analize
        m_strDataSavings = m_objData.GetProjectData
        CentralProcessingFunc()
    End Sub

    Public Sub New()
        'm_cFilename = "C:\Krimskrams\bla.sld"      'Zu Testzwecken
        CentralProcessingFunc()
    End Sub

#End Region  'Die Public Sub New()

    '##################################################################################################
    '######################################### FUNKTIONEN #############################################
    '##################################################################################################

#Region "Steuerungsfunktionen" '(incl. SLD-Tag Schreibanweisungen)

    '************************************************************************************************
    'Die Funktion steuert die Prozesse zentral. 
    '************************************************************************************************
    Private Function CentralProcessingFunc() As Boolean
        Dim bSuccess As Boolean
        bSuccess = False
        If frmMotherForm.m_enumLang = Motherform.Language.Deutsch Then
            frmMotherForm.CHLabelTop("Die Ausgabe in SLD")
            frmMotherForm.CHLabelBottom("Gespeicherte Daten werden verarbeitet")
        ElseIf frmMotherForm.m_enumLang = Motherform.Language.English Then
            frmMotherForm.CHLabelTop("The Output in SLD")
            frmMotherForm.CHLabelBottom("the stored data is beeing processed")
        End If
        
        ' m_objXMLHandle = New XMLHandle
        'CreateSLD()
        If WriteToSLD() = True Then
            bSuccess = True
        End If
        If frmMotherForm.m_enumLang = Motherform.Language.Deutsch Then
            frmMotherForm.CHLabelTop("Fertig")
        ElseIf frmMotherForm.m_enumLang = Motherform.Language.English Then
            frmMotherForm.CHLabelTop("Ready")
        End If

        If bSuccess = True Then
            If frmMotherForm.m_enumLang = Motherform.Language.Deutsch Then
                frmMotherForm.CHLabelBottom("Die Datei wurde angelegt.")
            ElseIf frmMotherForm.m_enumLang = Motherform.Language.English Then
                frmMotherForm.CHLabelBottom("The file has been generated.")
            End If

            If frmMotherForm.chkValidate.Checked = True Then
                Dim ValSLD As New ValidateSLD(frmMotherForm)
            Else
                frmMotherForm.CHLabelSmall("")
            End If
        Else
            If frmMotherForm.m_enumLang = Motherform.Language.Deutsch Then
                frmMotherForm.CHLabelBottom("Die Datei konnte nicht angelegt werden")
                frmMotherForm.CHLabelSmall("")
            ElseIf frmMotherForm.m_enumLang = Motherform.Language.English Then
                frmMotherForm.CHLabelBottom("Could't generate the file")
                frmMotherForm.CHLabelSmall("")
            End If
           
        End If

    End Function


    '************************************************************************************************
    'Creates SLD Doc: Document Description, Root-Node
    '************************************************************************************************
    Private Function CreateSLD(ByVal FileName As String) As Boolean
        m_objXMLHandle = New XMLHandle(FileName)
        m_objXMLHandle.CreateNewFile(True)
    End Function


    '************************************************************************************************
    'Extracts the data from the structs and writes to SLD
    'Firstly is decided, if the Layers are written in one or in separate files
    '************************************************************************************************
    Public Function WriteToSLD() As Boolean
        Dim i, j, l As Integer
        Dim cLayerName As String
        Dim objFieldValues As ArrayList
        Dim bDoOneLayer As Boolean

        'the decision if separate Layers or one Layer
        If m_bSepFiles = True Then
            bDoOneLayer = False
        Else
            bDoOneLayer = True
            'creation of the SLD with only one File using the user defined filename
            CreateSLD(m_cFilename)
        End If


        Try
            For i = 0 To m_strDataSavings.LayerCount - 1
                cLayerName = m_strDataSavings.LayerList(i).LayerName
                If frmMotherForm.m_enumLang = Motherform.Language.Deutsch Then
                    frmMotherForm.CHLabelBottom("Verarbeite Layer " & cLayerName)
                ElseIf frmMotherForm.m_enumLang = Motherform.Language.English Then
                    frmMotherForm.CHLabelBottom("processing layer " & cLayerName)
                End If

                'Creation of several SLD with that format: /UserDefinedPath/UserDefinedName_LayerName.sld
                If bDoOneLayer = False Then
                    CreateSLD(m_cFilename & "_" & cLayerName & ".sld")
                End If

                'XML-Schreibanweisungen auf Projektebene und Layerebene
                m_objXMLHandle.CreateElement("NamedLayer")
                m_objXMLHandle.CreateElement("LayerName")
                m_objXMLHandle.SetElementText(m_strDataSavings.LayerList(i).DatasetName)
                m_objXMLHandle.CreateElement("UserStyle")
                m_objXMLHandle.CreateElement("StyleName")
                m_objXMLHandle.SetElementText("Style1")
                m_objXMLHandle.CreateElement("FeatureTypeStyle")
                m_objXMLHandle.CreateElement("FeatureTypeName")
                m_objXMLHandle.SetElementText(m_strDataSavings.LayerList(i).DatasetName)

                'XML-Schreibanweisungen auf Layerebene und auf Symbolebene
                Dim objSymbols As ArrayList   'Die ArrayList mit den Symbolen eines Layers
                objSymbols = m_strDataSavings.LayerList(i).SymbolList
                For j = 0 To objSymbols.Count - 1   'IN DER SCHLEIFE AUF SYMBOLEBENE objSymbols(j) reprsentiert 1 Symbol!!!
                    If frmMotherForm.m_enumLang = Motherform.Language.Deutsch Then
                        frmMotherForm.CHLabelSmall("Symbol " & (j + 1).ToString & " von " & objSymbols.Count.ToString)
                    ElseIf frmMotherForm.m_enumLang = Motherform.Language.English Then
                        frmMotherForm.CHLabelSmall("Symbol " & (j + 1).ToString & " of " & objSymbols.Count.ToString)
                    End If

                    '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                    'HIER DIE UNTERSCHEIDUNGEN NACH DEN EINZELNEN RENDERERN: UNIQUEVALUERENDERER
                    If TypeOf m_strDataSavings.LayerList(i) Is StructUniqueValueRenderer Then
                        Dim objStructUVR As StructUniqueValueRenderer
                        objStructUVR = m_strDataSavings.LayerList(i)   'Zuweisung des StructsUVR. Reprsentiert je einen Layer!!!
                        m_objXMLHandle.CreateElement("Rule")
                        m_objXMLHandle.CreateElement("RuleName")
                        m_objXMLHandle.SetElementText(objSymbols(j).Label)
                        m_objXMLHandle.CreateElement("Title")
                        m_objXMLHandle.SetElementText(objSymbols(j).Label)
                        m_objXMLHandle.CreateElement("Filter")
                        If frmMotherForm.chkScale.Checked = True Then
                            m_objXMLHandle.CreateElement("MinScale")
                            m_objXMLHandle.SetElementText(frmMotherForm.cboLowScale.Text)
                            m_objXMLHandle.CreateElement("MaxScale")
                            m_objXMLHandle.SetElementText(frmMotherForm.cboHighScale.Text)
                        End If
                        objFieldValues = objSymbols(j).Fieldvalues
                        If objStructUVR.FieldCount > 1 Then    'Nur wenn nach mehr als 1 Feld klassifiziert wurde, wird der <AND>-Tag gesetzt
                            m_objXMLHandle.CreateElement("And")
                            For l = 0 To objStructUVR.FieldCount - 1   'Die Schleife ist nur dafr da, falls nach mehreren Feldern klassifiziert wurde
                                With m_objXMLHandle
                                    .CreateElement("PropertyIsEqualTo") 'Sie schreibt pro Feld nach dem klass. wurde das <PropertyIsEqualTo> und alle Kinder
                                    .CreateElement("PropertyName")
                                    .SetElementText(objStructUVR.FieldNames(l))
                                    .CreateElement("Fieldvalue")
                                    .SetElementText(objFieldValues(l))
                                End With
                            Next
                        ElseIf objStructUVR.FieldCount = 1 Then
                            If objFieldValues.Count > 1 Then
                                m_objXMLHandle.CreateElement("Or")
                            End If
                            For l = 0 To objFieldValues.Count - 1  'If multiple values grouped in same class
                                With m_objXMLHandle
                                    .CreateElement("PropertyIsEqualTo") 'Sie schreibt pro Feld nach dem klass. wurde das <PropertyIsEqualTo> und alle Kinder
                                    .CreateElement("PropertyName")
                                    .SetElementText(objStructUVR.FieldNames(0))
                                    .CreateElement("Fieldvalue")
                                    .SetElementText(objFieldValues(l))
                                End With
                            Next
                        End If
                        'UNTERSCHEIDUNG NACH FEATURECLASS DES BETREFFENDEN SYMBOLS
                        Select Case objStructUVR.FeatureCls
                            Case FeatureClass.PointFeature
                                WritePointFeatures(objSymbols(j))
                            Case FeatureClass.LineFeature
                                WriteLineFeatures(objSymbols(j))
                            Case FeatureClass.PolygonFeature
                                WritePolygonFeatures(objSymbols(j))
                        End Select
                        '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                        'HIER DIE UNTERSCHEIDUNGEN NACH DEN EINZELNEN RENDERERN: CLASSBREAKSRENDERER
                    ElseIf TypeOf m_strDataSavings.LayerList(i) Is StructClassBreaksRenderer Then
                        Dim objStructCBR As StructClassBreaksRenderer
                        objStructCBR = m_strDataSavings.LayerList(i)
                        m_objXMLHandle.CreateElement("Rule")
                        m_objXMLHandle.CreateElement("RuleName")
                        m_objXMLHandle.SetElementText(objSymbols(j).Label)
                        m_objXMLHandle.CreateElement("Title")
                        m_objXMLHandle.SetElementText(objSymbols(j).Label)
                        m_objXMLHandle.CreateElement("Filter")
                        If frmMotherForm.chkScale.Checked = True Then
                            m_objXMLHandle.CreateElement("MinScale")
                            m_objXMLHandle.SetElementText(frmMotherForm.cboLowScale.Text)
                            m_objXMLHandle.CreateElement("MaxScale")
                            m_objXMLHandle.SetElementText(frmMotherForm.cboHighScale.Text)
                        End If
                        m_objXMLHandle.CreateElement("PropertyIsBetween")
                        m_objXMLHandle.CreateElement("PropertyName")
                        m_objXMLHandle.SetElementText(objStructCBR.FieldName)
                        m_objXMLHandle.CreateElement("LowerBoundary")
                        m_objXMLHandle.CreateElement("Fieldvalue")
                        m_objXMLHandle.SetElementText(CommaToPoint(objSymbols(j).LowerLimit))
                        m_objXMLHandle.CreateElement("UpperBoundary")
                        m_objXMLHandle.CreateElement("Fieldvalue")
                        m_objXMLHandle.SetElementText(CommaToPoint(objSymbols(j).UpperLimit))
                        Select Case objStructCBR.FeatureCls
                            Case FeatureClass.PointFeature
                                WritePointFeatures(objSymbols(j))
                            Case FeatureClass.LineFeature
                                WriteLineFeatures(objSymbols(j))
                            Case FeatureClass.PolygonFeature
                                WritePolygonFeatures(objSymbols(j))
                        End Select
                        'HIER DIE UNTERSCHEIDUNGEN NACH DEN EINZELNEN RENDERERN: SIMPLERENDERER
                    ElseIf TypeOf m_strDataSavings.LayerList(i) Is StructSimpleRenderer Then
                        Dim objStructSR As StructSimpleRenderer
                        objStructSR = m_strDataSavings.LayerList(i)
                        m_objXMLHandle.CreateElement("Rule")
                        m_objXMLHandle.CreateElement("RuleName")
                        m_objXMLHandle.SetElementText(m_strDataSavings.LayerList(i).DatasetName)
                        m_objXMLHandle.CreateElement("Title")
                        m_objXMLHandle.SetElementText(m_strDataSavings.LayerList(i).DatasetName)
                        Select Case objStructSR.FeatureCls
                            Case FeatureClass.PointFeature
                                WritePointFeatures(objSymbols(j))
                            Case FeatureClass.LineFeature
                                WriteLineFeatures(objSymbols(j))
                            Case FeatureClass.PolygonFeature
                                WritePolygonFeatures(objSymbols(j))
                        End Select
                        WriteAnnotation(m_strDataSavings.LayerList(i).Annotation)
                    End If
                Next
                If bDoOneLayer = False Then
                    m_objXMLHandle.SaveDoc()    'If separate layer, the files have to be saved here
                End If
            Next
            If bDoOneLayer = True Then
                m_objXMLHandle.SaveDoc()    'else the file has to be saved here
            End If
            Return True
        Catch ex As Exception
            ErrorMsg("Konnte die SLD nicht schreiben", ex.Message, ex.StackTrace, "WriteToSLD")
            Return False
        End Try
    End Function

    Private Function WriteAnnotation(ByVal Annotation As StructAnnotation) As Boolean
        If Annotation.IsSingleProperty And Annotation.PropertyName <> "" Then
            With m_objXMLHandle
                .CreateElement("TextSymbolizer")
                .CreateElement("TextLabel")
                .CreateElement("TextLabelProperty")
                .SetElementText(Annotation.PropertyName)
                .CreateElement("TextFont")
                .CreateElement("TextFontCssParameter")
                .CreateAttribute("name")
                .SetAttributeValue("font-family")
                .SetElementText(GetValueFromSymbolstruct("TextFont", Annotation.TextSymbol))
                If GetValueFromSymbolstruct("TextFontAlt", Annotation.TextSymbol) <> "" Then
                    .CreateElement("TextFontCssParameter")
                    .CreateAttribute("name")
                    .SetAttributeValue("font-family")
                    .SetElementText(GetValueFromSymbolstruct("TextFontAlt", Annotation.TextSymbol))
                End If
                .CreateElement("TextFontCssParameter")
                .CreateAttribute("name")
                .SetAttributeValue("font-size")
                .SetElementText(GetValueFromSymbolstruct("TextFontSize", Annotation.TextSymbol))
                .CreateElement("TextFontCssParameter")
                .CreateAttribute("name")
                .SetAttributeValue("font-style")
                .SetElementText(GetValueFromSymbolstruct("TextFontStyle", Annotation.TextSymbol))
                .CreateElement("TextFontCssParameter")
                .CreateAttribute("name")
                .SetAttributeValue("font-weight")
                .SetElementText(GetValueFromSymbolstruct("TextFontWeight", Annotation.TextSymbol))
                .CreateElement("TextFill")
                .CreateElement("TextFillCssParameter")
                .CreateAttribute("name")
                .SetAttributeValue("fill")
                .SetElementText(GetValueFromSymbolstruct("TextColor", Annotation.TextSymbol))
                .CreateElement("TextFillCssParameter")
                .CreateAttribute("name")
                .SetAttributeValue("fill-opacity")
                .SetElementText("1.0")
            End With
        End If
        Return True
    End Function

    '************************************************************************************************
    'This function writes the SLD features for a PointSymbolizer based on the passed (point)symbol.
    'Parameter:
    '           Symbol=Die Symbolstruktur des aktuellen Symbols aus der Symbolsammlung    
    '************************************************************************************************
    Private Function WritePointFeatures(ByVal Symbol As Object) As Boolean
        Try
            Dim layerIdx As Integer
            Dim maxLayerIdx As Integer = 1

            If TypeOf Symbol Is StructMultilayerMarkerSymbol Then
                Dim objTempStruct As StructMultilayerMarkerSymbol
                objTempStruct = Symbol
                maxLayerIdx = objTempStruct.LayerCount
            End If
            For layerIdx = 0 To maxLayerIdx - 1
                With m_objXMLHandle
                    .CreateElement("PointSymbolizer")
                    .CreateElement("PointGraphic")
                    .CreateElement("Mark")
                    .CreateElement("PointWellKnownName")
                    .SetElementText(GetValueFromSymbolstruct("WellKnownName", Symbol, layerIdx))
                    If GetValueFromSymbolstruct("PointColor", Symbol, layerIdx) <> "" Then
                        .CreateElement("PointFill")
                        .CreateElement("PointFillCssParameter")
                        .CreateAttribute("name")
                        .SetAttributeValue("fill")
                        .SetElementText(GetValueFromSymbolstruct("PointColor", Symbol, layerIdx))
                        .CreateElement("PointFillCssParameter")
                        .CreateAttribute("name")
                        .SetAttributeValue("fill-opacity")
                        .SetElementText("1.0")
                    End If
                    If GetValueFromSymbolstruct("PointOutlineColor", Symbol, layerIdx) <> "" And GetValueFromSymbolstruct("PointOutlineSize", Symbol, layerIdx) <> "0" Then
                        .CreateElement("PointStroke")
                        .CreateElement("PointStrokeCssParameter")
                        .CreateAttribute("name")
                        .SetAttributeValue("stroke")
                        .SetElementText(GetValueFromSymbolstruct("PointOutlineColor", Symbol, layerIdx))
                        .CreateElement("PointStrokeCssParameter")
                        .CreateAttribute("name")
                        .SetAttributeValue("stroke-width")
                        .SetElementText(GetValueFromSymbolstruct("PointOutlineSize", Symbol, layerIdx))
                        .CreateElement("PointStrokeCssParameter")
                        .CreateAttribute("name")
                        .SetAttributeValue("stroke-opacity")
                        .SetElementText("1.0")
                    End If
                    .CreateElement("PointSize")
                    .SetElementText(GetValueFromSymbolstruct("PointSize", Symbol, layerIdx))
                    .CreateElement("PointRotation")
                    .SetElementText(GetValueFromSymbolstruct("PointRotation", Symbol, layerIdx))
                End With
            Next
            Return True
        Catch ex As Exception
            ErrorMsg("Fehler beim Schreiben der Schraffur", ex.Message, ex.StackTrace, "WritePointFeatures")
            Return False
        End Try
    End Function

    '************************************************************************************************
    'This function writes the SLD features for a LineSymbolizer based on the passed (line)symbol.
    'Parameter:
    '           Symbol=Die Symbolstruktur des aktuellen Symbols aus der Symbolsammlung    
    '************************************************************************************************
    Private Function WriteLineFeatures(ByVal Symbol As Object) As Boolean
        Try
            Dim layerIdx As Integer
            Dim maxLayerIdx As Integer = 1

            If TypeOf Symbol Is StructMultilayerLineSymbol Then
                Dim objTempStruct As StructMultilayerLineSymbol
                objTempStruct = Symbol
                maxLayerIdx = objTempStruct.LayerCount
            End If
            For layerIdx = 0 To maxLayerIdx - 1
                If GetValueFromSymbolstruct("LineColor", Symbol, layerIdx) <> "" And GetValueFromSymbolstruct("LineWidth", Symbol, layerIdx) <> "0" Then
                    With m_objXMLHandle
                        m_objXMLHandle.CreateElement("LineSymbolizer")
                        .CreateElement("LineStroke")
                        .CreateElement("LineCssParameter")
                        .CreateAttribute("name")
                        .SetAttributeValue("stroke")
                        .SetElementText(GetValueFromSymbolstruct("LineColor", Symbol, layerIdx))
                        .CreateElement("LineCssParameter")
                        .CreateAttribute("name")
                        .SetAttributeValue("stroke-width")
                        .SetElementText(GetValueFromSymbolstruct("LineWidth", Symbol, layerIdx))
                        .CreateElement("LineCssParameter")
                        .CreateAttribute("name")
                        .SetAttributeValue("stroke-opacity")
                        .SetElementText(GetValueFromSymbolstruct("LineOpacity", Symbol, layerIdx))
                        If GetValueFromSymbolstruct("LineDashArray", Symbol, layerIdx) <> "" Then
                            .CreateElement("LineCssParameter")
                            .CreateAttribute("name")
                            .SetAttributeValue("stroke-dasharray")
                            .SetElementText(GetValueFromSymbolstruct("LineDashArray", Symbol, layerIdx))
                        End If
                    End With
                End If
            Next
            Return True
        Catch ex As Exception
            ErrorMsg("Fehler beim Schreiben der Schraffur", ex.Message, ex.StackTrace, "WriteLineFeatures")
            Return False
        End Try
    End Function

    '************************************************************************************************
    'Die Funktion bernimmt das aktuelle Symbol und verteilt es auf die Unterfunktionen zum schreiben
    'Bei MultilayerFillsymbol rekursiver Aufruf
    'Parameter:
    '           Symbol=Die Symbolstruktur des aktuellen Symbols aus der Symbolsammlung    
    '************************************************************************************************
    Private Function WritePolygonFeatures(ByVal Symbol As Object) As Boolean
        ' WriteSolidFill(Symbol)
        Dim i, iSecure As Int16
        iSecure = 0
        Try
            If TypeOf Symbol Is StructSimpleFillSymbol Then
                WriteSolidFill(Symbol)
            ElseIf TypeOf Symbol Is StructMarkerFillSymbol Then
                WriteMarkerFill(Symbol)
            ElseIf TypeOf Symbol Is StructLineFillSymbol Then
                'Die Winkel der Schraffuren Original:(Schrg/Horizontal,Vertikal)-SLD(Kreuzschraffur schrg/Kreuzschraffur Achsparallel)
                If Symbol.Angle > 22.5 And Symbol.Angle < 67.5 Then
                    WriteSlopedHatching(Symbol)
                ElseIf Symbol.Angle > 67.5 And Symbol.Angle < 112.5 Then
                    WritePerpendicularHatching(Symbol)
                ElseIf Symbol.Angle > 112.5 And Symbol.Angle < 157.5 Then
                    WriteSlopedHatching(Symbol)
                ElseIf Symbol.Angle > 157.5 And Symbol.Angle < 202.5 Then
                    WritePerpendicularHatching(Symbol)
                ElseIf Symbol.Angle > 202.5 And Symbol.Angle < 247.5 Then
                    WriteSlopedHatching(Symbol)
                ElseIf Symbol.Angle > 247.5 And Symbol.Angle < 292.5 Then
                    WritePerpendicularHatching(Symbol)
                ElseIf Symbol.Angle > 292.5 And Symbol.Angle < 337.5 Then
                    WriteSlopedHatching(Symbol)
                ElseIf Symbol.Angle > 337.5 And Symbol.Angle <= 360.0 Or Symbol.Angle >= 0.0 And Symbol.Angle < 22.5 Then
                    WritePerpendicularHatching(Symbol)
                End If
            ElseIf TypeOf Symbol Is StructDotDensityFillSymbol Then
                WriteMarkerFill(Symbol)  'Ist z.Zt. sowieso nicht mglich, Dichte mit Punktfllungen auszugeben
            ElseIf TypeOf Symbol Is StructPictureFillSymbol Then
                'TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO
            ElseIf TypeOf Symbol Is StructGradientFillSymbol Then
                'TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO TODO
            ElseIf TypeOf Symbol Is StructMultilayerFillSymbol Then
                Dim MFS As StructMultilayerFillSymbol
                MFS = Symbol
                Dim bSwitch As Boolean  'Wenn mehr als 3 Symbollayer sind, und einer davon ist ein SimpleFill
                bSwitch = False
                'Hier muss aufgepasst werden: Manche Mapserver knnen nur 2 Symbollayer bereinander abbilden. Deshalb werden derzeit nur 2 Symbollayer gebildet
                If MFS.LayerCount = 1 Then
                    WritePolygonFeatures(MFS.MultiFillLayers(0))
                ElseIf MFS.LayerCount = 2 Then
                    For i = MFS.LayerCount - 1 To 0 Step -1
                        WritePolygonFeatures(MFS.MultiFillLayers(i))    'hier rekursiver Aufruf
                    Next
                ElseIf MFS.LayerCount > 2 Then
                    For i = MFS.LayerCount - 1 To 0 Step -1
                        If iSecure <= 1 Then
                            WritePolygonFeatures(MFS.MultiFillLayers(i))
                        End If
                        iSecure = iSecure + 1
                    Next
                End If
            End If
            Return True
        Catch ex As Exception
            ErrorMsg("Fehler beim Schreiben der Schraffur", ex.Message, ex.StackTrace, "WritePolygonFeatures")
            Return False
        End Try
    End Function


    '************************************************************************************************
    'Die Funktion schreibt die SLD-Anweisung fr eine SOLID COLOR FLCHENFLLUNG
    'und bernimmt die Eigenschaften aus der SimpleFill-Datenstruktur
    'Parameter:
    '           Symbol=Die Symbolstruktur des aktuellen Symbols aus der Symbolsammlung    
    '************************************************************************************************
    Private Function WriteSolidFill(ByVal Symbol As Object) As Boolean
        Try
            With m_objXMLHandle
                .CreateElement("PolygonSymbolizer")
                If GetValueFromSymbolstruct("PolygonColor", Symbol) <> "" Then
                    .CreateElement("Fill")
                    .CreateElement("PolyCssParameter")
                    .CreateAttribute("name")
                    .SetAttributeValue("fill")
                    .SetElementText(GetValueFromSymbolstruct("PolygonColor", Symbol))
                    .CreateElement("PolyCssParameter")
                    .CreateAttribute("name")
                    .SetAttributeValue("fill-opacity")
                    .SetElementText(GetValueFromSymbolstruct("PolygonOpacity", Symbol))
                End If
                If GetValueFromSymbolstruct("PolygonBorderColor", Symbol) <> "" And GetValueFromSymbolstruct("PolygonBorderWidth", Symbol) <> "0" Then
                    .CreateElement("PolygonStroke")
                    .CreateElement("PolyCssParameter")
                    .CreateAttribute("name")
                    .SetAttributeValue("stroke")
                    .SetElementText(GetValueFromSymbolstruct("PolygonBorderColor", Symbol))
                    .CreateElement("PolyCssParameter")
                    .CreateAttribute("name")
                    .SetAttributeValue("stroke-width")
                    .SetElementText(GetValueFromSymbolstruct("PolygonBorderWidth", Symbol))
                    .CreateElement("PolyCssParameter")
                    .CreateAttribute("name")
                    .SetAttributeValue("stroke-opacity")
                    .SetElementText(GetValueFromSymbolstruct("PolygonBorderOpacity", Symbol))
                End If
            End With
            Return True
        Catch ex As Exception
            ErrorMsg("Fehler beim Schreiben der Flchenfllung", ex.Message, ex.StackTrace, "WriteSimpleFill")
            Return False
        End Try

    End Function

    '************************************************************************************************
    'Die Funktion schreibt die SLD-Anweisung fr eine GEPUNKTETE FLCHENFLLUNG
    'und bernimmt die Eigenschaften aus der MarkerFill-Datenstruktur (und z.Zt. der DotDensity-Strukt.)
    'Parameter:
    '           Symbol=Die Symbolstruktur des aktuellen Symbols aus der Symbolsammlung    
    '************************************************************************************************
    Private Function WriteMarkerFill(ByVal Symbol As Object) As Boolean
        Try
            With m_objXMLHandle
                .CreateElement("PolygonSymbolizer")
                If GetValueFromSymbolstruct("PointColor", Symbol) <> "" Then
                    .CreateElement("Fill")
                    .CreateElement("PolygonGraphicFill")
                    .CreateElement("PolygonGraphic")
                    .CreateElement("PolygonMark")
                    .CreateElement("PolygonSize")
                    .SetElementText(GetValueFromSymbolstruct("PointSize", Symbol))
                    .CreateElement("PolygonWellKnownName")
                    .SetElementText("circle")
                    .CreateElement("PolygonGraphicParamFill")
                    .CreateElement("PolygonGraphicCssParameter")
                    .CreateAttribute("name")
                    .SetAttributeValue("fill")
                    .SetElementText(GetValueFromSymbolstruct("PointColor", Symbol))
                    .CreateElement("PolygonGraphicCssParameter")
                    .CreateAttribute("name")
                    .SetAttributeValue("fill-opacity")
                    .SetElementText("1.0")
                End If
                If GetValueFromSymbolstruct("PolygonBorderColor", Symbol) <> "" And GetValueFromSymbolstruct("PolygonBorderWidth", Symbol) <> "0" Then
                    .CreateElement("PolygonStroke")
                    .CreateElement("PolyCssParameter")
                    .CreateAttribute("name")
                    .SetAttributeValue("stroke")
                    .SetElementText(GetValueFromSymbolstruct("PolygonBorderColor", Symbol))
                    .CreateElement("PolyCssParameter")
                    .CreateAttribute("name")
                    .SetAttributeValue("stroke-width")
                    .SetElementText(GetValueFromSymbolstruct("PolygonBorderWidth", Symbol))
                    .CreateElement("PolyCssParameter")
                    .CreateAttribute("name")
                    .SetAttributeValue("stroke-opacity")
                    .SetElementText(GetValueFromSymbolstruct("PolygonBorderOpacity", Symbol))
                End If
            End With
            Return True
        Catch ex As Exception
            ErrorMsg("Fehler beim Schreiben der Punktfllung", ex.Message, ex.StackTrace, "WriteMarkerFill")
            Return False
        End Try

    End Function

    '************************************************************************************************
    'Die Funktion schreibt die SLD-Anweisung fr eine SCHRGE KREUZSCHRAFFUR-FLCHENFLLUNG
    'und bernimmt die Eigenschaften aus der LineFill-Datenstruktur.
    'Parameter:
    '           Symbol=Die Symbolstruktur des aktuellen Symbols aus der Symbolsammlung    
    '************************************************************************************************
    Private Function WriteSlopedHatching(ByVal Symbol As Object) As Boolean
        Dim dDummy As Double
        Try
            With m_objXMLHandle
                .CreateElement("PolygonSymbolizer")
                If Symbol.Color <> "" Then
                    .CreateElement("Fill")
                    .CreateElement("PolygonGraphicFill")
                    .CreateElement("PolygonGraphic")
                    '.SetElementText(GetValueFromSymbolstruct("LineWidth", Symbol))
                    .CreateElement("PolygonMark")
                    .CreateElement("PolygonSize")
                    'Schraffurgre
                    dDummy = Symbol.Separation + 5
                    .SetElementText(CommaToPoint(dDummy))
                    .CreateElement("PolygonWellKnownName")
                    .SetElementText("x")    'Macht die schrge Kreuzschraffur
                    .CreateElement("PolygonGraphicParamFill")
                    .CreateElement("PolygonGraphicCssParameter")
                    .CreateAttribute("name")
                    .SetAttributeValue("fill")
                    'Die Schraffurfarbe
                    .SetElementText(Symbol.Color)
                    '.SetElementText(GetValueFromSymbolstruct("LineColor", Symbol))
                    .CreateElement("PolygonGraphicCssParameter")
                    .CreateAttribute("name")
                    .SetAttributeValue("fill-opacity")
                    .SetElementText("1.0")
                End If
                If GetValueFromSymbolstruct("PolygonBorderColor", Symbol) <> "" And GetValueFromSymbolstruct("PolygonBorderWidth", Symbol) <> "0" Then
                    .CreateElement("PolygonStroke")
                    .CreateElement("PolyCssParameter")
                    .CreateAttribute("name")
                    .SetAttributeValue("stroke")
                    .SetElementText(GetValueFromSymbolstruct("PolygonBorderColor", Symbol))
                    .CreateElement("PolyCssParameter")
                    .CreateAttribute("name")
                    .SetAttributeValue("stroke-width")
                    .SetElementText(GetValueFromSymbolstruct("PolygonBorderWidth", Symbol))
                    .CreateElement("PolyCssParameter")
                    .CreateAttribute("name")
                    .SetAttributeValue("stroke-opacity")
                    .SetElementText(GetValueFromSymbolstruct("PolygonBorderOpacity", Symbol))
                End If
            End With
            Return True
        Catch ex As Exception
            ErrorMsg("Fehler beim Schreiben der Schraffur", ex.Message, ex.StackTrace, "WriteSlopedHatching")
            Return False
        End Try
    End Function


    '************************************************************************************************
    'Die Funktion schreibt die SLD-Anweisung fr eine SENKRECHTE KREUZSCHRAFFUR-FLCHENFLLUNG
    'und bernimmt die Eigenschaften aus der LineFill-Datenstruktur.
    'Parameter:
    '           Symbol=Die Symbolstruktur des aktuellen Symbols aus der Symbolsammlung    
    '************************************************************************************************
    Private Function WritePerpendicularHatching(ByVal Symbol As Object) As Boolean
        Dim dDummy As Double
        Try
            With m_objXMLHandle
                .CreateElement("PolygonSymbolizer")
                If Symbol.Color <> "" Then
                    .CreateElement("Fill")
                    .CreateElement("PolygonGraphicFill")
                    .CreateElement("PolygonGraphic")
                    '.SetElementText(GetValueFromSymbolstruct("LineWidth", Symbol))
                    .CreateElement("PolygonMark")
                    .CreateElement("PolygonSize")
                    'Schraffurgre
                    dDummy = Symbol.Separation + 5
                    .SetElementText(CommaToPoint(dDummy))
                    .CreateElement("PolygonWellKnownName")
                    .SetElementText("cross")
                    .CreateElement("PolygonGraphicParamFill")
                    .CreateElement("PolygonGraphicCssParameter")
                    .CreateAttribute("name")
                    .SetAttributeValue("fill")
                    'Die Schraffurfarbe
                    .SetElementText(Symbol.Color)
                    '.SetElementText(GetValueFromSymbolstruct("LineColor", Symbol))
                    .CreateElement("PolygonGraphicCssParameter")
                    .CreateAttribute("name")
                    .SetAttributeValue("fill-opacity")
                    .SetElementText("1.0")
                End If
                If GetValueFromSymbolstruct("PolygonBorderColor", Symbol) <> "" And GetValueFromSymbolstruct("PolygonBorderWidth", Symbol) <> "0" Then
                    .CreateElement("PolygonStroke")
                    .CreateElement("PolyCssParameter")
                    .CreateAttribute("name")
                    .SetAttributeValue("stroke")
                    .SetElementText(GetValueFromSymbolstruct("PolygonBorderColor", Symbol))
                    .CreateElement("PolyCssParameter")
                    .CreateAttribute("name")
                    .SetAttributeValue("stroke-width")
                    .SetElementText(GetValueFromSymbolstruct("PolygonBorderWidth", Symbol))
                    .CreateElement("PolyCssParameter")
                    .CreateAttribute("name")
                    .SetAttributeValue("stroke-opacity")
                    .SetElementText(GetValueFromSymbolstruct("PolygonBorderOpacity", Symbol))
                End If
            End With
            Return True
        Catch ex As Exception
            ErrorMsg("Fehler beim Schreiben der Punktfllung", ex.Message, ex.StackTrace, "WritePerpendicularHatching")
            Return False
        End Try
    End Function

#End Region


#Region "Symbolstructanalysefunktionen"


    '******************************************************************************************************************
    'DIESE FUNKTION GILT Z.ZT. NUR ZU TESTZWECKEN UND MUSS IM REALEN GEBRAUCH VERVOLLSTNDIGT WERDEN !!!
    '1. Die Funktion extrahiert die gewnschten Werte aus den jeweiligen Datenstrukturen
    '2. Hier werden die Generalisierungen gemacht von der komplexen ArcMap-Symbolisierung zur einfachen sld-Symbolisierung
    'TODO: Vervollstndigung der case's fr jede Struktur
    'Parameter:     SymbolStructure= die jeweilige Symbol-Datenstruktur
    '               ValueNameOfValueYouWant= ein String, der den Wert reprsentiert, den man erhalten mchte
    'z.Zt. mgliche Werte fr ValueNameOfValueYouWant sind:
    'Fr Polygone:  "PolygonColor, PolygonBorderWidth, PolygonBorderColor, PointSize, PointColor, LineWidth, LineColor"
    'Fr Linien:    "LineWidth, LineColor"
    'Fr Punkte:    "PointColor, PointSize, PointRotation, PointOutlineColor, WellKnownName"
    '******************************************************************************************************************
    Private Function GetValueFromSymbolstruct(ByVal ValueNameOfValueYouWant As String, ByVal SymbolStructure As Object) As String
        Return GetValueFromSymbolstruct(ValueNameOfValueYouWant, SymbolStructure, 0)
    End Function

    '******************************************************************************************************************
    'DIESE FUNKTION GILT Z.ZT. NUR ZU TESTZWECKEN UND MUSS IM REALEN GEBRAUCH VERVOLLSTNDIGT WERDEN !!!
    '1. Die Funktion extrahiert die gewnschten Werte aus den jeweiligen Datenstrukturen
    '2. Hier werden die Generalisierungen gemacht von der komplexen ArcMap-Symbolisierung zur einfachen sld-Symbolisierung
    'TODO: Vervollstndigung der case's fr jede Struktur
    'Parameter:     SymbolStructure= die jeweilige Symbol-Datenstruktur
    '               ValueNameOfValueYouWant= ein String, der den Wert reprsentiert, den man erhalten mchte
    '               LayerIdx = for multilayer symbols, the index of the layer to use.
    'z.Zt. mgliche Werte fr ValueNameOfValueYouWant sind:
    'Fr Polygone:  "PolygonColor, PolygonBorderWidth, PolygonBorderColor, PointSize, PointColor, LineWidth, LineColor"
    'Fr Linien:    "LineWidth, LineColor"
    'Fr Punkte:    "PointColor, PointSize, PointRotation, PointOutlineColor, WellKnownName"
    '******************************************************************************************************************
    Private Function GetValueFromSymbolstruct(ByVal ValueNameOfValueYouWant As String, ByVal SymbolStructure As Object, ByVal LayerIdx As Integer) As String
        Dim cReturn As String
        Dim bSwitch As Boolean
        bSwitch = False '(bentigt fr Multilayersymbole)der Schalter wird umgelegt, wenn es kein simple.. Symbol gibt. Dann wird der Wert des ersten Symbols genommen 
        cReturn = "0"   'Wenn keiner der bergebenen ValueNames passt, wird 0 zurckgegeben
        Try
            If TypeOf SymbolStructure Is StructSimpleMarkerSymbol Then
                cReturn = GetMarkerValue(ValueNameOfValueYouWant, SymbolStructure)
                Return cReturn
            ElseIf TypeOf SymbolStructure Is StructCharacterMarkerSymbol Then
                cReturn = GetMarkerValue(ValueNameOfValueYouWant, SymbolStructure)
                Return cReturn
            ElseIf TypeOf SymbolStructure Is StructPictureMarkerSymbol Then
                cReturn = GetMarkerValue(ValueNameOfValueYouWant, SymbolStructure)
                Return cReturn
            ElseIf TypeOf SymbolStructure Is StructArrowMarkerSymbol Then
                cReturn = GetMarkerValue(ValueNameOfValueYouWant, SymbolStructure)
                Return cReturn
            ElseIf TypeOf SymbolStructure Is StructSimpleLineSymbol Then
                Dim objTempStruct As StructSimpleLineSymbol
                objTempStruct = SymbolStructure
                Select Case ValueNameOfValueYouWant.ToUpper
                    Case "LineWidth".ToUpper
                        cReturn = CommaToPoint(objTempStruct.Width)
                    Case "LineColor".ToUpper
                        cReturn = objTempStruct.Color
                    Case "LineOpacity".ToUpper
                        Dim tmpTransparency As Double = objTempStruct.Transparency
                        cReturn = CommaToPoint(tmpTransparency / 255.0)
                        'Case "PointRotation".ToUpper
                    Case "LineDashArray".ToUpper
                        Select Case objTempStruct.publicStyle
                            Case "esriSLSDash"
                                cReturn = "10.0 10.0"
                            Case "esriSLSDashDot"
                                cReturn = "10.0 10.0 1.0 10.0"
                            Case "esriSLSDashDotDot"
                                cReturn = "10.0 10.0 1.0 10.0 1.0 10.0"
                            Case "esriSLSDot"
                                cReturn = "1.0 5.0"
                            Case Else
                                cReturn = ""
                        End Select
                End Select
                Return cReturn
            ElseIf TypeOf SymbolStructure Is StructCartographicLineSymbol Then
                Dim objTempStruct As StructCartographicLineSymbol
                objTempStruct = SymbolStructure
                Select Case ValueNameOfValueYouWant.ToUpper
                    Case "LineWidth".ToUpper
                        cReturn = CommaToPoint(objTempStruct.Width)
                    Case "LineColor".ToUpper
                        cReturn = objTempStruct.Color
                    Case "LineOpacity".ToUpper
                        Dim tmpTransparency As Double = objTempStruct.Transparency
                        cReturn = CommaToPoint(tmpTransparency / 255.0)
                    Case "LineDashArray".ToUpper
                        Dim dashIdx As Integer
                        Dim size As Double
                        cReturn = ""
                        For dashIdx = 0 To objTempStruct.DashArray.Count - 1
                            If dashIdx > 0 Then
                                cReturn = cReturn + " "
                            End If
                            size = objTempStruct.DashArray.Item(dashIdx)
                            cReturn = cReturn + CommaToPoint(size)
                        Next
                    Case ""

                End Select
            ElseIf TypeOf SymbolStructure Is StructHashLineSymbol Then
                Dim objTempStruct As StructHashLineSymbol
                objTempStruct = SymbolStructure
                Select Case ValueNameOfValueYouWant.ToUpper
                    Case "LineWidth".ToUpper, "LineDashArray".ToUpper
                        Select Case objTempStruct.kindOfLineStruct
                            Case LineStructs.StructCartographicLineSymbol
                                cReturn = GetValueFromSymbolstruct(ValueNameOfValueYouWant, objTempStruct.HashSymbol_CartographicLine)
                            Case LineStructs.StructMarkerLineSymbol
                                cReturn = GetValueFromSymbolstruct(ValueNameOfValueYouWant, objTempStruct.HashSymbol_MarkerLine)
                            Case LineStructs.StructMultilayerLineSymbol
                                cReturn = GetValueFromSymbolstruct(ValueNameOfValueYouWant, objTempStruct.HashSymbol_MultiLayerLines, LayerIdx)
                            Case LineStructs.StructPictureLineSymbol
                                cReturn = GetValueFromSymbolstruct(ValueNameOfValueYouWant, objTempStruct.HashSymbol_PictureLine)
                            Case LineStructs.StructSimpleLineSymbol
                                cReturn = GetValueFromSymbolstruct(ValueNameOfValueYouWant, objTempStruct.HashSymbol_SimpleLine)
                        End Select
                    Case "LineColor".ToUpper
                        cReturn = objTempStruct.Color
                    Case "LineOpacity".ToUpper
                        Dim tmpTransparency As Double = objTempStruct.Transparency
                        cReturn = CommaToPoint(tmpTransparency / 255.0)
                    Case ""

                End Select
                Return cReturn
            ElseIf TypeOf SymbolStructure Is StructMarkerLineSymbol Then
                Dim objTempStruct As StructMarkerLineSymbol
                objTempStruct = SymbolStructure
                Select Case ValueNameOfValueYouWant.ToUpper
                    Case "LineWidth".ToUpper
                        InfoMsg("Abfrage von Linienbreite der Markerlines ist im Augenblick nicht implementiert", "GetValueFromSymbolstruct")
                    Case "LineColor".ToUpper
                        cReturn = objTempStruct.Color
                    Case "LineOpacity".ToUpper
                        Dim tmpTransparency As Double = objTempStruct.Transparency
                        cReturn = CommaToPoint(tmpTransparency / 255.0)
                    Case ""

                End Select
                Return cReturn
            ElseIf TypeOf SymbolStructure Is StructPictureLineSymbol Then
                Dim objTempStruct As StructPictureLineSymbol
                objTempStruct = SymbolStructure
                Select Case ValueNameOfValueYouWant.ToUpper
                    Case "LineWidth".ToUpper
                        cReturn = CommaToPoint(objTempStruct.Width)
                    Case "LineColor".ToUpper
                        cReturn = objTempStruct.BackgroundColor
                    Case "LineOpacity".ToUpper
                        Dim tmpTransparency As Double = objTempStruct.BackgroundTransparency
                        cReturn = CommaToPoint(tmpTransparency / 255.0)
                    Case ""

                End Select
                Return cReturn
            ElseIf TypeOf SymbolStructure Is StructSimpleFillSymbol Then
                Dim objTempStruct As StructSimpleFillSymbol
                objTempStruct = SymbolStructure
                Select Case ValueNameOfValueYouWant.ToUpper
                    Case "PolygonColor".ToUpper
                        cReturn = objTempStruct.Color
                    Case "PolygonOpacity".ToUpper
                        Dim tmpTransparency As Double = objTempStruct.Transparency
                        cReturn = CommaToPoint(tmpTransparency / 255.0)
                    Case "PolygonBorderWidth".ToUpper
                        Select Case objTempStruct.kindOfOutlineStruct
                            Case LineStructs.StructHashLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_HashLine.Width)
                            Case LineStructs.StructMarkerLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_MarkerLine.Width)
                            Case LineStructs.StructMultilayerLineSymbol
                                cReturn = GetValueFromSymbolstruct("LineWidth", objTempStruct.Outline_MultiLayerLines)
                            Case LineStructs.StructPictureLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_PictureLine.Width)
                            Case LineStructs.StructSimpleLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_SimpleLine.Width)
                        End Select
                    Case "PolygonBorderColor".ToUpper
                        Select Case objTempStruct.kindOfOutlineStruct
                            Case LineStructs.StructHashLineSymbol
                                cReturn = objTempStruct.Outline_HashLine.Color
                            Case LineStructs.StructMarkerLineSymbol
                                cReturn = objTempStruct.Outline_MarkerLine.Color
                            Case LineStructs.StructMultilayerLineSymbol
                                cReturn = GetValueFromSymbolstruct("LineColor", objTempStruct.Outline_MultiLayerLines)
                            Case LineStructs.StructPictureLineSymbol
                                cReturn = objTempStruct.Outline_PictureLine.Color
                            Case LineStructs.StructSimpleLineSymbol
                                cReturn = objTempStruct.Outline_SimpleLine.Color
                        End Select
                    Case "PolygonBorderOpacity".ToUpper
                        Dim tmpTransparency As Double = 255.0
                        Select Case objTempStruct.kindOfOutlineStruct
                            Case LineStructs.StructHashLineSymbol
                                tmpTransparency = objTempStruct.Outline_HashLine.Transparency
                            Case LineStructs.StructMarkerLineSymbol
                                tmpTransparency = objTempStruct.Outline_MarkerLine.Transparency
                            Case LineStructs.StructMultilayerLineSymbol
                                tmpTransparency = 255 * GetValueFromSymbolstruct("LineOpacity", objTempStruct.Outline_MultiLayerLines)
                            Case LineStructs.StructPictureLineSymbol
                                tmpTransparency = objTempStruct.Outline_PictureLine.Transparency
                            Case LineStructs.StructSimpleLineSymbol
                                tmpTransparency = objTempStruct.Outline_SimpleLine.Transparency
                        End Select
                        cReturn = CommaToPoint(tmpTransparency / 255.0)
                End Select
                Return cReturn
            ElseIf TypeOf SymbolStructure Is StructMarkerFillSymbol Then
                Dim objTempStruct As StructMarkerFillSymbol
                objTempStruct = SymbolStructure
                Select Case ValueNameOfValueYouWant.ToUpper
                    Case "PolygonColor".ToUpper
                        cReturn = objTempStruct.Color
                    Case "PolygonOpacity".ToUpper
                        Dim tmpTransparency As Double = objTempStruct.Transparency
                        cReturn = CommaToPoint(tmpTransparency / 255.0)
                    Case "PolygonBorderWidth".ToUpper
                        Select Case objTempStruct.kindOfOutlineStruct
                            Case LineStructs.StructHashLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_HashLine.Width)
                            Case LineStructs.StructMarkerLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_MarkerLine.Width)
                            Case LineStructs.StructMultilayerLineSymbol
                                cReturn = GetValueFromSymbolstruct("LineWidth", objTempStruct.Outline_MultiLayerLines)
                            Case LineStructs.StructPictureLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_PictureLine.Width)
                            Case LineStructs.StructSimpleLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_SimpleLine.Width)
                        End Select
                    Case "PolygonBorderColor".ToUpper
                        Select Case objTempStruct.kindOfOutlineStruct
                            Case LineStructs.StructHashLineSymbol
                                cReturn = objTempStruct.Outline_HashLine.Color
                            Case LineStructs.StructMarkerLineSymbol
                                cReturn = objTempStruct.Outline_MarkerLine.Color
                            Case LineStructs.StructMultilayerLineSymbol
                                cReturn = GetValueFromSymbolstruct("LineColor", objTempStruct.Outline_MultiLayerLines)
                            Case LineStructs.StructPictureLineSymbol
                                cReturn = objTempStruct.Outline_PictureLine.Color
                            Case LineStructs.StructSimpleLineSymbol
                                cReturn = objTempStruct.Outline_SimpleLine.Color
                        End Select
                    Case "PolygonBorderOpacity".ToUpper
                        Dim tmpTransparency As Double = 255.0
                        Select Case objTempStruct.kindOfOutlineStruct
                            Case LineStructs.StructHashLineSymbol
                                tmpTransparency = objTempStruct.Outline_HashLine.Transparency
                            Case LineStructs.StructMarkerLineSymbol
                                tmpTransparency = objTempStruct.Outline_MarkerLine.Transparency
                            Case LineStructs.StructMultilayerLineSymbol
                                tmpTransparency = 255 * GetValueFromSymbolstruct("LineOpacity", objTempStruct.Outline_MultiLayerLines)
                            Case LineStructs.StructPictureLineSymbol
                                tmpTransparency = objTempStruct.Outline_PictureLine.Transparency
                            Case LineStructs.StructSimpleLineSymbol
                                tmpTransparency = objTempStruct.Outline_SimpleLine.Transparency
                        End Select
                        cReturn = CommaToPoint(tmpTransparency / 255.0)
                    Case "PointSize".ToUpper
                        Select Case objTempStruct.kindOfMarkerStruct
                            Case MarkerStructs.StructArrowMarkerSymbol
                                cReturn = CommaToPoint(objTempStruct.MarkerSymbol_ArrowMarker.Size)
                            Case MarkerStructs.StructCharacterMarkerSymbol
                                cReturn = CommaToPoint(objTempStruct.MarkerSymbol_CharacterMarker.Size)
                            Case MarkerStructs.StructMultilayerMarkerSymbol
                                cReturn = GetValueFromSymbolstruct("PointSize", objTempStruct.MarkerSymbol_MultilayerMarker)
                            Case MarkerStructs.StructPictureMarkerSymbol
                                cReturn = CommaToPoint(objTempStruct.MarkerSymbol_PictureMarker.Size)
                            Case MarkerStructs.StructSimpleMarkerSymbol
                                cReturn = CommaToPoint(objTempStruct.MarkerSymbol_SimpleMarker.Size)
                        End Select
                    Case "PointColor".ToUpper
                        Select Case objTempStruct.kindOfMarkerStruct
                            Case MarkerStructs.StructArrowMarkerSymbol
                                cReturn = objTempStruct.MarkerSymbol_ArrowMarker.Color
                            Case MarkerStructs.StructCharacterMarkerSymbol
                                cReturn = objTempStruct.MarkerSymbol_CharacterMarker.Color
                            Case MarkerStructs.StructMultilayerMarkerSymbol
                                cReturn = GetValueFromSymbolstruct("PointColor", objTempStruct.MarkerSymbol_MultilayerMarker)
                            Case MarkerStructs.StructPictureMarkerSymbol
                                cReturn = objTempStruct.MarkerSymbol_PictureMarker.Color
                            Case MarkerStructs.StructSimpleMarkerSymbol
                                cReturn = objTempStruct.MarkerSymbol_SimpleMarker.Color
                        End Select
                End Select
                Return cReturn
            ElseIf TypeOf SymbolStructure Is StructLineFillSymbol Then
                Dim objTempStruct As StructLineFillSymbol
                objTempStruct = SymbolStructure
                Select Case ValueNameOfValueYouWant.ToUpper
                    Case "PolygonColor".ToUpper
                        cReturn = objTempStruct.Color
                    Case "PolygonOpacity".ToUpper
                        Dim tmpTransparency As Double = objTempStruct.Transparency
                        cReturn = CommaToPoint(tmpTransparency / 255.0)
                    Case "PolygonBorderWidth".ToUpper
                        Select Case objTempStruct.kindOfOutlineStruct
                            Case LineStructs.StructHashLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_HashLine.Width)
                            Case LineStructs.StructMarkerLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_MarkerLine.Width)
                            Case LineStructs.StructMultilayerLineSymbol
                                cReturn = GetValueFromSymbolstruct("LineWidth", objTempStruct.Outline_MultiLayerLines)
                            Case LineStructs.StructPictureLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_PictureLine.Width)
                            Case LineStructs.StructSimpleLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_SimpleLine.Width)
                        End Select
                    Case "PolygonBorderColor".ToUpper
                        Select Case objTempStruct.kindOfOutlineStruct
                            Case LineStructs.StructHashLineSymbol
                                cReturn = objTempStruct.Outline_HashLine.Color
                            Case LineStructs.StructMarkerLineSymbol
                                cReturn = objTempStruct.Outline_MarkerLine.Color
                            Case LineStructs.StructMultilayerLineSymbol
                                cReturn = GetValueFromSymbolstruct("LineColor", objTempStruct.Outline_MultiLayerLines)
                            Case LineStructs.StructPictureLineSymbol
                                cReturn = objTempStruct.Outline_PictureLine.Color
                            Case LineStructs.StructSimpleLineSymbol
                                cReturn = objTempStruct.Outline_SimpleLine.Color
                        End Select
                    Case "PolygonBorderOpacity".ToUpper
                        Dim tmpTransparency As Double = 255.0
                        Select Case objTempStruct.kindOfOutlineStruct
                            Case LineStructs.StructHashLineSymbol
                                tmpTransparency = objTempStruct.Outline_HashLine.Transparency
                            Case LineStructs.StructMarkerLineSymbol
                                tmpTransparency = objTempStruct.Outline_MarkerLine.Transparency
                            Case LineStructs.StructMultilayerLineSymbol
                                tmpTransparency = 255 * GetValueFromSymbolstruct("LineOpacity", objTempStruct.Outline_MultiLayerLines)
                            Case LineStructs.StructPictureLineSymbol
                                tmpTransparency = objTempStruct.Outline_PictureLine.Transparency
                            Case LineStructs.StructSimpleLineSymbol
                                tmpTransparency = objTempStruct.Outline_SimpleLine.Transparency
                        End Select
                        cReturn = CommaToPoint(tmpTransparency / 255.0)
                    Case "LineWidth".ToUpper
                        Select Case objTempStruct.kindOfLineStruct
                            Case LineStructs.StructCartographicLineSymbol
                                cReturn = CommaToPoint(objTempStruct.LineSymbol_CartographicLine.Width)
                            Case LineStructs.StructHashLineSymbol
                                cReturn = CommaToPoint(objTempStruct.LineSymbol_HashLine.Width)
                            Case LineStructs.StructMarkerLineSymbol
                                cReturn = CommaToPoint(objTempStruct.LineSymbol_MarkerLine.Width)
                            Case LineStructs.StructMultilayerLineSymbol
                                cReturn = GetValueFromSymbolstruct("LineWidth", objTempStruct.LineSymbol_MultiLayerLines)
                            Case LineStructs.StructPictureLineSymbol
                                cReturn = CommaToPoint(objTempStruct.LineSymbol_PictureLine.Width)
                            Case LineStructs.StructSimpleLineSymbol
                                cReturn = CommaToPoint(objTempStruct.LineSymbol_SimpleLine.Width)
                        End Select
                    Case "LineColor".ToUpper
                        Select Case objTempStruct.kindOfLineStruct
                            Case LineStructs.StructCartographicLineSymbol
                                cReturn = objTempStruct.LineSymbol_CartographicLine.Color
                            Case LineStructs.StructHashLineSymbol
                                cReturn = objTempStruct.LineSymbol_HashLine.Color
                            Case LineStructs.StructMarkerLineSymbol
                                cReturn = objTempStruct.LineSymbol_MarkerLine.Color
                            Case LineStructs.StructMultilayerLineSymbol
                                cReturn = GetValueFromSymbolstruct("LineColor", objTempStruct.LineSymbol_MultiLayerLines)
                            Case LineStructs.StructPictureLineSymbol
                                cReturn = objTempStruct.LineSymbol_PictureLine.Color
                            Case LineStructs.StructSimpleLineSymbol
                                cReturn = objTempStruct.LineSymbol_SimpleLine.Color
                        End Select
                    Case "LineOpacity".ToUpper
                        Dim tmpTransparency As Double = 255.0
                        Select Case objTempStruct.kindOfLineStruct
                            Case LineStructs.StructCartographicLineSymbol
                                tmpTransparency = objTempStruct.LineSymbol_CartographicLine.Transparency
                            Case LineStructs.StructHashLineSymbol
                                tmpTransparency = objTempStruct.LineSymbol_HashLine.Transparency
                            Case LineStructs.StructMarkerLineSymbol
                                tmpTransparency = objTempStruct.LineSymbol_MarkerLine.Transparency
                            Case LineStructs.StructMultilayerLineSymbol
                                tmpTransparency = 255 * GetValueFromSymbolstruct("LineOpacity", objTempStruct.LineSymbol_MultiLayerLines)
                            Case LineStructs.StructPictureLineSymbol
                                tmpTransparency = objTempStruct.LineSymbol_PictureLine.Transparency
                            Case LineStructs.StructSimpleLineSymbol
                                tmpTransparency = objTempStruct.LineSymbol_SimpleLine.Transparency
                        End Select
                        cReturn = CommaToPoint(tmpTransparency / 255.0)
                End Select
                Return cReturn
            ElseIf TypeOf SymbolStructure Is StructDotDensityFillSymbol Then
                Dim objTempStruct As StructDotDensityFillSymbol
                objTempStruct = SymbolStructure
                Select Case ValueNameOfValueYouWant.ToUpper
                    Case "PolygonColor".ToUpper
                        cReturn = objTempStruct.BackgroundColor
                    Case "PolygonOpacity".ToUpper
                        Dim tmpTransparency As Double = objTempStruct.BackgroundTransparency
                        cReturn = CommaToPoint(tmpTransparency / 255.0)
                    Case "PolygonBorderWidth".ToUpper
                        Select Case objTempStruct.kindOfOutlineStruct
                            Case LineStructs.StructHashLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_HashLine.Width)
                            Case LineStructs.StructMarkerLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_MarkerLine.Width)
                            Case LineStructs.StructMultilayerLineSymbol
                                cReturn = GetValueFromSymbolstruct("LineWidth", objTempStruct.Outline_MultiLayerLines)
                            Case LineStructs.StructPictureLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_PictureLine.Width)
                            Case LineStructs.StructSimpleLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_SimpleLine.Width)
                        End Select
                    Case "PolygonBorderColor".ToUpper
                        Select Case objTempStruct.kindOfOutlineStruct
                            Case LineStructs.StructHashLineSymbol
                                cReturn = objTempStruct.Outline_HashLine.Color
                            Case LineStructs.StructMarkerLineSymbol
                                cReturn = objTempStruct.Outline_MarkerLine.Color
                            Case LineStructs.StructMultilayerLineSymbol
                                cReturn = GetValueFromSymbolstruct("LineColor", objTempStruct.Outline_MultiLayerLines)
                            Case LineStructs.StructPictureLineSymbol
                                cReturn = objTempStruct.Outline_PictureLine.Color
                            Case LineStructs.StructSimpleLineSymbol
                                cReturn = objTempStruct.Outline_SimpleLine.Color
                        End Select
                    Case "PolygonBorderOpacity".ToUpper
                        Dim tmpTransparency As Double = 255.0
                        Select Case objTempStruct.kindOfOutlineStruct
                            Case LineStructs.StructHashLineSymbol
                                tmpTransparency = objTempStruct.Outline_HashLine.Transparency
                            Case LineStructs.StructMarkerLineSymbol
                                tmpTransparency = objTempStruct.Outline_MarkerLine.Transparency
                            Case LineStructs.StructMultilayerLineSymbol
                                tmpTransparency = 255 * GetValueFromSymbolstruct("LineOpacity", objTempStruct.Outline_MultiLayerLines)
                            Case LineStructs.StructPictureLineSymbol
                                tmpTransparency = objTempStruct.Outline_PictureLine.Transparency
                            Case LineStructs.StructSimpleLineSymbol
                                tmpTransparency = objTempStruct.Outline_SimpleLine.Transparency
                        End Select
                        cReturn = CommaToPoint(tmpTransparency / 255.0)
                    Case "PointSize".ToUpper
                        cReturn = CommaToPoint(objTempStruct.DotSize)
                    Case "PointColor".ToUpper
                        cReturn = objTempStruct.Color
                End Select
                Return cReturn
            ElseIf TypeOf SymbolStructure Is StructPictureFillSymbol Then
                Dim objTempStruct As StructPictureFillSymbol
                objTempStruct = SymbolStructure
                Select Case ValueNameOfValueYouWant.ToUpper
                    Case "PolygonColor".ToUpper
                        cReturn = objTempStruct.BackgroundColor
                    Case "PolygonOpacity".ToUpper
                        Dim tmpTransparency As Double = objTempStruct.BackgroundTransparency
                        cReturn = CommaToPoint(tmpTransparency / 255.0)
                    Case "PolygonBorderWidth".ToUpper
                        Select Case objTempStruct.kindOfOutlineStruct
                            Case LineStructs.StructHashLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_HashLine.Width)
                            Case LineStructs.StructMarkerLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_MarkerLine.Width)
                            Case LineStructs.StructMultilayerLineSymbol
                                cReturn = GetValueFromSymbolstruct("LineWidth", objTempStruct.Outline_MultiLayerLines)
                            Case LineStructs.StructPictureLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_PictureLine.Width)
                            Case LineStructs.StructSimpleLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_SimpleLine.Width)
                        End Select
                    Case "PolygonBorderColor".ToUpper
                        Select Case objTempStruct.kindOfOutlineStruct
                            Case LineStructs.StructHashLineSymbol
                                cReturn = objTempStruct.Outline_HashLine.Color
                            Case LineStructs.StructMarkerLineSymbol
                                cReturn = objTempStruct.Outline_MarkerLine.Color
                            Case LineStructs.StructMultilayerLineSymbol
                                cReturn = GetValueFromSymbolstruct("LineColor", objTempStruct.Outline_MultiLayerLines)
                            Case LineStructs.StructPictureLineSymbol
                                cReturn = objTempStruct.Outline_PictureLine.Color
                            Case LineStructs.StructSimpleLineSymbol
                                cReturn = objTempStruct.Outline_SimpleLine.Color
                        End Select
                    Case "PolygonBorderOpacity".ToUpper
                        Dim tmpTransparency As Double = 255.0
                        Select Case objTempStruct.kindOfOutlineStruct
                            Case LineStructs.StructHashLineSymbol
                                tmpTransparency = objTempStruct.Outline_HashLine.Transparency
                            Case LineStructs.StructMarkerLineSymbol
                                tmpTransparency = objTempStruct.Outline_MarkerLine.Transparency
                            Case LineStructs.StructMultilayerLineSymbol
                                tmpTransparency = 255 * GetValueFromSymbolstruct("LineOpacity", objTempStruct.Outline_MultiLayerLines)
                            Case LineStructs.StructPictureLineSymbol
                                tmpTransparency = objTempStruct.Outline_PictureLine.Transparency
                            Case LineStructs.StructSimpleLineSymbol
                                tmpTransparency = objTempStruct.Outline_SimpleLine.Transparency
                        End Select
                        cReturn = CommaToPoint(tmpTransparency / 255.0)
                End Select
                Return cReturn
            ElseIf TypeOf SymbolStructure Is StructGradientFillSymbol Then
                Dim objTempStruct As StructGradientFillSymbol
                objTempStruct = SymbolStructure
                Select Case ValueNameOfValueYouWant.ToUpper
                    Case "PolygonColor".ToUpper
                        cReturn = objTempStruct.Color
                    Case "PolygonOpacity".ToUpper
                        Dim tmpTransparency As Double = objTempStruct.Transparency
                        cReturn = CommaToPoint(tmpTransparency / 255.0)
                    Case "PolygonBorderWidth".ToUpper
                        Select Case objTempStruct.kindOfOutlineStruct
                            Case LineStructs.StructHashLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_HashLine.Width)
                            Case LineStructs.StructMarkerLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_MarkerLine.Width)
                            Case LineStructs.StructMultilayerLineSymbol
                                cReturn = GetValueFromSymbolstruct("LineWidth", objTempStruct.Outline_MultiLayerLines)
                            Case LineStructs.StructPictureLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_PictureLine.Width)
                            Case LineStructs.StructSimpleLineSymbol
                                cReturn = CommaToPoint(objTempStruct.Outline_SimpleLine.Width)
                        End Select
                    Case "PolygonBorderColor".ToUpper
                        Select Case objTempStruct.kindOfOutlineStruct
                            Case LineStructs.StructHashLineSymbol
                                cReturn = objTempStruct.Outline_HashLine.Color
                            Case LineStructs.StructMarkerLineSymbol
                                cReturn = objTempStruct.Outline_MarkerLine.Color
                            Case LineStructs.StructMultilayerLineSymbol
                                cReturn = GetValueFromSymbolstruct("LineColor", objTempStruct.Outline_MultiLayerLines)
                            Case LineStructs.StructPictureLineSymbol
                                cReturn = objTempStruct.Outline_PictureLine.Color
                            Case LineStructs.StructSimpleLineSymbol
                                cReturn = objTempStruct.Outline_SimpleLine.Color
                        End Select
                    Case "PolygonBorderOpacity".ToUpper
                        Dim tmpTransparency As Double = 255.0
                        Select Case objTempStruct.kindOfOutlineStruct
                            Case LineStructs.StructHashLineSymbol
                                tmpTransparency = objTempStruct.Outline_HashLine.Transparency
                            Case LineStructs.StructMarkerLineSymbol
                                tmpTransparency = objTempStruct.Outline_MarkerLine.Transparency
                            Case LineStructs.StructMultilayerLineSymbol
                                tmpTransparency = 255 * GetValueFromSymbolstruct("LineOpacity", objTempStruct.Outline_MultiLayerLines)
                            Case LineStructs.StructPictureLineSymbol
                                tmpTransparency = objTempStruct.Outline_PictureLine.Transparency
                            Case LineStructs.StructSimpleLineSymbol
                                tmpTransparency = objTempStruct.Outline_SimpleLine.Transparency
                        End Select
                        cReturn = CommaToPoint(tmpTransparency / 255.0)
                End Select
                Return cReturn
            ElseIf TypeOf SymbolStructure Is StructBarChartSymbol Then
                Dim objTempStruct As StructBarChartSymbol
                objTempStruct = SymbolStructure
                Select Case ValueNameOfValueYouWant.ToUpper
                    Case ""

                    Case ""

                    Case ""

                End Select
                Return cReturn
            ElseIf TypeOf SymbolStructure Is StructPieChartSymbol Then
                Dim objTempStruct As StructPieChartSymbol
                objTempStruct = SymbolStructure
                Select Case ValueNameOfValueYouWant.ToUpper
                    Case ""

                    Case ""

                    Case ""

                End Select
                Return cReturn
            ElseIf TypeOf SymbolStructure Is StructStackedChartSymbol Then
                Dim objTempStruct As StructStackedChartSymbol
                objTempStruct = SymbolStructure
                Select Case ValueNameOfValueYouWant.ToUpper
                    Case ""

                    Case ""

                    Case ""

                End Select
                Return cReturn
            ElseIf TypeOf SymbolStructure Is StructTextSymbol Then
                Dim objTempStruct As StructTextSymbol
                objTempStruct = SymbolStructure
                Select Case ValueNameOfValueYouWant.ToUpper
                    Case "TextColor".ToUpper
                        cReturn = objTempStruct.Color
                    Case "TextFont".ToUpper
                        cReturn = objTempStruct.Font
                    Case "TextFontAlt".ToUpper
                        Select Case objTempStruct.Font.ToUpper
                            Case "ARIAL", "ARIAL BLACK", "HELVETICA", "LUCIDA SANS UNICODE", "MICROSOFT SANS SERIF", "TAHOMA", "VERDANA"
                                cReturn = "Sans-Serif"
                            Case "COURIER", "COURIER NEW", "LUCIDA CONSOLE"
                                cReturn = "Monospaced"
                            Case "PALATINO LINOTYPE", "TIMES", "TIMES NEW ROMAN"
                                cReturn = "Serif"
                        End Select
                    Case "TextFontSize".ToUpper
                        cReturn = objTempStruct.Size
                    Case "TextFontStyle".ToUpper
                        cReturn = objTempStruct.Style
                    Case "TextFontWeight".ToUpper
                        cReturn = objTempStruct.Weight
                End Select
                Return cReturn
                '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
                'Die Multilayer-Symbolstructs
            ElseIf TypeOf SymbolStructure Is StructMultilayerMarkerSymbol Then
                Dim objTempStruct As StructMultilayerMarkerSymbol
                objTempStruct = SymbolStructure
                cReturn = GetValueFromSymbolstruct(ValueNameOfValueYouWant, objTempStruct.MultiMarkerLayers(LayerIdx))
                Return cReturn
            ElseIf TypeOf SymbolStructure Is StructMultilayerLineSymbol Then
                Dim objTempStruct As StructMultilayerLineSymbol
                objTempStruct = SymbolStructure
                Dim i As Int16
                If objTempStruct.LayerCount > 1 Then
                    For i = 0 To objTempStruct.LayerCount - 1
                        If TypeOf objTempStruct.MultiLineLayers(i) Is StructSimpleLineSymbol Then
                            Dim SLFS As StructSimpleLineSymbol
                            SLFS = objTempStruct.MultiLineLayers(i)
                            cReturn = GetValueFromSymbolstruct(ValueNameOfValueYouWant, SLFS)
                            bSwitch = True
                        End If
                    Next
                    If bSwitch = False Then
                        cReturn = GetValueFromSymbolstruct(ValueNameOfValueYouWant, objTempStruct.MultiLineLayers(LayerIdx))
                    End If
                Else
                    cReturn = GetValueFromSymbolstruct(ValueNameOfValueYouWant, objTempStruct.MultiLineLayers(LayerIdx))
                End If
                Return cReturn
            ElseIf TypeOf SymbolStructure Is StructMultilayerFillSymbol Then
                Dim objTempStruct As StructMultilayerFillSymbol
                objTempStruct = SymbolStructure
                Dim i As Int16
                If objTempStruct.LayerCount > 1 Then
                    For i = 0 To objTempStruct.LayerCount - 1
                        Select Case ValueNameOfValueYouWant.ToUpper
                            Case "PolygonColor".ToUpper, "PolygonOpacity".ToUpper, "PolygonBorderWidth".ToUpper, "PolygonBorderColor".ToUpper, "PolygonBorderOpacity".ToUpper
                                If TypeOf objTempStruct.MultiFillLayers(i) Is StructSimpleFillSymbol Then
                                    Dim SSFS As StructSimpleFillSymbol
                                    SSFS = objTempStruct.MultiFillLayers(i)
                                    cReturn = GetValueFromSymbolstruct(ValueNameOfValueYouWant, SSFS)
                                    bSwitch = True
                                End If
                            Case "PointColor".ToUpper, "PointSize".ToUpper
                                If TypeOf objTempStruct.MultiFillLayers(i) Is StructMarkerFillSymbol Then
                                    Dim SMFS As StructMarkerFillSymbol
                                    SMFS = objTempStruct.MultiFillLayers(i)
                                    cReturn = GetValueFromSymbolstruct(ValueNameOfValueYouWant, SMFS)
                                    bSwitch = True
                                End If
                            Case "LineWidth".ToUpper, "LineColor".ToUpper, "LineOpacity".ToUpper
                                If TypeOf objTempStruct.MultiFillLayers(i) Is StructLineFillSymbol Then
                                    Dim SLFS As StructLineFillSymbol
                                    SLFS = objTempStruct.MultiFillLayers(i)
                                    cReturn = GetValueFromSymbolstruct(ValueNameOfValueYouWant, SLFS)
                                    bSwitch = True
                                End If
                        End Select
                    Next
                    If bSwitch = False Then
                        cReturn = GetValueFromSymbolstruct(ValueNameOfValueYouWant, objTempStruct.MultiFillLayers(LayerIdx))
                    End If
                Else
                    cReturn = GetValueFromSymbolstruct(ValueNameOfValueYouWant, objTempStruct.MultiFillLayers(LayerIdx))
                End If
                Return cReturn
            End If
            Return cReturn
        Catch ex As Exception
            ErrorMsg("Konnte den Wert aus der SymbolStruct nicht auswerten.", ex.Message, ex.StackTrace, "GetValueFromSymbolstruct")
        End Try
    End Function

    '******************************************************************************************************************
    'This function tries to guess the SLD marker information from the ESRI point symbol.
    'It is incomplete by default because ESRI has many font sets.
    'The mapping is imperfect since only five well known names exist, with so many symbols in ESRI.
    'Parameter:     SymbolStructure= The point Symbol-Datastructure
    '               ValueNameOfValueYouWant= ein String, der den Wert reprsentiert, den man erhalten mchte
    'z.Zt. mgliche Werte fr ValueNameOfValueYouWant sind:
    '"PointColor, PointSize, PointRotation, PointOutlineColor, WellKnownName"
    '******************************************************************************************************************
    Private Function GetMarkerValue(ByVal ValueNameOfValueYouWant As String, ByVal SymbolStructure As Object) As String
        Dim cReturn As String
        Dim cColor As String
        Dim cOutlineColor As String
        Dim bSwitch As Boolean
        bSwitch = False '(bentigt fr Multilayersymbole)der Schalter wird umgelegt, wenn es kein simple.. Symbol gibt. Dann wird der Wert des ersten Symbols genommen 
        cReturn = "0"   'Wenn keiner der bergebenen ValueNames passt, wird 0 zurckgegeben
        Try
            If TypeOf SymbolStructure Is StructSimpleMarkerSymbol Then
                Dim objTempStruct As StructSimpleMarkerSymbol
                objTempStruct = SymbolStructure
                Select Case ValueNameOfValueYouWant.ToUpper
                    Case "WellKnownName".ToUpper
                        Select Case objTempStruct.Style
                            Case "esriSMSCircle"
                                cReturn = "circle"
                            Case "esriSMSSquare"
                                cReturn = "square"
                            Case "esriSMSCross"
                                cReturn = "cross"
                            Case "esriSMSX"
                                cReturn = "x"
                            Case "esriSMSDiamond"
                                cReturn = "triangle"
                            Case Else
                                cReturn = "star"
                        End Select
                    Case "PointColor".ToUpper
                        If objTempStruct.Filled Then
                            cReturn = objTempStruct.Color
                        Else
                            cReturn = ""
                        End If
                    Case "PointSize".ToUpper
                        cReturn = CommaToPoint(objTempStruct.Size)
                    Case "PointRotation".ToUpper
                        cReturn = CommaToPoint(objTempStruct.Angle)
                    Case "PointOutlineColor".ToUpper
                        If objTempStruct.Outline Then
                            cReturn = objTempStruct.OutlineColor
                        Else
                            cReturn = ""
                        End If
                    Case "PointOutlineSize".ToUpper
                        cReturn = CommaToPoint(objTempStruct.OutlineSize)
                End Select
                Return cReturn
            ElseIf TypeOf SymbolStructure Is StructCharacterMarkerSymbol Then
                Dim objTempStruct As StructCharacterMarkerSymbol
                objTempStruct = SymbolStructure
                Select Case ValueNameOfValueYouWant.ToUpper
                    Case "WellKnownName".ToUpper
                        cReturn = "circle" 'Default
                        Select Case objTempStruct.Font.ToUpper
                            Case "ESRI DEFAULT MARKER"
                                Select Case objTempStruct.CharacterIndex
                                    Case 33, 40, 46, 53, 60 To 67, 72, 79 To 82, 90 To 93, 171, 172, 183, 196, 199, 200, 8729
                                        cReturn = "circle"
                                    Case 34, 41, 47, 54, 74, 83, 84, 104, 174, 175, 179, 190, 192, 194, 198, 201
                                        cReturn = "square"
                                    Case 35, 42, 48, 55, 73, 86, 184, 185
                                        cReturn = "triangle"
                                    Case 68
                                        cReturn = "X"
                                    Case 69, 70, 71, 203 To 211
                                        cReturn = "cross"
                                    Case 94, 95, 96, 106, 107, 108
                                        cReturn = "star"
                                End Select
                            Case "ESRI IGL FONT22"
                                Select Case objTempStruct.CharacterIndex
                                    Case 65 To 69, 93 To 96, 103, 105, 106
                                        cReturn = "circle"
                                    Case 70, 71, 88 To 92, 118 To 121
                                        cReturn = "square"
                                    Case 72, 73, 75, 81, 85, 86, 99 To 102, 104
                                        cReturn = "triangle"
                                    Case 114 To 117
                                        cReturn = "X"
                                End Select
                            Case "ESRI GEOMETRIC SYMBOLS"
                                Select Case objTempStruct.CharacterIndex
                                    Case 33 To 35, 39 To 41, 47, 48, 56 To 58, 65, 68 To 71, 74 To 77, 82, 83, 86 To 89, 92 To 95, 98 To 101, 104 To 107, 110 To 113, 116 To 125, 161, 171, 177 To 186, 244, 246 To 249, 8729
                                        cReturn = "circle"
                                    Case 37, 42, 43, 50, 55, 67, 73, 79, 85, 91, 97, 103, 109, 115, 170, 172, 200 To 205, 208, 209, 210, 226 To 241, 243, 250
                                        cReturn = "square"
                                    Case 36, 46, 49, 66, 72, 78, 84, 90, 96, 102, 108, 114, 162, 168, 169, 175, 176, 186 To 190, 213 To 220, 245
                                        cReturn = "triangle"
                                    Case 195 To 199, 206, 207
                                        cReturn = "X"
                                End Select
                        End Select
                    Case "PointColor".ToUpper, "PointOutlineColor".ToUpper
                        cColor = objTempStruct.Color 'Default
                        cOutlineColor = "" 'Default
                        Select Case objTempStruct.Font.ToUpper
                            Case "ESRI DEFAULT MARKER"
                                Select Case objTempStruct.CharacterIndex
                                    Case 33 To 39, 67 To 69, 71, 81, 88, 97 To 103, 107, 113, 116, 118, 161, 163, 165, 167, 168, 172, 174, 175, 179, 182 To 186, 190, 192 To 201, 203 To 211, 215, 219, 8729
                                        cColor = objTempStruct.Color
                                        cOutlineColor = ""
                                    Case Else
                                        cColor = ""
                                        cOutlineColor = objTempStruct.Color
                                End Select
                            Case "ESRI IGL FONT22"
                                Select Case objTempStruct.CharacterIndex
                                    Case 72 To 80, 100, 118 To 121
                                        cColor = objTempStruct.Color
                                        cOutlineColor = ""
                                    Case Else
                                        cColor = ""
                                        cOutlineColor = objTempStruct.Color
                                End Select
                            Case "ESRI GEOMETRIC SYMBOLS"
                                Select Case objTempStruct.CharacterIndex
                                    Case 34 To 40, 120, 161 To 167, 187, 188, 194 To 200, 202 To 215, 217, 218, 221 To 229, 231 To 249
                                        cColor = objTempStruct.Color
                                        cOutlineColor = ""
                                    Case Else
                                        cColor = ""
                                        cOutlineColor = objTempStruct.Color
                                End Select
                        End Select
                        If ValueNameOfValueYouWant.ToUpper = "PointColor".ToUpper Then
                            cReturn = cColor
                        Else
                            cReturn = cOutlineColor
                        End If
                        Return cReturn
                    Case "PointSize".ToUpper
                        cReturn = CommaToPoint(objTempStruct.Size)
                    Case "PointRotation".ToUpper
                        cReturn = CommaToPoint(objTempStruct.Angle)
                    Case "PointOutlineSize".ToUpper
                        cReturn = "1"
                        Select Case objTempStruct.Font.ToUpper
                            Case "ESRI GEOMETRIC SYMBOLS"
                                Select Case objTempStruct.CharacterIndex
                                    Case 85, 88, 89, 91, 94, 95, 97, 98, 100
                                        cReturn = "2"
                                    Case 41, 65 To 83, 122 To 125, 168, 169, 188, 189, 216, 230, 250
                                        cReturn = "3"
                                    Case 161, 170 To 178, 186
                                        cReturn = "4"
                                End Select
                        End Select
                End Select
                Return cReturn
            ElseIf TypeOf SymbolStructure Is StructPictureMarkerSymbol Then
                Dim objTempStruct As StructPictureMarkerSymbol
                objTempStruct = SymbolStructure
                Select Case ValueNameOfValueYouWant.ToUpper
                    Case "WellKnownName".ToUpper
                        cReturn = "circle" 'TODO
                    Case "PointColor".ToUpper
                        cReturn = objTempStruct.BackgroundColor
                    Case "PointSize".ToUpper
                        cReturn = CommaToPoint(objTempStruct.Size)
                    Case "PointRotation".ToUpper
                        cReturn = CommaToPoint(objTempStruct.Angle)
                    Case "PointOutlineColor".ToUpper
                        cReturn = "" 'TODO
                    Case "PointOutlineSize".ToUpper
                        cReturn = "0" 'TODO
                End Select
                Return cReturn
            ElseIf TypeOf SymbolStructure Is StructArrowMarkerSymbol Then
                Dim objTempStruct As StructArrowMarkerSymbol
                objTempStruct = SymbolStructure
                Select Case ValueNameOfValueYouWant.ToUpper
                    Case "WellKnownName".ToUpper
                        cReturn = "triangle"
                    Case "PointColor".ToUpper
                        cReturn = objTempStruct.Color
                    Case "PointSize".ToUpper
                        cReturn = CommaToPoint(objTempStruct.Size)
                    Case "PointRotation".ToUpper
                        cReturn = CommaToPoint(objTempStruct.Angle)
                    Case "PointOutlineColor".ToUpper
                        cReturn = "" 'Never an outline
                    Case "PointOutlineSize".ToUpper
                        cReturn = "0" 'Never an outline
                End Select
                Return cReturn
            End If
        Catch ex As Exception
            ErrorMsg("Konnte den Wert aus der SymbolStruct nicht auswerten.", ex.Message, ex.StackTrace, "GetValueFromSymbolstruct")
        End Try
        Return cReturn
    End Function
#End Region

#Region "Hilfsfunktionen"

    '************************************************************************************************
    '****** bernimmt einen double der Form X,Y und wandelt ihn in die Form string X.Y **************
    '************************************************************************************************
    Private Function CommaToPoint(ByVal value As Double) As String
        Dim cReturn As String
        cReturn = value.ToString
        cReturn = cReturn.Replace(",", ".")
        Return cReturn
    End Function
    'berladen bernimmt einen String X,Y und wandelt ihn in die Form X.Y
    Private Function CommaToPoint(ByVal value As String) As String
        Dim cReturn As String
        cReturn = value
        cReturn = cReturn.Replace(",", ".")
        Return cReturn
    End Function


    '************************************************************************************************
    '********************************* Die zentrale Fehlermeldung ***********************************
    '************************************************************************************************
    Private Function ErrorMsg(ByVal message As String, ByVal exMessage As String, ByVal stack As String, ByVal functionname As String)
        MessageBox.Show(message & "." & vbCrLf & exMessage & vbCrLf & stack, "ArcGIS_SLD_Converter | Output_SLD | " & functionname, MessageBoxButtons.OK, MessageBoxIcon.Error)
        MyTermination()
    End Function

    '************************************************************************************************
    '********************************* Die zentrale Infomeldung ***********************************
    '************************************************************************************************
    Private Function InfoMsg(ByVal message As String, ByVal functionname As String)
        MessageBox.Show(message, "ArcGIS_SLD_Converter | Output_SLD | " & functionname, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
    End Function

    '************************************************************************************************
    '********************************* Beenden der Anwendung ****************************************
    '************************************************************************************************
    Public Function MyTermination()
        End
        m_objData.MyTermination()
        'oder: application.exit
    End Function


#End Region

End Class
