'####################################################################################################################
'*******************ArcGIS_SLD_Converter / Rel. 1.2.1****************************************************************
'*******************Class: ValidateSLD*******************************************************************************
'*******************AUTHOR: Albrecht Weiser, University of applied Sciences in Mainz, Germany 2005*******************
'The Application was part of my Diploma thesis:**********************************************************************
'"Transforming map-properties of maps in esri-data to an OGC-conformous SLD-document, for publishing the ArcGIS-map * 
'with an OGC- conformous map-server"*********************************************************************************
'ABSTRACT:
'The program so called "ArcGIS-map to SLD Converter" analyses an
'ArcMap-Project with respect to its symbolisation and assembles an SLD
'for the OGC-Web Map Service (WMS) from the gathered data. The program
'is started parallel to a running ArcMap 9.X-session. Subsequently the
'application deposits an SLD-file which complies the symbolisation of
'the available ArcMap-project. With the SLD a WMS-project may be
'classified and styled according to the preceding ArcMap-project. The
'application is written in VisualBasic.NET and uses the .NET 2.0
'Framework (plus XML files for configuration). For more informtion
'refer to: 
'http://arcmap2sld.geoinform.fh-mainz.de/ArcMap2SLDConverter_Eng.htm.
'LICENSE:
'This program is free software under the license of the GNU Lesser General Public License (LGPL) As published by the Free Software Foundation. 
'With the use and further development of this code you accept the terms of LGPL. For questions of the License refer to:
'http://www.gnu.org/licenses/lgpl.html
'DISCLAIMER:
'THE USE OF THE SOFTWARE ArcGIS-map to SLD Converter HAPPENS AT OWN RISK.
'I CANNOT ISSUE A GUARANTEE FOR ANY DISADVANTAGES (INCLUDING LOSS OF DATA; ETC.) THAT
'MAY ARISE FROM USING THIS SOFTWARE.
'DESCRIPTION: 
'The class is made to validate the resulting SLD. It needs the OGC-schemas: epr.xsd, filter.xsd, geometry.xsd
'StyledLayerDescriptor.xsd and XLinks.xsd. The resulting messages are displayed in "Validation_Message".
'CHANGES:
'04.02.2006: Language-customizing english/german
'####################################################################################################################

Imports System
Imports System.IO
Imports System.Xml
Imports System.Xml.Schema

Public Class ValidateSLD

    Private m_cResultSLD As String      'Der Dateipfad zur SLD  
    Private m_cSchemaXSD As String      'Der Dateipfad zur Schemadatei
    Private frmMotherform As Motherform 'Instanz des Mutterformulars
    Private m_frmValMess As Validation_Message  'Die Instanz des Meldungsformulars  
    Private m_cValidatMessage As String 'Der Text der Fehlermeldung
    Private m_cAltValidatMessage As String 'Alternativ Der Text der Fehlermeldung im Kontext der SLD-Struktur
    Private m_bValidatMessage As Boolean    'Der Flag wird gesetzt, wenn ein Formular erzeugt wurde
    Private m_bValid As Boolean 'Der Flag, ob die SLD gltig ist
    Private m_bFirstError As Boolean    'Damit sicher ist, dass die Kopfzeile der Meldung nur 1x geschrieben wird
    'Die XML-Informationen ber das SLD-Dokument:
    Private m_iLinenumber As Integer
    Private m_cNodeType As String
    Private m_cNodeName As String
    Private m_cNodePosition As String


    '##################################################################################################
    '######################################### FUNKTIONEN #############################################
    '##################################################################################################


    '************************************************************************************************
    '**********************************Die Steuerfunktion********************************************
    '************************************************************************************************
    Private Function CentralProcessing() As Boolean
        If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
            frmMotherform.CHLabelTop("berprfung der SLD-Datei")
            frmMotherform.CHLabelBottom("berprfung luft")
            frmMotherform.CHLabelSmall("berprfung kann ein paar Minuten dauern")
        ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
            frmMotherform.CHLabelTop("Checking the SLD-file")
            frmMotherform.CHLabelBottom("examination running")
            frmMotherform.CHLabelSmall("examination can take some time")
        End If
        frmMotherform.ShowWorld()
        Dim args As String() = {m_cResultSLD, m_cSchemaXSD}
        Run(args)
        frmMotherform.HideWorld()
        'Meldung wird nur angezeigt, wenn berhaupt ein berprfungsereignis ausgelst wurde
        If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
           frmMotherform.CHLabelBottom("berprfung abgeschlossen")
        ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
            frmMotherform.CHLabelBottom("examination terminated")
        End If

        If m_bValid = False Then
            CreateValidationMessage()
            If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
                frmMotherform.CHLabelSmall("Die SLD ist ungltig")
            ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
                frmMotherform.CHLabelSmall("The SLD isn't valid")
            End If

        Else
            If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
                frmMotherform.CHLabelSmall("Die SLD ist gltig")
            ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
                frmMotherform.CHLabelSmall("The SLD is valid")
            End If

        End If
    End Function


    '##################################################################################################
    '########################################### ROUTINEN #############################################
    '##################################################################################################


    Public Sub New(ByVal Mother As Motherform)
        frmMotherform = Mother
        m_cResultSLD = Mother.GetSLDFilename
        m_cSchemaXSD = Mother.GetXSDFilename
        m_bValidatMessage = False
        m_bValid = True
        m_bFirstError = False
        CentralProcessing()
    End Sub


    '************************************************************************************************
    '********************************Die Gltigkeitsberprfung**************************************
    '************************************************************************************************
    Public Sub Run(ByVal args As String())
        Dim reader As XmlValidatingReader
        Dim xsc As XmlSchemaCollection = New XmlSchemaCollection

        Try
            xsc.Add(Nothing, New XmlTextReader(args(1)))
            reader = New XmlValidatingReader(New XmlTextReader(args(0)))
            Dim lineInfo As IXmlLineInfo = CType(reader, IXmlLineInfo)  'Objekt fr Zeilennummern, etc.
            ' Set the validation event handler
            Dim valdel As ValidationEventHandler = New ValidationEventHandler(AddressOf ValidationEvent)

            AddHandler reader.ValidationEventHandler, valdel
            reader.Schemas.Add(xsc)
            reader.ValidationType = ValidationType.Schema

            While (reader.Read())
                Dim i As Integer
                'For i = 0 To (reader.Depth - 1)
                '    Console.Write(Chr(9))
                'Next
                'Console.WriteLine("{0}<{1}>{2}", reader.NodeType, reader.Name, reader.Value)

                If (reader.NodeType = XmlNodeType.Element) Then
                    While (reader.MoveToNextAttribute())
                        m_iLinenumber = lineInfo.LineNumber.ToString
                        m_cNodePosition = lineInfo.LinePosition.ToString
                        m_cNodeType = reader.NodeType.ToString
                        m_cNodeName = reader.Name.ToString
                        'For i = 0 To (reader.Depth - 1)
                        '    Console.Write(Chr(9))
                        'Next
                        'Console.WriteLine("{0}<{1}>{2}", reader.NodeType, reader.Name, reader.Value)
                    End While
                End If
            End While

        Catch e As XmlSchemaException
            If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
                m_cValidatMessage = m_cValidatMessage & "XML-SCHEMA AUSNAHME!" & vbCrLf & vbCrLf
                m_cValidatMessage = m_cValidatMessage & "Die Schemadatei, die Sie geladen haben ist ungltig. " & vbCrLf & "Mglicherweise haben Sie die Namespace-URL, die Sie in der folgenden Fehlermeldung wiederfinden nicht richtig eingebunden: " & vbCrLf & e.Message & vbCrLf & "Ausgelster Fehler in Zeile Nr.: " & e.LinePosition & vbCrLf
                m_cValidatMessage = m_cValidatMessage & vbCrLf & vbCrLf
                m_bValid = False
            ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
                m_cValidatMessage = m_cValidatMessage & "XML-SCHEMA EXCEPTION!" & vbCrLf & vbCrLf
                m_cValidatMessage = m_cValidatMessage & "The schema-file you've loaded seems to be unvalid. " & vbCrLf & "Perhaps you haven't included the namespace-URL well, you'll find in the following errormessage: " & vbCrLf & e.Message & vbCrLf & "Causing error in line Nr.: " & e.LinePosition & vbCrLf
                m_cValidatMessage = m_cValidatMessage & vbCrLf & vbCrLf
                m_bValid = False
            End If
        Catch e As XmlException
            If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
                m_cValidatMessage = m_cValidatMessage & "XML-SYNTAX AUSNAHME!" & vbCrLf & vbCrLf
                m_cValidatMessage = m_cValidatMessage & "Die Datei enthlt einen XML-Syntaxfehler. " & vbCrLf & e.Message
                m_cValidatMessage = m_cValidatMessage & vbCrLf & vbCrLf
                m_bValid = False
            ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
                m_cValidatMessage = m_cValidatMessage & "XML-SYNTAX EXCEPTION!" & vbCrLf & vbCrLf
                m_cValidatMessage = m_cValidatMessage & "the file comprises an XML-Syntax error. " & vbCrLf & e.Message
                m_cValidatMessage = m_cValidatMessage & vbCrLf & vbCrLf
                m_bValid = False
            End If
        Catch e As Exception
            If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
                m_cValidatMessage = m_cValidatMessage & "SCHWERE AUSNAHME!" & vbCrLf & vbCrLf
                m_cValidatMessage = m_cValidatMessage & "Unbekannter Fehler beim Gltigkeitstest der SLD" & vbCrLf & e.Message
                m_cValidatMessage = m_cValidatMessage & e.StackTrace & vbCrLf
                m_cValidatMessage = m_cValidatMessage & vbCrLf & vbCrLf
                m_bValid = False
            ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
                m_cValidatMessage = m_cValidatMessage & "SERIOUS EXCEPTION!" & vbCrLf & vbCrLf
                m_cValidatMessage = m_cValidatMessage & "Unknown error while validating SLD" & vbCrLf & e.Message
                m_cValidatMessage = m_cValidatMessage & e.StackTrace & vbCrLf
                m_cValidatMessage = m_cValidatMessage & vbCrLf & vbCrLf
                m_bValid = False
            End If
        Finally
            If (Not reader Is Nothing) Then
                reader.Close()
            End If
        End Try
    End Sub


    '************************************************************************************************
    '***********************Der Eventhandler fr die Gltigkeitsberprfung**************************
    '************************************************************************************************
    Private Sub ValidationEvent(ByVal errorid As Object, ByVal args As ValidationEventArgs)
        m_bValid = False
        'Schreibt 1x den Messageheader
        If m_bFirstError = False Then
            WriteMessageHeader()
        End If

        m_cValidatMessage = m_cValidatMessage & "Validierungs-Meldung: " & vbCrLf & args.Message & vbCrLf

        If (args.Severity = XmlSeverityType.Warning) Then
            If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
                m_cValidatMessage = m_cValidatMessage & "Es wurde kein Schema gefunden, um die Gltigkeitsprfung zu erzwingen." & vbCrLf & vbCrLf
            ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
                m_cValidatMessage = m_cValidatMessage & "There was no schema-file found to enforce the validation." & vbCrLf & vbCrLf
            End If
        ElseIf (args.Severity = XmlSeverityType.Error) Then
            If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
                m_cValidatMessage = m_cValidatMessage & "Whrend der Instanzberprfung wurde ein ungltiges " & m_cNodeType & " in der SLD entdeckt: " & vbCrLf
            ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
                m_cValidatMessage = m_cValidatMessage & "During the examination was an unvalid " & m_cNodeType & " found in the SLD: " & vbCrLf
            End If
        End If

        If Not (args.Exception Is Nothing) Then ' XSD schema validation error
            If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
                m_cValidatMessage = m_cValidatMessage & "Das " & m_cNodeType & " '" & m_cNodeName & "' " & " in Zeile: " & m_iLinenumber & " an der Position " & m_cNodePosition & vbCrLf
            ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
                m_cValidatMessage = m_cValidatMessage & "The " & m_cNodeType & " '" & m_cNodeName & "' " & " at line: " & m_iLinenumber & " , Position: " & m_cNodePosition & vbCrLf
            End If
        End If
        m_cValidatMessage = m_cValidatMessage & "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~" & vbCrLf
        'if (myXmlValidatingReader.Reader.LineNumber > 0)
        '    Console.WriteLine("Line: " & myXmlValidatingReader.Reader.LineNumber & " Position: " & myXmlValidatingReader.Reader.LinePosition)
        'end if
    End Sub

    '************************************************************************************************
    '*************************Die Routine schreibt den Header der Textmeldung************************
    '************************************************************************************************
    Private Sub WriteMessageHeader()
        m_bFirstError = True
        If frmMotherform.m_enumLang = Motherform.Language.Deutsch Then
            m_cValidatMessage = m_cValidatMessage & "Bei der berprfung der Gltigkeit des SLD-Dokuments mit der Schemadatei " & vbCrLf
            m_cValidatMessage = m_cValidatMessage & m_cSchemaXSD
            m_cValidatMessage = m_cValidatMessage & " wurden ungltige Inhalte entdeckt. Vielleicht haben Sie die SLD gegen ein " & vbCrLf
            m_cValidatMessage = m_cValidatMessage & "veraltetes Schema validiert. In diesem Fall knnen Sie ein neueres Schema " & vbCrLf
            m_cValidatMessage = m_cValidatMessage & "einbinden und erhalten evtl. die Besttigung fr die Korrektheit ihrer Datei." & vbCrLf
            m_cValidatMessage = m_cValidatMessage & vbCrLf & vbCrLf
        ElseIf frmMotherform.m_enumLang = Motherform.Language.English Then
            m_cValidatMessage = m_cValidatMessage & "Unvalid contents were found during examination on validity of the SLD with the schema-file  " & vbCrLf
            m_cValidatMessage = m_cValidatMessage & m_cSchemaXSD
            m_cValidatMessage = m_cValidatMessage & " . Maybe you have checked the SLD against a deprecated schema. " & vbCrLf
            m_cValidatMessage = m_cValidatMessage & "In that case you may take a newer schema and then perhaps get an " & vbCrLf
            m_cValidatMessage = m_cValidatMessage & "affirmation of the validity of your SLD-file." & vbCrLf
            m_cValidatMessage = m_cValidatMessage & vbCrLf & vbCrLf
        End If
    End Sub

    '************************************************************************************************
    '************Das Anzeigeformular wird gemacht. Ihm wird die Fehlermeldung bergeben**************
    '************************************************************************************************
    Private Sub CreateValidationMessage()
        m_bValidatMessage = True
        m_bValid = False
        m_frmValMess = New Validation_Message(m_cValidatMessage, frmMotherform)
        m_frmValMess.Visible = True
        m_frmValMess.Show()
        m_frmValMess.Activate()
    End Sub

End Class
