'####################################################################################################################
'*******************ArcGIS_SLD_Converter*****************************************************************************
'*******************Class: XMLHandle*********************************************************************************
'*******************AUTHOR: Albrecht Weiser, University of applied Sciences in Mainz, Germany 2005*******************
'The Application was part of my Diploma thesis:**********************************************************************
'"Transforming map-properties of maps in esri-data to an OGC-conformous SLD-document, for publishing the ArcGIS-map * 
'with an OGC- conformous map-server"*********************************************************************************
'ABSTRACT:
'The program so called "ArcGIS-map to SLD Converter" analyses an
'ArcMap-Project with respect to its symbolisation and assembles an SLD
'for the OGC-Web Map Service (WMS) from the gathered data. The program
'is started parallel to a running ArcMap 9.X-session. Subsequently the
'application deposits an SLD-file which complies the symbolisation of
'the available ArcMap-project. With the SLD a WMS-project may be
'classified and styled according to the preceding ArcMap-project. The
'application is written in VisualBasic.NET and uses the .NET 2.0
'Framework (plus XML files for configuration). For more informtion
'refer to: 
'http://arcmap2sld.geoinform.fh-mainz.de/ArcMap2SLDConverter_Eng.htm.
'LICENSE:
'This program is free software under the license of the GNU Lesser General Public License (LGPL) As published by the Free Software Foundation. 
'With the use and further development of this code you accept the terms of LGPL. For questions of the License refer to:
'http://www.gnu.org/licenses/lgpl.html
'DISCLAIMER:
'THE USE OF THE SOFTWARE ArcGIS-map to SLD Converter HAPPENS AT OWN RISK.
'I CANNOT ISSUE A GUARANTEE FOR ANY DISADVANTAGES (INCLUDING LOSS OF DATA; ETC.) THAT
'MAY ARISE FROM USING THIS SOFTWARE.
'DESCRIPTION:
'The class mainly consists of some public functions that depend to a current xml-file (the sld). There is always a pointer
'on the last created Node. (one for the last element: "m_objActiveNode" and one for the last attribute: "m_objActiveNode")
'If a calling function wants to create a new element or attribute it calls the function CreateElement or CreateAttribute.
'The parameter for the call is a trivial name of the element that wants to be created (trivial names, the according OGC-
'Names and the relating XPath devices can be found in the file LUT_sld_mapping_file.xml). This function handoff the trivial name
' to the function "NavigateElement". NavigateElement looks in the LUT_sld_mapping_file.xml for the according XPath-expression
'and with this expression navigates back in XML-hierarchy until the expression is true. Then the current element/attribute will set
'to the new activeNode and the new element/attribute can be created.
'CHANGES:
'####################################################################################################################

Imports System
Imports System.Xml
Imports System.IO
Imports System.Collections
Imports System.Collections.Specialized
Imports System.Xml.XPath
Imports ArcGIS_SLD_Converter.Store2Fields

Public Class XMLHandle

    '##################################################################################################
    '######################################## DEKLARATIONEN ###########################################
    '##################################################################################################

#Region "Membervariablen"

    'Private m_objParent As Output_SLD               'Instanz der aufrufenden Klasse
    Private m_cXMLFilename As String                'Der ...\Pfad\Dateiname
    Private m_enDocMode As XMLState                 'Der aktuelle Status des Dokuments (geschlossen, geffnet,...)
    Private m_objDoc As XmlDocument                 'Das aktuelle Dokument
    Private m_objNameDict As StringDictionary        'Im Schlsselfeld:Tag-Aliase aus Programmcode; im Datenfeld: ogc-Name des sld-Tags
    Private m_objNamespaceDict As StringDictionary  'Im Schlsselfeld:Tag-Aliase aus Programmcode; im Datenfeld: der zum Namen gehrende Namespacekrzel
    Private m_objXPathDict As Hashtable             'Im Schlsselfeld:Tag-Aliase aus Programmcode; Datenfeld:String-Collection mit zugehrigen XPath-Ausdrcken
    Private m_objRoot As XmlElement                 'Der Wurzelknoten des xml-elements
    Private m_iLevelCount As Int16                  'Die nullbasierte Anzahl der Ebenen des XML-Dokuments
    Private m_objActiveNode As XmlNode              'Der derzeit aktive Knoten
    Private m_objActiveAttribute As XmlAttribute    'Das derzeit aktive Attribut
    Private m_cXMLVersion As String
    Private m_cSLDVersion As String
    Private m_cXMLEncoding As String
    Private m_objNamespaceURL As ArcGIS_SLD_Converter.Store2Fields        'Enthlt die Namespacelinks
    Private m_objNSManager As XmlNamespaceManager   'Enthlt die Namespaces und URL's


#End Region


    '##################################################################################################
    '######################################## ENUMERATIONEN ###########################################
    '##################################################################################################

#Region "Enums"

    'Der Status des XML-Handle wird hier festgelegt (geffnetes Dokument, geschlossen,....)
    Private Enum XMLState
        xmlDocClosed = 0
        xmlDocOpen = 1
    End Enum

#End Region


    '##################################################################################################
    '#################################### DATENSTRUKTUREN #############################################
    '##################################################################################################

#Region "Datenstrukturen"



#End Region


    '##################################################################################################
    '########################################### ROUTINEN #############################################
    '##################################################################################################

#Region "Routinen"

    ''fr schreibenden Zugriff
    'Public Sub New(ByVal Parent As Output_SLD)
    '    m_objParent = Parent
    '    start()
    'End Sub

    ''Fr lesenden Zugriff
    'Public Sub New(ByVal FileName As String, ByVal Parent As Output_SLD)
    '    'm_objParent = Parent
    '    m_cXMLFilename = FileName
    '    start()
    'End Sub

    Public Sub New(ByVal FileName As String)
        'm_objParent = Parent
        m_cXMLFilename = FileName
        start()
    End Sub

    Public Sub start()
        ReadLUT()
        m_enDocMode = XMLState.xmlDocClosed
        m_objDoc = New XmlDocument
        m_iLevelCount = 0
        If Not m_cXMLFilename = "" Then
            OpenDoc()
        End If

    End Sub

    '************************************************************************************************
    'READ ACCESS
    'Die Routine ffnet ein vorhandenes XML-Dokument mit dem bergebenen Filename und setzt den 
    'XML-Status auf geffnet. Der Zeiger sitzt auf dem Root-Element
    '************************************************************************************************
    Public Sub OpenDoc()
        Try
            If m_enDocMode = XMLState.xmlDocClosed Then
                If File.Exists(m_cXMLFilename) = True Then
                    If Not m_cXMLFilename = "" Then
                        m_objDoc = New XmlDocument
                        m_objDoc.Load(m_cXMLFilename)
                        m_objRoot = m_objDoc.DocumentElement
                        If m_objRoot Is Nothing Then
                            Throw New Exception("Das Dokument ist leer. Bitte whlen Sie eine existierende, gltige XML-Datei aus")
                        End If
                        m_enDocMode = XMLState.xmlDocOpen
                    Else
                        Throw New Exception("Es wurde noch kein Dateiname/Speicherort fr das XML-Dokument angegeben")
                    End If
                End If
            Else
                MessageBox.Show("Das Dokument ist schon geffnet!")
                Exit Sub
            End If
        Catch ex As Exception
            ErrorMsg("Konnte XML-Dokument nicht ffnen", ex.Message, ex.StackTrace, "OpenDoc")
        End Try
    End Sub

#End Region


    '##################################################################################################
    '######################################### PROPERTIES #############################################
    '##################################################################################################

#Region "Properties"

    Public Property XMLFilename() As String
        Get
            Return m_cXMLFilename
        End Get
        Set(ByVal Value As String)
            m_cXMLFilename = Value
        End Set
    End Property

    Public ReadOnly Property GetRoot() As XmlElement
        Get
            m_iLevelCount = 0
            Return m_objRoot
        End Get
    End Property

    Public ReadOnly Property LevelNumber() As Int16
        Get
            Return m_iLevelCount
        End Get
    End Property

#End Region


    '##################################################################################################
    '######################################### FUNKTIONEN #############################################
    '##################################################################################################

#Region "Read-Write-Functions"

    '************************************************************************************************
    'READ-ACCESS XPath/DOM
    'Hier sollte nach Mglichkeit nichts verndert werden !!!!!
    'Die Zentrale Navigationsfunktion. Das Prinzip: der gerade aktive Knoten wird mit einem eingelesenen
    'XPath-Ausdruck verglichen. Es wird solange in der Hierarchie nach hinten (zum Elternelement) navigiert,
    'bis der Ausdruck stimmt.Ist das der Fall, ist die richtige Ebene erreich,um das neue Element anzulegen
    'Parameter: AliasTagName=   der programminterne Aliasname (zum Vergleich des ogc-Tagnamens aus 
    '                           der LUT)
    '************************************************************************************************
    Public Function NavigateElement(ByVal AliasTagName As String) As Boolean
        Dim objNav As XPathNavigator     'Das Navigatorobjekt, mit dem im Dokument navigiert wird
        Dim objXPathColl As StringCollection     'Die XPath-Ausdrcke, die den ogc-Tag reprsentieren welcher durch den AliasTagName benannt wird
        Dim objNodelist As XmlNodeList
        Dim objEvalNode, objTempNode, objTempNode2 As XmlNode
        Dim i, j, iInsurance As Int16
        iInsurance = 0
        i = 0
        Dim bSwitch As Boolean  'Der Flag steuert die do while und wird dann true, wenn der knoten, der mit dem XPath-Ausdruck bereinstimmt gefunden wurde
        bSwitch = False

        Try
            If m_objXPathDict.ContainsKey(AliasTagName) Then
                objXPathColl = m_objXPathDict(AliasTagName)
                objTempNode = m_objActiveNode   'Aktiver Knoten
                objNav = objTempNode.CreateNavigator()
                Do Until bSwitch = True
                    'Wenn mehr als 1 Knoten auf der Ebene gibt, mssen alle Knoten getestet werden
                    If objTempNode.ParentNode.ChildNodes.Count > 1 Then
                        objTempNode2 = objTempNode
                        'Die Do-until Schleife testet alle Geschwisterknoten des gerade aktiven Knotens
                        Do Until objTempNode Is Nothing
                            For i = 0 To objXPathColl.Count - 1   'Luft solange es XPathausdrcke zum auswerten gibt
                                objNodelist = m_objActiveNode.SelectNodes(objXPathColl(i), m_objNSManager) 'alle in Frage kommenden Knoten
                                objEvalNode = objNodelist.ItemOf(objNodelist.Count - 1) 'Es muss immer zum letzten Knoten gegangen werden, da die Datei seriell geschrieben wird
                                'Wenn die Bedingung erfllt ist, wird der objTempNode (der ja der richtige Knoten ist) zum aktiven Knoten
                                If (objEvalNode Is objTempNode) = True Then
                                    m_objActiveNode = objTempNode
                                    bSwitch = True
                                    Return True
                                End If
                            Next
                            objTempNode = objTempNode.PreviousSibling
                        Loop
                        objTempNode = objTempNode2
                    Else    'Wenn es nur einen Knoten auf der Ebene gibt
                        For i = 0 To objXPathColl.Count - 1   'Luft solange es XPathausdrcke zum auswerten gibt
                            objNodelist = m_objActiveNode.SelectNodes(objXPathColl(i), m_objNSManager) 'alle in Frage kommenden Knoten
                            objEvalNode = objNodelist.ItemOf(objNodelist.Count - 1) 'Es muss immer zum letzten Knoten gegangen werden, da die Datei seriell geschrieben wird
                            'Wenn die Bedingung erfllt ist, wird der objTempNode (der ja der richtige Knoten ist) zum aktiven Knoten
                            If (objEvalNode Is objTempNode) = True Then
                                m_objActiveNode = objTempNode
                                bSwitch = True
                                Return True
                            End If
                        Next
                    End If

                    'Wenn alle XPathausdrcke nicht zutrafen, wird zum Elternknoten navigiert
                    If Not (objNav.Matches("/")) Then   'Hier die Navigation rckwrts im Tree 
                        objNav.MoveToParent()
                        objTempNode = objTempNode.ParentNode
                    End If
                    'Dient lediglich zur Sicherheit, da die do while ewig laufen kann, wenn Bedingung nicht erfllt ist
                    iInsurance = iInsurance + 1
                    If iInsurance > 100 Then    'Keiner nimmt 100 passende XPath-Ausdrcke in die LUT auf!
                        Throw New Exception("Kein gltiger XPathausdruck fr '" & AliasTagName & "' gefunden - Sicherheitsabbruch")
                    End If
                Loop
            Else
                Throw New Exception("Die Datei LUT_sld_mapping_file.xml enthlt den Tag-Alias '" & AliasTagName & "' nicht.")
            End If
            Return True
        Catch ex As Exception
            ErrorMsg("Der XPath-Ausdruck '" & AliasTagName & "' in der LUT-Datei stimmt nicht; oder Navigieren nicht mglich.", ex.Message, ex.StackTrace, "NavigateElement")
            Return False
        End Try
    End Function



    '************************************************************************************************
    'WRITE-ACCESS DOM
    'Legt einen neuen Kindknoten fr den gerade aktiven Knoten an
    'Parameter: TagName =   der Name des zu schreibenden Knotens (TagName ist nur Vergleichswert. 
    '                       Wahrer Wert wird aus der LUT genommen)
    '************************************************************************************************
    Public Function CreateElement(ByVal AliasTagName As String) As Boolean
        Dim objNode As XmlElement
        Dim cNamespacePrefix, cNamespaceURL As String
        Try
            If NavigateElement(AliasTagName) = True Then
                cNamespacePrefix = GetNamespacePrefix(AliasTagName)
                cNamespaceURL = GetNamespaceURL(cNamespacePrefix)
                objNode = m_objDoc.CreateElement(cNamespacePrefix, GetOGCName(AliasTagName), cNamespaceURL)
                m_objActiveNode.AppendChild(objNode)
                m_objActiveNode = objNode   'Der gerade gemachte Knoten wird aktiv gesetzt
                Return True
            End If
        Catch ex As Exception
            ErrorMsg("Konnte Elementknoten nicht erstellen.", ex.Message, ex.StackTrace, "CreateElement")
            Return False
        End Try
    End Function


    '************************************************************************************************
    'WRITE-ACCESS DOM
    'Fgt Text in den gerade aktiven Knoten ein
    'Parameter: InnerText =   der Text, der in dem Element stehen soll
    '************************************************************************************************
    Public Function SetElementText(ByVal InnerText As String) As Boolean
        Try
            m_objActiveNode.InnerText = InnerText
            Return True
        Catch ex As Exception
            ErrorMsg("Konnte Elementtext nicht schreiben.", ex.Message, ex.StackTrace, "SetElementText")
            Return False
        End Try
    End Function


    '************************************************************************************************
    'WRITE-ACCESS DOM
    'Erstellt ein neues Attribut in dem aktiven Knoten
    'Parameter: AttributeName = das Attribut, das an den aktiven Knoten angehngt werden soll
    '************************************************************************************************
    Public Function CreateAttribute(ByVal AttributeName As String) As Boolean
        Dim objXmlAttribute As XmlAttribute
        Try
            objXmlAttribute = m_objDoc.CreateAttribute(AttributeName)
            m_objActiveNode.Attributes.Append(objXmlAttribute)
            m_objActiveAttribute = objXmlAttribute  'Das gerade gemachte Attribut wird aktiv gesetzt
            Return True
        Catch ex As Exception
            ErrorMsg("Konnte Attribut nicht erstellen", ex.Message, ex.StackTrace, "CreateAttribute")
            Return False
        End Try
    End Function


    '************************************************************************************************
    'WRITE-ACCESS DOM
    'Erstellt ein neues Attribut in dem aktiven Knoten
    'AttributeValue: AttributeValue = der Attributwert (Text) des aktiven Attributs
    '************************************************************************************************
    Public Function SetAttributeValue(ByVal AttributeValue As String) As Boolean
        Try
            m_objActiveAttribute.Value = AttributeValue
            Return True
        Catch ex As Exception
            ErrorMsg("Konnte Attribut nicht erstellen", ex.Message, ex.StackTrace, "SetAttributeValue")
            Return False
        End Try
    End Function


    '************************************************************************************************
    'READ-ACCESS DOM-basierte Parsing-Funktion
    'Parst durch das geffnete XML-Dokument-ab dem Knoten,der bergeben wird. Rekursiver Aufruf!!
    'Parameter:     CurrentNode - der Knoten, ab dem die Suche begonnen werden soll
    '************************************************************************************************
    Public Function ParseDoc(ByVal CurrentNode As XmlElement) As Boolean
        Dim objNode As XmlElement   'Der jeweilige Kindknoten
        Static Dim iLevelCount As Int16 'Durch die Deklaration als static vergisst die Variable ihren Wert nicht bei Verlassen der Funktion
        iLevelCount = iLevelCount + 1   'Anzahl der Levels im XML-Dok ab dem CurrentNode

        Try
            If m_enDocMode = XMLState.xmlDocOpen Then
                If CurrentNode.HasChildNodes Then
                    objNode = CurrentNode.FirstChild
                    Do Until objNode Is Nothing
                        If objNode.HasChildNodes Then
                            If TypeOf objNode.FirstChild Is XmlElement Then 'Auch InnerText oder Attribute ist ein ChildNode
                                ParseDoc(objNode)  'Rekursion
                            End If
                        End If
                        objNode = objNode.NextSibling
                    Loop
                End If
            Else
                MessageBox.Show("Das Dokument ist noch nicht geffnet")
                Exit Function
            End If
            m_iLevelCount = iLevelCount
            iLevelCount = 0
            Return True
        Catch ex As Exception
            ErrorMsg("Das Dokument ist unbrauchbar", ex.Message, ex.StackTrace, "ParseDoc")
        End Try
    End Function




    '********************************************************************************************
    'WRITE-ACCESS
    ' Erstellen einer neuen XML-Datei
    ' ---------------------------------------------------------------
    ' Voraussetzung : Der Dateiname muss bekannt sein (Property "XMLfilename")
    ' Parameter :   OverWrite = Soll bestehende Datei berschrieben werden (Ja / Nein)
    ' Rckgabewerte: 
    '       true = Ausfhrung korrekt; Datei wurde erstellt
    '       false = Bei der Ausfhrung trat ein Fehler auf
    ' Zeiger sitzt auf Root-Knoten
    '********************************************************************************************
    Public Function CreateNewFile(ByVal OverWrite As Boolean) As Boolean
        Dim objDeclare As XmlDeclaration
        Dim i, j As Int16
        Dim cNamePre As String

        Try
            If File.Exists(m_cXMLFilename) = True Then
                If OverWrite = False Then
                    Throw New Exception("Fehler beim Anlegen der XML-Datei (Datei besteht bereits)")
                    Return False
                End If
                File.Delete(m_cXMLFilename)
            End If
            cNamePre = GetNamespacePrefix("StyledLayerDescriptor")
            m_objDoc = New XmlDocument
            m_objRoot = m_objDoc.CreateElement(cNamePre, GetOGCName("StyledLayerDescriptor"), GetNamespaceURL(cNamePre))
            m_objDoc.AppendChild(m_objRoot)
            m_objActiveNode = m_objRoot

            CreateAttribute("version")
            SetAttributeValue(m_cSLDVersion)
            'Hier werden die Namespaces geschrieben
            For i = 0 To m_objNamespaceURL.Count - 1
                CreateAttribute("xmlns" & ":" & m_objNamespaceURL.GetString1ByIndex(i))
                SetAttributeValue(m_objNamespaceURL.GetString2ByIndex(i))
            Next


            objDeclare = m_objDoc.CreateXmlDeclaration(m_cXMLVersion, m_cXMLEncoding, "yes") 'Version muss z.Zt. 1.0 sein!
            m_objDoc.InsertBefore(objDeclare, m_objRoot)
            SaveDoc()   'Speichert das aktuelle Dokument mit dem aktuellen Dateinamen
            m_enDocMode = XMLState.xmlDocOpen
            'Der Namespacemanager, der ntig ist, fr die XPath-Navigation mit Namespaceprfixen
            m_objNSManager = New XmlNamespaceManager(m_objDoc.NameTable)
            For j = 0 To m_objNamespaceURL.Count - 1
                m_objNSManager.AddNamespace(m_objNamespaceURL.GetString1ByIndex(j), m_objNamespaceURL.GetString2ByIndex(j))
            Next
            Return True
        Catch ex As Exception
            ErrorMsg("Fehler beim Anlegen der XML-Datei (" & ex.Message.ToString & ")", ex.Message, ex.StackTrace, "CreateNewFile")
            Return Nothing
        End Try
    End Function

    ''************************************************************************************************
    ''Legt Eigenschaften fr den Root-Knoten fest
    ''************************************************************************************************
    'Public Function SetRootAttribute(ByVal AttributeName As String, ByVal AttributeValue As String) As Boolean
    '    Dim objRootAttrib As XmlAttribute
    '    Try
    '        If m_enDocMode = XMLState.xmlDocOpen Then
    '            objRootAttrib = m_objDoc.CreateAttribute(AttributeName)
    '            m_objRoot.Attributes.Append(objRootAttrib)
    '            objRootAttrib.Value = AttributeValue
    '        End If
    '    Catch ex As Exception
    '        ErrorMsg("Fehler beim Anlegen des RootAttributes", ex.Message, ex.StackTrace, SetRootAttribute)
    '    End Try
    'End Function

#End Region


#Region "Hilfsfunktionen"

    '******************************************************************************************************
    'Read-Access DOM
    'Liest die LUT-Datei fr die SLD-Tagumwandlungen ein und speichert die Elemente in der StringDictionary
    '******************************************************************************************************
    Private Function ReadLUT() As Boolean
        Dim cFilename As String
        Dim objLUTDoc As XmlDocument      'Das doc der LUT-XML fr die sld-Tags 
        Dim objRoot, objNode, objNode2, objNode3 As XmlElement
        Dim objXPathExp As StringCollection  'Die Sammlung der Xpath-Ausdrcke 
        m_objNameDict = New StringDictionary
        m_objNamespaceDict = New StringDictionary
        m_objXPathDict = New Hashtable
        m_objNamespaceURL = New ArcGIS_SLD_Converter.Store2Fields

        Try
            cFilename = AppDomain.CurrentDomain.BaseDirectory & "LUT_sld_mapping_file.xml"
            If File.Exists(cFilename) Then
                objLUTDoc = New XmlDocument
                objLUTDoc.Load(cFilename)
                objRoot = objLUTDoc.DocumentElement
                objNode = objRoot.FirstChild
                Do Until objNode Is Nothing
                    'Die Unterelemente des Knotens sldSyntax werden ausgelesen und abgespeichert
                    If objNode.Name = "sldSyntax" Then
                        objNode2 = objNode.FirstChild
                        Do Until objNode2 Is Nothing
                            m_objNameDict.Add(objNode2.Name, objNode2.GetAttribute("ogcTag"))
                            m_objNamespaceDict.Add(objNode2.Name, objNode2.GetAttribute("Namespace"))
                            objXPathExp = New StringCollection
                            objNode3 = objNode2.FirstChild
                            'Die Kindelemente mit den XPath-Ausdrcken
                            Do Until objNode3 Is Nothing
                                objXPathExp.Add(objNode3.InnerText)
                                objNode3 = objNode3.NextSibling
                            Loop
                            m_objXPathDict.Add(objNode2.Name, objXPathExp)
                            objNode2 = objNode2.NextSibling
                        Loop
                        'Die Unterelemente des Knotens "sldConfiguration" werden ausgelesen und abgespeichert
                    ElseIf objNode.Name = "sldConfiguration" Then
                        objNode2 = objNode.FirstChild
                        Do Until objNode2 Is Nothing
                            Select Case objNode2.Name
                                Case "xmlVersion"
                                    m_cXMLVersion = objNode2.InnerText
                                Case "xmlEncoding"
                                    m_cXMLEncoding = objNode2.InnerText
                                Case "Namespaces"
                                    objNode3 = objNode2.FirstChild
                                    Do Until objNode3 Is Nothing
                                        m_objNamespaceURL.Add2Strings(objNode3.Name, objNode3.InnerText)
                                        objNode3 = objNode3.NextSibling
                                    Loop
                                Case "sldVersion"
                                    m_cSLDVersion = objNode2.InnerText
                            End Select
                            objNode2 = objNode2.NextSibling
                        Loop
                    End If
                    objNode = objNode.NextSibling
                Loop
            Else
                Throw New FileNotFoundException
                Return False
            End If
            Return True
        Catch ex As FileNotFoundException
            ErrorMsg("Die Datei LUT_sld_mapping_file.xml muss im Anwendungsverzeichnis stehen. Bitte kopieren SIe die Datei an diese Stelle und starten Sie die Anwendung erneut.", ex.Message, ex.StackTrace, "ReadLUT")
        Catch ex As Exception
            ErrorMsg("Fehler beim ffnen der Konfigurationsdatei", ex.Message, ex.StackTrace, "ReadLUT")
        End Try
    End Function


    '******************************************************************************************************
    'Gibt den im String-Dictionary eingelesenen Tagnamen zurck nach einem Vergleich mit dem Trivialnamen
    'PARAMETER:     Value = AliasTagName
    '******************************************************************************************************
    Private Function GetOGCName(ByVal Value As String) As String
        Dim cRightTag As String
        Try
            If m_objNameDict.ContainsKey(Value) = True Then
                cRightTag = m_objNameDict(Value)
            Else
                Throw New Exception("Der Tag " & Value & " ist noch nicht in die LUT-Datei aufgenommen")
                Return "ERROR:" & Value
            End If
            Return cRightTag
        Catch ex As Exception
            ErrorMsg("Fehler beim Beziehen der OGC-Tagnames", ex.Message, ex.StackTrace, "GetOGCName")
        End Try
    End Function


    '******************************************************************************************************
    'Gibt den im String-Dictionary eingelesenen Namespacekrzel zurck nach einem Vergleich mit dem Trivialnamen
    'PARAMETER:     Value = AliasTagName
    '******************************************************************************************************
    Private Function GetNamespacePrefix(ByVal Value As String) As String
        Dim cRightTag As String
        Try
            If m_objNamespaceDict.ContainsKey(Value) = True Then
                cRightTag = m_objNamespaceDict(Value)
            Else
                Throw New Exception("Leider ist dieser Namensraumkrzel " & Value & " noch nicht in die LUT aufgenommen")
                Return "ERROR:" & Value
            End If
            Return cRightTag
        Catch ex As Exception
            ErrorMsg("Fehler beim Beziehen der Namensraumkrzel", ex.Message, ex.StackTrace, "GetNamespacePrefix")
        End Try

    End Function


    '******************************************************************************************************
    'Gibt den im String-Dictionary eingelesenen NamespaceURL zurck nach einem Vergleich mit dem Namespacekrzel
    'PARAMETER:     Value = Namespacekrzel
    '******************************************************************************************************
    Private Function GetNamespaceURL(ByVal Value) As String
        Dim cRightTag As String

        Try
            If m_objNamespaceURL.ContainsString1(Value) = True Then
                cRightTag = m_objNamespaceURL.GetString2ForString1(Value)
            Else
                Throw New Exception("Leider ist dieser URL " & Value & " noch nicht in die LUT aufgenommen")
                Return "ERROR:" & Value
            End If
            Return cRightTag
        Catch ex As Exception
            ErrorMsg("Fehler beim Beziehen der Namensraum-URL", ex.Message, ex.StackTrace, "GetNamespaceURL")
        End Try
    End Function


    '************************************************************************************************
    '**********************************Die zentrale Fehlermeldung************************************
    '************************************************************************************************
    Private Function ErrorMsg(ByVal message As String, ByVal exMessage As String, ByVal stack As String, ByVal functionname As String)
        MessageBox.Show(message & "." & vbCrLf & exMessage & vbCrLf & stack, "ArcGIS_SLD_Converter | XMLHandle | " & functionname, MessageBoxButtons.OK, MessageBoxIcon.Error)
        'WriteToFile()
        MyTermination()
    End Function


    '************************************************************************************************
    '**********************************Die zentrale Infomeldung************************************
    '************************************************************************************************
    Private Function InfoMsg(ByVal message As String, ByVal functionname As String)
        MessageBox.Show(message, "ArcGIS_SLD_Converter | XMLHandle | " & functionname, MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
    End Function


    ''************************************************************************************************
    ''*************************************gibt die letzte fehlerhafte SLD aus************************
    ''************************************************************************************************
    'Private Function WriteToFile()
    '    Dim dummy As XmlComment
    '    Try
    '        If Not m_objDoc Is Nothing Then
    '            dummy = m_objDoc.CreateComment("DIESE SLD IST DIE LETZTE FEHLERHAFTE SLD, DIE VOM PROGRAMM AUSGEGEBEN WURDE")
    '            m_objDoc.AppendChild(dummy)
    '            m_objDoc.Save(AppDomain.CurrentDomain.BaseDirectory & "Fehler_SLD.sld")
    '        End If
    '    Catch ex As Exception
    '        'MyTermination()
    '    End Try
    'End Function


    '************************************************************************************************
    '*************************************** Speicher-Funktion **************************************
    'Speichert das aktuelle Dokument mit dem aktuellen Dateinamen
    '************************************************************************************************
    Public Function SaveDoc() As Boolean
        Try
            m_objDoc.Save(m_cXMLFilename)
        Catch ex As Exception
            ErrorMsg("Fehler beim speichern der Datei", ex.Message, ex.StackTrace, "SaveDoc")
        End Try
    End Function


    '************************************************************************************************
    '*****************************     Beenden der Anwendung     ************************************
    '************************************************************************************************
    Private Function MyTermination()
        End
        'oder: application.exit
    End Function
#End Region


End Class
